# encoding: utf-8

# File:	clients/responsefile_auto.ycp
# Package:	Configuration of responsefile
# Summary:	Client for autoinstallation
# Authors:	Blair Merrell <bmerrell@novell.com>
#
# $Id: responsefile_auto.ycp 20583 2004-12-16 10:26:30Z mvidner $
#
# This is a client for autoinstallation. It takes its arguments,
# goes through the configuration and return the setting.
# Does not do any changes to the configuration.

# @param function to execute
# @param map/list of responsefile settings
# @return [Hash] edited settings, Summary or boolean on success depending on called function
# @example map mm = $[ "FAIL_DELAY" : "77" ];
# @example map ret = WFM::CallFunction ("responsefile_auto", [ "Summary", mm ]);
module Yast
  class ResponsefileAutoClient < Client
    def main
      Yast.import "UI"

      textdomain "novell-responsefile"

      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("Responsefile auto started")

      Yast.import "Responsefile"
      Yast.include self, "novell-responsefile/wizards.rb"

      @ret = nil
      @func = ""
      @param = {}

      # Check arguments
      if Ops.greater_than(Builtins.size(WFM.Args), 0) &&
          Ops.is_string?(WFM.Args(0))
        @func = Convert.to_string(WFM.Args(0))
        if Ops.greater_than(Builtins.size(WFM.Args), 1) &&
            Ops.is_map?(WFM.Args(1))
          @param = Convert.to_map(WFM.Args(1))
        end
      end
      Builtins.y2debug("func=%1", @func)
      Builtins.y2debug("param=%1", @param)

      # Create a summary
      if @func == "Summary"
        @ret = Ops.get_string(Responsefile.Summary, 0, "")
      # Reset configuration
      elsif @func == "Reset"
        Responsefile.Import({})
        @ret = {}
      # Change configuration (run AutoSequence)
      elsif @func == "Change"
        @ret = ResponsefileAutoSequence()
      # Import configuration
      elsif @func == "Import"
        @ret = Responsefile.Import(@param)
      # Return actual state
      elsif @func == "Export"
        @ret = Responsefile.Export
      # Return needed packages
      elsif @func == "Packages"
        @ret = Responsefile.AutoPackages
      # Read current state
      elsif @func == "Read"
        Yast.import "Progress"
        @progress_orig = Progress.set(false)
        @ret = Responsefile.Read
        Progress.set(@progress_orig)
      # Write given settings
      elsif @func == "Write"
        Yast.import "Progress"
        @progress_orig = Progress.set(false)
        Responsefile.write_only = true
        @ret = Responsefile.Write
        Progress.set(@progress_orig)
      else
        Builtins.y2error("Unknown function: %1", @func)
        @ret = false
      end

      Builtins.y2debug("ret=%1", @ret)
      Builtins.y2milestone("Responsefile auto finished")
      Builtins.y2milestone("----------------------------------------")

      deep_copy(@ret) 

      # EOF
    end
  end
end

Yast::ResponsefileAutoClient.new.main
