# encoding: utf-8

# File:	clients/responsefile_proposal.ycp
# Package:	Configuration of responsefile
# Summary:	Proposal function dispatcher.
# Authors:	Blair Merrell <bmerrell@novell.com>
#
# $Id: responsefile_proposal.ycp 20583 2004-12-16 10:26:30Z mvidner $
#
# Proposal function dispatcher for responsefile configuration.
# See source/installation/proposal/proposal-API.txt
module Yast
  class ResponsefileProposalClient < Client
    def main

      textdomain "novell-responsefile"

      Yast.import "Responsefile"
      Yast.import "Progress"

      # The main ()
      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("Responsefile proposal started")

      @func = Convert.to_string(WFM.Args(0))
      @param = Convert.to_map(WFM.Args(1))
      @ret = {}

      # create a textual proposal
      if @func == "MakeProposal"
        @proposal = ""
        @warning = ""
        @warning_level = nil
        @force_reset = Ops.get_boolean(@param, "force_reset", false)

        if @force_reset || !Responsefile.proposal_valid
          Responsefile.proposal_valid = true
          @progress_orig = Progress.set(false)
          Responsefile.Read
          Progress.set(@progress_orig)
        end
        @sum = Responsefile.Summary
        @proposal = Ops.get_string(@sum, 0, "")

        @ret = {
          "preformatted_proposal" => @proposal,
          "warning_level"         => @warning_level,
          "warning"               => @warning
        }
      # run the module
      elsif @func == "AskUser"
        @stored = Responsefile.Export
        @seq = Convert.to_symbol(
          WFM.CallFunction("responsefile", [path(".propose")])
        )
        Responsefile.Import(@stored) if @seq != :next
        Builtins.y2debug("stored=%1", @stored)
        Builtins.y2debug("seq=%1", @seq)
        @ret = { "workflow_sequence" => @seq }
      # create titles
      elsif @func == "Description"
        @ret = {
          # Rich text title for Responsefile in proposals
          "rich_text_title" => _(
            "Responsefile"
          ),
          # Menu title for Responsefile in proposals
          "menu_title"      => _(
            "&Responsefile"
          ),
          "id"              => "responsefile"
        }
      # write the proposal
      elsif @func == "Write"
        Responsefile.Write
      else
        Builtins.y2error("unknown function: %1", @func)
      end

      # Finish
      Builtins.y2debug("ret=%1", @ret)
      Builtins.y2milestone("Responsefile proposal finished")
      Builtins.y2milestone("----------------------------------------")
      deep_copy(@ret) 

      # EOF
    end
  end
end

Yast::ResponsefileProposalClient.new.main
