# encoding: utf-8

# File:	include/novell-responsefile/complex.ycp
# Package:	Configuration of responsefile
# Summary:	Dialogs definitions
# Authors:	Blair Merrell <bmerrell@novell.com>
#
# $Id: complex.ycp 13891 2004-02-05 15:16:57Z msvec $
module Yast
  module NovellResponsefileComplexInclude
    def initialize_novell_responsefile_complex(include_target)
      Yast.import "UI"

      textdomain "novell-responsefile"
      Yast.import "Label"
      Yast.import "Popup"
      Yast.import "Wizard"
      Yast.import "UIHelper"
      Yast.import "Report"
      Yast.import "Responsefile"

      Yast.include include_target, "novell-responsefile/helps.rb"
    end

    # Return a modification status
    # @return true if data was modified
    def Modified
      Responsefile.Modified
    end

    def ReallyAbort
      !Responsefile.Modified || Popup.ReallyAbort(true)
    end

    def PollAbort
      UI.PollInput == :abort
    end

    # Read settings dialog
    # @return `abort if aborted and `next otherwise
    def ReadDialog
      Builtins.y2milestone("Responsefile::ReadDialog() started")
      Wizard.RestoreHelp(Ops.get_string(@HELPS, "read", ""))
      # Responsefile::AbortFunction = PollAbort;
      ret = Responsefile.Read
      ret ? :next : :abort
    end

    # Write settings dialog
    # @return `abort if aborted and `next otherwise
    def WriteDialog
      Builtins.y2milestone("Responsefile::WriteDialog() started")
      Wizard.RestoreHelp(Ops.get_string(@HELPS, "write", ""))
      # Responsefile::AbortFunction = PollAbort;
      ret = Responsefile.Write
      ret ? :next : :abort
    end

    # Write settings dialog
    # @return `abort if aborted and `next otherwise
    def SelectTypeDialog
      Builtins.y2milestone("Responsefile::SelectTypeDialog() started")
      # SelectType dialog caption
      caption = _("Select Install Type")

      contents = HVCenter(
        VBox(
          Left(
            Heading(
              _(
                "Select the installation type and the optional template file location."
              )
            )
          ),
          VSpacing(1),
          Left(
            Frame(
              _("Hardware Type"),
              MarginBox(
                1.5,
                0.2,
                RadioButtonGroup(
                  Id(:hardwaretype),
                  VBox(
                    Left(RadioButton(Id(0), _("Virtual"))),
                    Left(RadioButton(Id(1), _("Physical")))
                  )
                )
              )
            )
          ),
          VSpacing(1),
          Left(
            Frame(
              _("Installation Type"),
              MarginBox(
                1.5,
                0.2,
                RadioButtonGroup(
                  Id(:installtype),
                  VBox(
                    Left(RadioButton(Id(2), _("New Server"))),
                    Left(RadioButton(Id(3), _("Server Upgrade"))),
                    Left(RadioButton(Id(5), _("Factory Install - Image"))),
                    Left(RadioButton(Id(4), _("Factory Install - Configure")))
                  )
                )
              )
            )
          ),
          VSpacing(2),
          Left(CheckBox(Id(:template), _("Use a response file template"))),
          VSpacing(0.8),
          Left(
            MinWidth(
              45,
              HBox(
                TextEntry(
                  Id(:path),
                  Opt(:hstretch),
                  _("Response file template")
                ),
                VBox(Label(""), PushButton(Id(:browse), _("Browse")))
              )
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "select", ""),
        Label.BackButton,
        Label.NextButton
      )
      if Ops.get_boolean(Responsefile.map_branch_type, "netware", false) == false
        Wizard.DisableBackButton
      end
      UI.SetFocus(Id(:next))
      UI.ChangeWidget(Id(:template), :Notify, true)

      UI.ChangeWidget(
        Id(:hardwaretype),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_select_type, "hardwaretype", 0)
      )
      UI.ChangeWidget(
        Id(:installtype),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_select_type, "installtype", 0)
      )
      if Ops.get_boolean(Responsefile.map_select_type, "template", false)
        UI.ChangeWidget(Id(:template), :Value, true)
        UI.ChangeWidget(Id(:path), :Enabled, true)
        UI.ChangeWidget(Id(:browse), :Enabled, true)
      else
        UI.ChangeWidget(Id(:template), :Value, false)
        UI.ChangeWidget(Id(:path), :Enabled, false)
        UI.ChangeWidget(Id(:browse), :Enabled, false)
      end
      UI.ChangeWidget(
        Id(:path),
        :Value,
        Ops.get_string(Responsefile.map_select_type, "path", "")
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          pathname = Convert.to_string(UI.QueryWidget(Id(:path), :Value))
          break if !Convert.to_boolean(UI.QueryWidget(Id(:template), :Value))
          if Responsefile.ValidatePathName(pathname, true)
            if SCR.Execute(path(".target.bash"), Ops.add("test -f ", pathname)) == 0
              rmap = Convert.to_map(
                SCR.Execute(
                  path(".target.bash_output"),
                  Builtins.sformat("test `stat -c %%s %1` -le 100000", pathname)
                )
              )
              Builtins.y2milestone(
                "Responsefile::SelectTypeDialog() rmap %1",
                rmap
              )

              if Ops.get_integer(rmap, "exit", -1) != 0
                Report.Error(
                  Builtins.sformat(_("%1 file is invalid."), pathname)
                )
              else
                break
              end
            else
              Report.Error(
                Builtins.sformat(_("%1 file does not exist."), pathname)
              )
            end
          end
        elsif ret == :template
          if Convert.to_boolean(UI.QueryWidget(Id(:template), :Value))
            UI.ChangeWidget(Id(:path), :Enabled, true)
            UI.ChangeWidget(Id(:browse), :Enabled, true)
          else
            UI.ChangeWidget(Id(:path), :Enabled, false)
            UI.ChangeWidget(Id(:browse), :Enabled, false)
          end
        # back
        elsif ret == :back
          break
        elsif ret == :browse
          templatePath = Convert.to_string(UI.QueryWidget(Id(:path), :Value))
          Builtins.y2milestone(
            "Responsefile::SelectTypeDialog() Template Path %1",
            templatePath
          )
          templatePath = UI.AskForExistingFile(
            templatePath,
            "",
            _("Browse RF Path")
          )
          UI.ChangeWidget(Id(:path), :Value, templatePath)
        else
          Builtins.y2error(
            "Responsefile::SelectTypeDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_select_type,
          "hardwaretype",
          Convert.to_integer(UI.QueryWidget(Id(:hardwaretype), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_select_type,
          "installtype",
          Convert.to_integer(UI.QueryWidget(Id(:installtype), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_select_type,
          "template",
          Convert.to_boolean(UI.QueryWidget(Id(:template), :Value))
        )
        Ops.set(
          Responsefile.map_select_type,
          "path",
          Convert.to_string(UI.QueryWidget(Id(:path), :Value))
        )
        installtype = Ops.get_integer(
          Responsefile.map_select_type,
          "installtype",
          0
        )
        if installtype == 2 || installtype == 4 || installtype == 5
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add("cp -f ", Responsefile.homesrcpath),
                  "default.rsp "
                ),
                Responsefile.homebuildpath
              ),
              "default"
            )
          )
        else
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add("cp -f ", Responsefile.homesrcpath),
                  "default1.rsp "
                ),
                Responsefile.homebuildpath
              ),
              "default"
            )
          )
        end

        SCR.Execute(
          path(".target.bash"),
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add("cp -f ", Responsefile.homesrcpath),
                "upgrade.ips "
              ),
              Responsefile.homebuildpath
            ),
            "upgrade"
          )
        )
        SCR.Execute(
          path(".target.bash"),
          Ops.add(Ops.add("chmod 666 ", Responsefile.homebuildpath), "default")
        )
        SCR.Execute(
          path(".target.bash"),
          Ops.add(Ops.add("chmod 666 ", Responsefile.homebuildpath), "upgrade")
        )
        Responsefile.InitializeVariables
        if Ops.get_boolean(Responsefile.map_select_type, "template", false)
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    "cp -f ",
                    Ops.get_string(Responsefile.map_select_type, "path", "")
                  ),
                  " "
                ),
                Responsefile.homebuildpath
              ),
              "template"
            )
          )
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add("chmod 666 ", Responsefile.homebuildpath),
              "template"
            )
          )
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add("cd ", Responsefile.homebuildpath),
                  "; dos2unix "
                ),
                Responsefile.homebuildpath
              ),
              "template"
            )
          )
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add("chmod 666 ", Responsefile.homebuildpath),
              "template"
            )
          )
          Responsefile.ImportRFTemplate
          tmpstr = Ops.get_string(Responsefile.map_select_type, "path", "")
          tmpint = Builtins.findlastof(tmpstr, "/")
          temint = Builtins.size(tmpstr)
          Builtins.y2milestone(
            "Responsefile::SelectTypeDialog() last %1 size %2 String %3",
            tmpint,
            temint,
            tmpstr
          )
          if Ops.greater_than(tmpint, -1) &&
              Ops.greater_than(Ops.subtract(temint, 1), tmpint)
            tmpstr1 = Builtins.substring(tmpstr, 0, Ops.add(tmpint, 1))
            Ops.set(Responsefile.map_save_rf, "tmprfpath", tmpstr1)
            tmpstr1 = Builtins.substring(
              tmpstr,
              Ops.add(tmpint, 1),
              Ops.subtract(Ops.subtract(temint, tmpint), 1)
            )
            Ops.set(Responsefile.map_save_rf, "rfname", tmpstr1)
          end
        end
        Ops.set(Responsefile.map_select_type, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # Summary Dialog of all the other Dialogs
    # @return `abort if aborted and `next otherwise
    def SummaryDialog
      Builtins.y2milestone("Responsefile::SummaryDialog() started")
      # Groupwise summary dialog caption
      caption = _("Response File Generator")

      # change menu
      itemsList = []

      installtype = Ops.get_integer(
        Responsefile.map_select_type,
        "installtype",
        0
      )
      hardwaretype = Ops.get_integer(
        Responsefile.map_select_type,
        "hardwaretype",
        0
      )
      Builtins.y2milestone(
        "Responsefile::SummaryDialog() install type %1",
        installtype
      )

      if installtype != 4 && hardwaretype == 1
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:srcAddr), _("Source Address"))
        )
        Ops.set(Responsefile.map_src_addr, "visible", true)
        if Ops.get_integer(Responsefile.map_src_addr, "type", 0) == 0
          itemsList = Builtins.add(
            itemsList,
            Item(Id(:srceDir), _("Source eDirectory"))
          )
          Ops.set(Responsefile.map_src_edir, "visible", true)
          Builtins.y2milestone(
            "Responsefile::SummaryDialog() map_src_edir true"
          )
        else
          Ops.set(Responsefile.map_src_edir, "visible", false)
          Builtins.y2milestone(
            "Responsefile::SummaryDialog() map_src_edir false"
          )
        end
      else
        Ops.set(Responsefile.map_src_addr, "visible", false)
        Ops.set(Responsefile.map_src_edir, "visible", false)
      end
      if installtype != 3 && installtype != 5
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:destAddr), _("Destination Address"))
        )
        Ops.set(Responsefile.map_dest_addr, "visible", true)
      else
        Ops.set(Responsefile.map_dest_addr, "visible", false)
      end
      if installtype != 5
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:desteDir), _("Destination eDirectory"))
        )
        Ops.set(Responsefile.map_dest_edir, "visible", true)
      else
        Ops.set(Responsefile.map_dest_edir, "visible", false)
      end
      if installtype == 5
        itemsList = Builtins.add(itemsList, Item(Id(:destLang), _("Language")))
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:selectPattern), _("Pattern Selection"))
        )
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:storeConfig), _("Storage Configuration"))
        )
        Ops.set(Responsefile.map_license, "visible", false)
        Ops.set(Responsefile.map_dest_addr, "visible1", false)
        Ops.set(Responsefile.map_dest_lang, "visible", true)
        Ops.set(Responsefile.map_dest_slp, "visible", false)
        Ops.set(Responsefile.map_dest_dns, "visible", false)
        Ops.set(Responsefile.map_timezones, "visible", false)
        Ops.set(Responsefile.map_timesync, "visible", false)
        Ops.set(Responsefile.map_install, "visible", false)
        Ops.set(Responsefile.map_install, "visible1", false)
        Ops.set(Responsefile.map_storage, "visible", true)
        Ops.set(Responsefile.map_patterns, "visible", true)
        if Ops.get_integer(Responsefile.map_patterns, "pattern", 0) == 0
          itemsList = Builtins.add(
            itemsList,
            Item(Id(:selectProducts), _("Product Selection"))
          )
          Ops.set(Responsefile.map_products, "visible", true)
        else
          Ops.set(Responsefile.map_products, "visible", false)
        end
      elsif installtype != 3
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:destLicense), _("License"))
        )
        itemsList = Builtins.add(itemsList, Item(Id(:destProt), _("Protocols")))
        itemsList = Builtins.add(itemsList, Item(Id(:destLang), _("Language")))
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:destSLP), _("SLP Configuration"))
        )
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:destDNS), _("DNS Configuration"))
        )
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:timeZone), _("Time Zone Configuration"))
        )
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:timeSync), _("Time Sync Configuration"))
        )
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:installSet), _("Install Settings"))
        )
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:selectPattern), _("Pattern Selection"))
        )
        if installtype != 4
          itemsList = Builtins.add(
            itemsList,
            Item(Id(:storeConfig), _("Storage Configuration"))
          )
          Ops.set(Responsefile.map_storage, "visible", true)
        else
          Ops.set(Responsefile.map_storage, "visible", false)
        end
        Ops.set(Responsefile.map_license, "visible", true)
        Ops.set(Responsefile.map_dest_addr, "visible1", true)
        Ops.set(Responsefile.map_dest_lang, "visible", true)
        Ops.set(Responsefile.map_dest_slp, "visible", true)
        Ops.set(Responsefile.map_dest_dns, "visible", true)
        Ops.set(Responsefile.map_timezones, "visible", true)
        Ops.set(Responsefile.map_timesync, "visible", true)
        Ops.set(Responsefile.map_install, "visible", true)
        Ops.set(Responsefile.map_install, "visible1", false)
        Ops.set(Responsefile.map_patterns, "visible", true)
        if Ops.get_integer(Responsefile.map_patterns, "pattern", 0) == 0
          itemsList = Builtins.add(
            itemsList,
            Item(Id(:selectProducts), _("Product Selection"))
          )
          Ops.set(Responsefile.map_products, "visible", true)
        else
          Ops.set(Responsefile.map_products, "visible", false)
        end
      else
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:upgradeSet), _("Upgrade Settings"))
        )
        itemsList = Builtins.add(
          itemsList,
          Item(Id(:selectProducts), _("Product Selection"))
        )
        Ops.set(Responsefile.map_license, "visible", false)
        Ops.set(Responsefile.map_dest_addr, "visible1", false)
        Ops.set(Responsefile.map_dest_lang, "visible", false)
        Ops.set(Responsefile.map_dest_slp, "visible", false)
        Ops.set(Responsefile.map_dest_dns, "visible", false)
        Ops.set(Responsefile.map_timezones, "visible", false)
        Ops.set(Responsefile.map_timesync, "visible", false)
        Ops.set(Responsefile.map_install, "visible", false)
        Ops.set(Responsefile.map_install, "visible1", true)
        Ops.set(Responsefile.map_storage, "visible", false)
        Ops.set(Responsefile.map_patterns, "visible", false)
        Ops.set(Responsefile.map_products, "visible", true)
      end

      if installtype != 5
        if Responsefile.CheckProductPrameter(13)
          itemsList = Builtins.add(
            itemsList,
            Item(Id(:nmasConfigure), _("NMAS Configuration"))
          )
          Ops.set(Responsefile.map_nmas, "visible", true)
        else
          Ops.set(Responsefile.map_nmas, "visible", false)
        end
        if Responsefile.CheckProductPrameter(15)
          itemsList = Builtins.add(
            itemsList,
            Item(Id(:dnsdhcpConfigure), _("DNS/DHCP Configuration"))
          )
          Ops.set(Responsefile.map_dns_dhcp, "visible", true)
        else
          Ops.set(Responsefile.map_dns_dhcp, "visible", false)
        end
        if Responsefile.CheckProductPrameter(16)
          itemsList = Builtins.add(
            itemsList,
            Item(Id(:ifolderConfigure), _("iFolder Server Configuration"))
          )
          Ops.set(Responsefile.map_ifolder, "visible", true)
        else
          Ops.set(Responsefile.map_ifolder, "visible", false)
        end
        if Responsefile.CheckProductPrameter(19)
          itemsList = Builtins.add(
            itemsList,
            Item(Id(:mysqlConfigure), _("MySQL Configuration"))
          )
          Ops.set(Responsefile.map_mysql, "visible", true)
        else
          Ops.set(Responsefile.map_mysql, "visible", false)
        end
        if Responsefile.CheckProductPrameter(21)
          itemsList = Builtins.add(
            itemsList,
            Item(
              Id(:extendConfigure),
              _("exteNd Application Server Configuration")
            )
          )
          Ops.set(Responsefile.map_extend, "visible", true)
        else
          Ops.set(Responsefile.map_extend, "visible", false)
        end
      else
        Ops.set(Responsefile.map_nmas, "visible", false)
        Ops.set(Responsefile.map_dns_dhcp, "visible", false)
        Ops.set(Responsefile.map_ifolder, "visible", false)
        Ops.set(Responsefile.map_mysql, "visible", false)
        Ops.set(Responsefile.map_extend, "visible", false)
      end
      Ops.set(Responsefile.map_select_type, "visible", true)
      Ops.set(Responsefile.map_save_rf, "visible", true)

      summary = Responsefile.Summary
      unconfigured = Ops.get_list(summary, 1, [])
      configured = Ops.get_string(summary, 0, "")
      menu_box = HBox(
        HStretch(),
        MenuButton(Id(:menu_dummy), _("Change..."), itemsList),
        HStretch()
      )

      contents = VBox(
        Left(
          Label(
            _(
              "Click any headline to make changes or use the \"Changes...\" menu below."
            )
          )
        ),
        HBox(RichText(Id(:proposal), configured)),
        VSpacing(0.1),
        menu_box,
        VSpacing(0.2)
      )


      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "summary", ""),
        Label.BackButton,
        Label.NextButton
      )
      # Set keyboard focus to the [Next] button
      UI.SetFocus(Id(:next))

      ret = nil
      while true
        ret = UI.UserInput
        Builtins.y2error("Responsefile::SummaryDialog() Return code: %1", ret)
        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          next if !Responsefile.CheckIfComplete
          break
        elsif ret == :back
          break
        elsif ret == :srcAddr
          break
        elsif ret == :srceDir
          break
        elsif ret == :destAddr
          break
        elsif ret == :desteDir
          break
        elsif ret == :destLicense
          break
        elsif ret == :destProt
          break
        elsif ret == :destLang
          break
        elsif ret == :destSLP
          break
        elsif ret == :destDNS
          break
        elsif ret == :timeZone
          break
        elsif ret == :timeSync
          break
        elsif ret == :installSet
          break
        elsif ret == :upgradeSet
          break
        elsif ret == :storeConfig
          break
        elsif ret == :selectPattern
          break
        elsif ret == :selectProducts
          break
        elsif ret == :nmasConfigure
          break
        elsif ret == :dnsdhcpConfigure
          break
        elsif ret == :ifolderConfigure
          break
        elsif ret == :mysqlConfigure
          break
        elsif ret == :extendConfigure
          break
        elsif Ops.is_string?(ret)
          if Builtins.issubstring(Convert.to_string(ret), "srcAddr")
            ret = :srcAddr
          elsif Builtins.issubstring(Convert.to_string(ret), "srceDir")
            ret = :srceDir
          elsif Builtins.issubstring(Convert.to_string(ret), "destAddr")
            ret = :destAddr
          elsif Builtins.issubstring(Convert.to_string(ret), "desteDir")
            ret = :desteDir
          elsif Builtins.issubstring(Convert.to_string(ret), "destProt")
            ret = :destProt
          elsif Builtins.issubstring(Convert.to_string(ret), "destLicense")
            ret = :destLicense
          elsif Builtins.issubstring(Convert.to_string(ret), "destLang")
            ret = :destLang
          elsif Builtins.issubstring(Convert.to_string(ret), "destSLP")
            ret = :destSLP
          elsif Builtins.issubstring(Convert.to_string(ret), "destDNS")
            ret = :destDNS
          elsif Builtins.issubstring(Convert.to_string(ret), "timeZone")
            ret = :timeZone
          elsif Builtins.issubstring(Convert.to_string(ret), "timeSync")
            ret = :timeSync
          elsif Builtins.issubstring(Convert.to_string(ret), "installSet")
            ret = :installSet
          elsif Builtins.issubstring(Convert.to_string(ret), "upgradeSet")
            ret = :upgradeSet
          elsif Builtins.issubstring(Convert.to_string(ret), "storeConfig")
            ret = :storeConfig
          elsif Builtins.issubstring(Convert.to_string(ret), "selectPattern")
            ret = :selectPattern
          elsif Builtins.issubstring(Convert.to_string(ret), "selectProducts")
            ret = :selectProducts
          elsif Builtins.issubstring(Convert.to_string(ret), "nmasConfigure")
            ret = :nmasConfigure
          elsif Builtins.issubstring(Convert.to_string(ret), "dnsdhcpConfigure")
            ret = :dnsdhcpConfigure
          elsif Builtins.issubstring(Convert.to_string(ret), "ifolderConfigure")
            ret = :ifolderConfigure
          elsif Builtins.issubstring(Convert.to_string(ret), "mysqlConfigure")
            ret = :mysqlConfigure
          elsif Builtins.issubstring(Convert.to_string(ret), "extendConfigure")
            ret = :extendConfigure
          else
            Builtins.y2error(
              "Responsefile::SummaryDialog() Unexpected string return code: %1",
              ret
            )
            next
          end
          break
        else
          Builtins.y2error(
            "Responsefile::SummaryDialog() Unexpected symbol return code: %1",
            ret
          )
          next
        end
      end
      Convert.to_symbol(ret)
    end

    # Source Address dialog
    # @return dialog result
    def SourceAddrDialog
      Builtins.y2milestone("Responsefile::SourceAddrDialog() started")
      # SourceAddr dialog caption
      caption = _("Source Properties")
      radioButtonString0 = ""
      radioButtonString1 = ""

      installtype = Ops.get_integer(
        Responsefile.map_select_type,
        "installtype",
        0
      )
      hardwaretype = Ops.get_integer(
        Responsefile.map_select_type,
        "hardwaretype",
        0
      )

      if installtype == 3 || installtype == 1
        radioButtonString0 = _("NetWare Server")
        radioButtonString1 = _("Workstation")
      else
        radioButtonString0 = _("Remote NetWare Server")
        radioButtonString1 = _("Local Storage (HDD, CD-ROM)")
      end

      contents = HVCenter(
        VBox(
          Left(
            Heading(
              _("Enter the source information for the installation files.")
            )
          ),
          VSpacing(1),
          Left(
            Frame(
              _("Location type"),
              MarginBox(
                1.5,
                0.2,
                RadioButtonGroup(
                  Id(:loctype),
                  VBox(
                    Left(RadioButton(Id(0), radioButtonString0, true)),
                    Left(RadioButton(Id(1), radioButtonString1))
                  )
                )
              )
            )
          ),
          VSpacing(2),
          Left(
            MinWidth(
              35,
              TextEntry(Id(:srcname), Opt(:hstretch), _("Server Name"))
            )
          ),
          VSpacing(1),
          Left(
            MinWidth(
              35,
              TextEntry(Id(:srcipaddr), Opt(:hstretch), _("Server IP Address"))
            )
          ),
          VSpacing(1),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:srclocpath),
                Opt(:hstretch),
                _("Source Location Path")
              )
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "srcaddr", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      if installtype != 1
        UI.ChangeWidget(
          Id(:loctype),
          :CurrentButton,
          Ops.get_integer(Responsefile.map_src_addr, "type", 0)
        )
      else
        UI.ChangeWidget(Id(:loctype), :CurrentButton, 0)
        UI.ChangeWidget(Id(:loctype), :Enabled, true)
        UI.ChangeWidget(Id(0), :Enabled, false)
        UI.ChangeWidget(Id(1), :Enabled, false)
      end

      UI.ChangeWidget(
        Id(:srcname),
        :Value,
        Ops.get_string(Responsefile.map_src_addr, "srcname", "")
      )
      UI.ChangeWidget(
        Id(:srcipaddr),
        :Value,
        Ops.get_string(Responsefile.map_src_addr, "srcipaddr", "")
      )
      UI.ChangeWidget(
        Id(:srclocpath),
        :Value,
        Ops.get_string(Responsefile.map_src_addr, "srclocpath", "")
      )

      Builtins.y2milestone(
        "Responsefile::SourceAddrDialog() installtype %1  hardwaretype %2",
        installtype,
        hardwaretype
      )
      if installtype != 3
        UI.ChangeWidget(Id(0), :Notify, true)
        UI.ChangeWidget(Id(1), :Notify, true)

        if Convert.to_integer(UI.QueryWidget(Id(:loctype), :CurrentButton)) == 1
          UI.ChangeWidget(Id(:srcname), :Enabled, false)
          UI.ChangeWidget(Id(:srcipaddr), :Enabled, false)
          UI.ChangeWidget(Id(:srclocpath), :Enabled, false)
        end
      end

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_integer(UI.QueryWidget(Id(:loctype), :CurrentButton)) == 0
            if Convert.to_string(UI.QueryWidget(Id(:srcname), :Value)) == ""
              Report.Error(_("All entries must be filled in."))
              next
            end
            if !Responsefile.ValidateIPMsg(
                Convert.to_string(UI.QueryWidget(Id(:srcipaddr), :Value)),
                _("Invalid IP address.")
              ) ||
                !Responsefile.ValidateNWPathname(
                  Convert.to_string(UI.QueryWidget(Id(:srclocpath), :Value))
                )
              next
            end
          end
          break
        elsif ret == :back
          break
        elsif Ops.is_integer?(ret)
          if ret == 0 # mode == NetWare Server
            UI.ChangeWidget(Id(:srcname), :Enabled, true)
            UI.ChangeWidget(Id(:srcipaddr), :Enabled, true)
            UI.ChangeWidget(Id(:srclocpath), :Enabled, true)
          else
            UI.ChangeWidget(Id(:srcname), :Enabled, false)
            UI.ChangeWidget(Id(:srcipaddr), :Enabled, false)
            UI.ChangeWidget(Id(:srclocpath), :Enabled, false)
          end
        else
          Builtins.y2error(
            "Responsefile::SourceAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_src_addr,
          "type",
          Convert.to_integer(UI.QueryWidget(Id(:loctype), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_src_addr,
          "srcname",
          Convert.to_string(UI.QueryWidget(Id(:srcname), :Value))
        )
        Ops.set(
          Responsefile.map_src_addr,
          "srcipaddr",
          Convert.to_string(UI.QueryWidget(Id(:srcipaddr), :Value))
        )
        Ops.set(
          Responsefile.map_src_addr,
          "srclocpath",
          Convert.to_string(UI.QueryWidget(Id(:srclocpath), :Value))
        )
        Ops.set(Responsefile.map_src_addr, "complete", true)
      end
      Convert.to_symbol(ret)
    end

    # Source eDir dialog
    # @return dialog result
    def SourceDirDialog
      Builtins.y2milestone("Responsefile::SourceDirDialog() started")
      # SourceAddr dialog caption
      caption = _("Source eDirectory Properties")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the source eDirectory information."))),
          VSpacing(1),
          Left(
            Frame(
              _("eDirectory Information"),
              MarginBox(
                1.5,
                0.2,
                VBox(
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:srctreename),
                        Opt(:hstretch),
                        _("Tree Name")
                      )
                    )
                  ),
                  VSpacing(0.5),
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:srcsrvctx),
                        Opt(:hstretch),
                        _("Context for Server Object")
                      )
                    )
                  )
                )
              )
            )
          ),
          VSpacing(0.8),
          Left(
            Frame(
              _("User Information"),
              MarginBox(
                1.5,
                0.2,
                VBox(
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:srcusername),
                        Opt(:hstretch),
                        _("User Name")
                      )
                    )
                  ),
                  VSpacing(0.5),
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:srcuserctx),
                        Opt(:hstretch),
                        _("User Context")
                      )
                    )
                  ),
                  VSpacing(0.5),
                  Left(
                    MinWidth(
                      35,
                      Password(Id(:srcpassword), Opt(:hstretch), _("Password"))
                    )
                  ),
                  VSpacing(0.5),
                  Left(
                    MinWidth(
                      35,
                      Password(
                        Id(:srcrpassword),
                        Opt(:hstretch),
                        _("Retype Password")
                      )
                    )
                  )
                )
              )
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "srcedir", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:srctreename),
        :Value,
        Ops.get_string(Responsefile.map_src_edir, "srctreename", "")
      )
      UI.ChangeWidget(
        Id(:srcsrvctx),
        :Value,
        Ops.get_string(Responsefile.map_src_edir, "srcsrvctx", "")
      )
      UI.ChangeWidget(
        Id(:srcusername),
        :Value,
        Ops.get_string(Responsefile.map_src_edir, "srcusername", "")
      )
      UI.ChangeWidget(
        Id(:srcuserctx),
        :Value,
        Ops.get_string(Responsefile.map_src_edir, "srcuserctx", "")
      )
      UI.ChangeWidget(
        Id(:srcpassword),
        :Value,
        Ops.get_string(Responsefile.map_src_edir, "srcpassword", "")
      )
      UI.ChangeWidget(
        Id(:srcrpassword),
        :Value,
        Ops.get_string(Responsefile.map_src_edir, "srcpassword", "")
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:srctreename), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:srcsrvctx), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:srcusername), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:srcuserctx), :Value)) == ""
            Report.Error(_("All entries must be filled in."))
            next
          end
          if !Responsefile.ValidatePasswords(
              Convert.to_string(UI.QueryWidget(Id(:srcpassword), :Value)),
              Convert.to_string(UI.QueryWidget(Id(:srcrpassword), :Value))
            )
            next
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error(
            "Responsefile::SourcDirDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_src_edir,
          "srctreename",
          Convert.to_string(UI.QueryWidget(Id(:srctreename), :Value))
        )
        Ops.set(
          Responsefile.map_src_edir,
          "srcsrvctx",
          Convert.to_string(UI.QueryWidget(Id(:srcsrvctx), :Value))
        )
        Ops.set(
          Responsefile.map_src_edir,
          "srcusername",
          Convert.to_string(UI.QueryWidget(Id(:srcusername), :Value))
        )
        Ops.set(
          Responsefile.map_src_edir,
          "srcuserctx",
          Convert.to_string(UI.QueryWidget(Id(:srcuserctx), :Value))
        )
        Ops.set(
          Responsefile.map_src_edir,
          "srcpassword",
          Convert.to_string(UI.QueryWidget(Id(:srcpassword), :Value))
        )
        Ops.set(Responsefile.map_src_edir, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # Destination Address dialog
    # @return dialog result
    def DestinationAddrDialog
      Builtins.y2milestone("Responsefile::DestinationAddrDialog() started")
      # SourceAddr dialog caption
      caption = _("Destination Properties")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the destination server information."))),
          VSpacing(1),
          Left(
            MinWidth(
              35,
              TextEntry(Id(:destname), Opt(:hstretch), _("Server Name"))
            )
          ),
          VSpacing(1),
          Left(
            MinWidth(
              35,
              TextEntry(Id(:destipaddr), Opt(:hstretch), _("Server IP Address"))
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "destaddr", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:destname),
        :Value,
        Ops.get_string(Responsefile.map_dest_addr, "destname", "")
      )
      UI.ChangeWidget(
        Id(:destipaddr),
        :Value,
        Ops.get_string(Responsefile.map_dest_addr, "destipaddr", "")
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:destname), :Value)) == ""
            Report.Error(_("All entries must be filled in."))
            next
          end
          if !Responsefile.ValidateIPMsg(
              Convert.to_string(UI.QueryWidget(Id(:destipaddr), :Value)),
              _("Invalid IP address.")
            )
            next
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error(
            "Responsefile::DestinationAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_dest_addr,
          "destname",
          Convert.to_string(UI.QueryWidget(Id(:destname), :Value))
        )
        Ops.set(
          Responsefile.map_dest_addr,
          "destipaddr",
          Convert.to_string(UI.QueryWidget(Id(:destipaddr), :Value))
        )
        Ops.set(Responsefile.map_dest_addr, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # Destination eDir dialog
    # @return dialog result
    def DestinationeDirDialog
      Builtins.y2milestone("Responsefile::DestinationeDirDialog() started")
      # SourceAddr dialog caption
      caption = _("Destination eDirectory Properties")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the destination eDirectory information."))),
          VSpacing(0.6),
          Left(CheckBox(Id(:destnewtree), _("New tree"))),
          VSpacing(0.4),
          Left(
            Frame(
              _("eDirectory Information"),
              MarginBox(
                1.5,
                0.2,
                VBox(
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:destreename),
                        Opt(:hstretch),
                        _("Tree Name")
                      )
                    )
                  ),
                  VSpacing(0.2),
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:destsrvctx),
                        Opt(:hstretch),
                        _("Context for Server Object")
                      )
                    )
                  ),
                  VSpacing(0.2),
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:replicaipaddr),
                        Opt(:hstretch),
                        _("Replica Server IP Address")
                      )
                    )
                  )
                )
              )
            )
          ),
          VSpacing(0.8),
          Left(
            Frame(
              _("User Information"),
              MarginBox(
                1.5,
                0.2,
                VBox(
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:destadminname),
                        Opt(:hstretch),
                        _("Admin Name")
                      )
                    )
                  ),
                  VSpacing(0.2),
                  Left(
                    MinWidth(
                      35,
                      TextEntry(
                        Id(:destadminctx),
                        Opt(:hstretch),
                        _("Admin Context")
                      )
                    )
                  ),
                  VSpacing(0.2),
                  Left(
                    MinWidth(
                      35,
                      Password(Id(:destpassword), Opt(:hstretch), _("Password"))
                    )
                  ),
                  VSpacing(0.2),
                  Left(
                    MinWidth(
                      35,
                      Password(
                        Id(:destrpassword),
                        Opt(:hstretch),
                        _("Retype Password")
                      )
                    )
                  )
                )
              )
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "destedir", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))
      type = Ops.get_integer(Responsefile.map_select_type, "installtype", 0)

      if type == 3
        UI.ChangeWidget(Id(:destnewtree), :Value, false)
        UI.ChangeWidget(Id(:destnewtree), :Enabled, false)
      else
        UI.ChangeWidget(
          Id(:destnewtree),
          :Value,
          Ops.get_boolean(Responsefile.map_dest_edir, "destnewtree", false)
        )
        UI.ChangeWidget(Id(:destnewtree), :Enabled, true)
        UI.ChangeWidget(Id(:destnewtree), :Notify, true)
      end
      UI.ChangeWidget(
        Id(:destreename),
        :Value,
        Ops.get_string(Responsefile.map_dest_edir, "destreename", "")
      )
      UI.ChangeWidget(
        Id(:destsrvctx),
        :Value,
        Ops.get_string(Responsefile.map_dest_edir, "destsrvctx", "")
      )
      if Convert.to_boolean(UI.QueryWidget(Id(:destnewtree), :Value)) == false
        UI.ChangeWidget(
          Id(:replicaipaddr),
          :Value,
          Ops.get_string(Responsefile.map_dest_edir, "replicaipaddr", "")
        )
        UI.ChangeWidget(Id(:replicaipaddr), :Enabled, true)
      else
        UI.ChangeWidget(Id(:replicaipaddr), :Value, "")
        UI.ChangeWidget(Id(:replicaipaddr), :Enabled, false)
      end
      UI.ChangeWidget(
        Id(:destadminname),
        :Value,
        Ops.get_string(Responsefile.map_dest_edir, "destadminname", "")
      )
      UI.ChangeWidget(
        Id(:destadminctx),
        :Value,
        Ops.get_string(Responsefile.map_dest_edir, "destadminctx", "")
      )
      UI.ChangeWidget(
        Id(:destpassword),
        :Value,
        Ops.get_string(Responsefile.map_dest_edir, "destpassword", "")
      )
      UI.ChangeWidget(
        Id(:destrpassword),
        :Value,
        Ops.get_string(Responsefile.map_dest_edir, "destpassword", "")
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:destreename), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:destsrvctx), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:destadminname), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:destadminctx), :Value)) == ""
            Report.Error(_("An entry is missing."))
            next
          end
          if !Responsefile.ValidatePasswords(
              Convert.to_string(UI.QueryWidget(Id(:destpassword), :Value)),
              Convert.to_string(UI.QueryWidget(Id(:destrpassword), :Value))
            )
            next
          end
          break
        elsif ret == :back
          break
        elsif ret == :destnewtree
          if Convert.to_boolean(UI.QueryWidget(Id(:destnewtree), :Value)) == false
            UI.ChangeWidget(
              Id(:replicaipaddr),
              :Value,
              Ops.get_string(Responsefile.map_dest_edir, "replicaipaddr", "")
            )
            UI.ChangeWidget(Id(:replicaipaddr), :Enabled, true)
          else
            UI.ChangeWidget(Id(:replicaipaddr), :Value, "")
            UI.ChangeWidget(Id(:replicaipaddr), :Enabled, false)
          end
        else
          Builtins.y2error(
            "Responsefile::DestinationeDirDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_dest_edir,
          "destnewtree",
          Convert.to_boolean(UI.QueryWidget(Id(:destnewtree), :Value))
        )
        Ops.set(
          Responsefile.map_dest_edir,
          "destreename",
          Convert.to_string(UI.QueryWidget(Id(:destreename), :Value))
        )
        Ops.set(
          Responsefile.map_dest_edir,
          "destsrvctx",
          Convert.to_string(UI.QueryWidget(Id(:destsrvctx), :Value))
        )
        Ops.set(
          Responsefile.map_dest_edir,
          "replicaipaddr",
          Convert.to_string(UI.QueryWidget(Id(:replicaipaddr), :Value))
        )
        Ops.set(
          Responsefile.map_dest_edir,
          "destadminname",
          Convert.to_string(UI.QueryWidget(Id(:destadminname), :Value))
        )
        Ops.set(
          Responsefile.map_dest_edir,
          "destadminctx",
          Convert.to_string(UI.QueryWidget(Id(:destadminctx), :Value))
        )
        Ops.set(
          Responsefile.map_dest_edir,
          "destpassword",
          Convert.to_string(UI.QueryWidget(Id(:destpassword), :Value))
        )
        Ops.set(Responsefile.map_dest_edir, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # License dialog
    # @return dialog result
    def LicenseDialog
      Builtins.y2milestone("Responsefile::LicenseDialog() started")
      # SourceAddr dialog caption
      caption = _("License Properties")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the license information."))),
          VSpacing(1),
          Left(
            MinWidth(
              35,
              TextEntry(Id(:context), Opt(:hstretch), _("NDS License Context"))
            )
          ),
          VSpacing(1),
          Left(CheckBox(Id(:nolicense), _("Install without licenses")))
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "license", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))
      UI.ChangeWidget(Id(:nolicense), :Notify, true)


      UI.ChangeWidget(
        Id(:context),
        :Value,
        Ops.get_string(Responsefile.map_license, "context", "")
      )
      UI.ChangeWidget(
        Id(:nolicense),
        :Value,
        Ops.get_boolean(Responsefile.map_license, "nolicense", false)
      )
      if Convert.to_boolean(UI.QueryWidget(Id(:nolicense), :Value))
        UI.ChangeWidget(Id(:context), :Enabled, false)
      end

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if !Convert.to_boolean(UI.QueryWidget(Id(:nolicense), :Value)) &&
              Convert.to_string(UI.QueryWidget(Id(:context), :Value)) == ""
            Report.Error(_("The License context must be filled in."))
            next
          end
          break
        elsif ret == :back
          break
        elsif ret == :nolicense
          if Convert.to_boolean(UI.QueryWidget(Id(:nolicense), :Value))
            UI.ChangeWidget(Id(:context), :Enabled, false)
          else
            UI.ChangeWidget(Id(:context), :Enabled, true)
          end
          next
        else
          Builtins.y2error(
            "Responsefile::DestinationAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_license,
          "context",
          Convert.to_string(UI.QueryWidget(Id(:context), :Value))
        )
        Ops.set(
          Responsefile.map_license,
          "nolicense",
          Convert.to_boolean(UI.QueryWidget(Id(:nolicense), :Value))
        )
        Ops.set(Responsefile.map_license, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # Protocols dialog
    # @return dialog result
    def ProtocolsDialog
      Builtins.y2milestone("Responsefile::ProtocolsDialog() started")
      # SourceAddr dialog caption
      caption = _("Protocols Properties")

      contents = HVCenter(
        VBox(
          Left(Heading("Enter the destination server protocol information.")),
          VSpacing(1),
          HBox(
            VBox(
              Left(
                Frame(
                  _("IP"),
                  MarginBox(
                    1.5,
                    0.2,
                    VBox(
                      Left(CheckBox(Id(:selectip), _("IP"), true)),
                      VSpacing(0.8),
                      Left(
                        MinWidth(
                          35,
                          TextEntry(
                            Id(:destipaddr),
                            Opt(:hstretch),
                            _("IP Address")
                          )
                        )
                      ),
                      VSpacing(0.5),
                      Left(
                        MinWidth(
                          35,
                          TextEntry(
                            Id(:destsubnet),
                            Opt(:hstretch),
                            _("Subnet Mask")
                          )
                        )
                      ),
                      VSpacing(0.5),
                      Left(
                        MinWidth(
                          35,
                          TextEntry(
                            Id(:destrouter),
                            Opt(:hstretch),
                            _("Router (Gateway)")
                          )
                        )
                      )
                    )
                  )
                )
              )
            ),
            HSpacing(0.8),
            VBox(
              Left(
                Frame(
                  _("IPX"),
                  MarginBox(
                    1.5,
                    0.2,
                    VBox(
                      Left(CheckBox(Id(:selectipx), _("IPX"))),
                      VSpacing(0.8),
                      VBox(
                        Left(
                          Frame(
                            "Frame types",
                            MarginBox(
                              1.5,
                              0.2,
                              RadioButtonGroup(
                                Id(:rb),
                                VBox(
                                  Left(
                                    RadioButton(
                                      Id(0),
                                      Ops.get(
                                        Responsefile.list_frame_types,
                                        0,
                                        ""
                                      )
                                    )
                                  ),
                                  Left(
                                    RadioButton(
                                      Id(1),
                                      Ops.get(
                                        Responsefile.list_frame_types,
                                        1,
                                        ""
                                      )
                                    )
                                  ),
                                  Left(
                                    RadioButton(
                                      Id(2),
                                      Ops.get(
                                        Responsefile.list_frame_types,
                                        2,
                                        ""
                                      )
                                    )
                                  ),
                                  Left(
                                    RadioButton(
                                      Id(3),
                                      Ops.get(
                                        Responsefile.list_frame_types,
                                        3,
                                        ""
                                      )
                                    )
                                  )
                                )
                              )
                            )
                          )
                        )
                      ),
                      VSpacing(0.5),
                      Left(
                        MinWidth(
                          35,
                          TextEntry(Id(:destipxid), Opt(:hstretch), "Server ID")
                        )
                      )
                    )
                  )
                )
              )
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "protocols", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))
      UI.ChangeWidget(Id(:selectipx), :Notify, true)

      UI.ChangeWidget(Id(:selectip), :Enabled, false)
      UI.ChangeWidget(
        Id(:destipaddr),
        :Value,
        Ops.get_string(Responsefile.map_dest_addr, "destipaddr", "")
      )
      UI.ChangeWidget(
        Id(:destsubnet),
        :Value,
        Ops.get_string(Responsefile.map_dest_addr, "destsubnet", "")
      )
      UI.ChangeWidget(
        Id(:destrouter),
        :Value,
        Ops.get_string(Responsefile.map_dest_addr, "destrouter", "")
      )

      UI.ChangeWidget(
        Id(:rb),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_dest_addr, "ipxframe", 0)
      )
      UI.ChangeWidget(
        Id(:destipxid),
        :Value,
        Ops.get_string(Responsefile.map_dest_addr, "destipxid", "")
      )
      if Ops.get_boolean(Responsefile.map_dest_addr, "selectipx", false)
        UI.ChangeWidget(Id(:selectipx), :Value, true)
        UI.ChangeWidget(Id(:rb), :Enabled, true)
        UI.ChangeWidget(Id(0), :Enabled, true)
        UI.ChangeWidget(Id(1), :Enabled, true)
        UI.ChangeWidget(Id(2), :Enabled, true)
        UI.ChangeWidget(Id(3), :Enabled, true)
        UI.ChangeWidget(Id(:destipxid), :Enabled, true)
      else
        UI.ChangeWidget(Id(:selectipx), :Value, false)
        UI.ChangeWidget(Id(:rb), :Enabled, false)
        UI.ChangeWidget(Id(0), :Enabled, false)
        UI.ChangeWidget(Id(1), :Enabled, false)
        UI.ChangeWidget(Id(2), :Enabled, false)
        UI.ChangeWidget(Id(3), :Enabled, false)
        UI.ChangeWidget(Id(:destipxid), :Enabled, false)
      end

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if !Responsefile.ValidateIPMsg(
              Convert.to_string(UI.QueryWidget(Id(:destipaddr), :Value)),
              _("Invalid IP address.")
            ) ||
              !Responsefile.ValidateIPMsg(
                Convert.to_string(UI.QueryWidget(Id(:destsubnet), :Value)),
                _("Invalid Subnet mask.")
              ) ||
              !Responsefile.ValidateIPMsg(
                Convert.to_string(UI.QueryWidget(Id(:destrouter), :Value)),
                _("Invalid Router address.")
              )
            next
          end
          if Convert.to_boolean(UI.QueryWidget(Id(:selectipx), :Value))
            if Convert.to_string(UI.QueryWidget(Id(:destipxid), :Value)) == ""
              Report.Error(_("Server ID must be filled in."))
              next
            end
          end
          break
        elsif ret == :back
          break
        elsif ret == :selectipx
          if Convert.to_boolean(UI.QueryWidget(Id(:selectipx), :Value))
            UI.ChangeWidget(Id(:rb), :Enabled, true)
            UI.ChangeWidget(Id(0), :Enabled, true)
            UI.ChangeWidget(Id(1), :Enabled, true)
            UI.ChangeWidget(Id(2), :Enabled, true)
            UI.ChangeWidget(Id(3), :Enabled, true)
            UI.ChangeWidget(Id(:destipxid), :Enabled, true)
            Ops.set(Responsefile.map_dest_addr, "selectipx", true)
          else
            UI.ChangeWidget(Id(:rb), :Enabled, false)
            UI.ChangeWidget(Id(0), :Enabled, false)
            UI.ChangeWidget(Id(1), :Enabled, false)
            UI.ChangeWidget(Id(2), :Enabled, false)
            UI.ChangeWidget(Id(3), :Enabled, false)
            UI.ChangeWidget(Id(:destipxid), :Enabled, false)
            Ops.set(Responsefile.map_dest_addr, "selectipx", false)
          end
        else
          Builtins.y2error(
            "Responsefile::ProtocolsDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_dest_addr,
          "destipaddr",
          Convert.to_string(UI.QueryWidget(Id(:destipaddr), :Value))
        )
        Ops.set(
          Responsefile.map_dest_addr,
          "destsubnet",
          Convert.to_string(UI.QueryWidget(Id(:destsubnet), :Value))
        )
        Ops.set(
          Responsefile.map_dest_addr,
          "destrouter",
          Convert.to_string(UI.QueryWidget(Id(:destrouter), :Value))
        )

        Ops.set(
          Responsefile.map_dest_addr,
          "selectipx",
          Convert.to_boolean(UI.QueryWidget(Id(:selectipx), :Value))
        )
        Ops.set(
          Responsefile.map_dest_addr,
          "ipxframe",
          Convert.to_integer(UI.QueryWidget(Id(:rb), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_dest_addr,
          "destipxid",
          Convert.to_string(UI.QueryWidget(Id(:destipxid), :Value))
        )
        Ops.set(Responsefile.map_dest_addr, "complete1", true)
      end

      Convert.to_symbol(ret)
    end

    # ChangeCodePage
    # @return [Array] of codepages (strings)
    def ChangeCodePage(countryCode)
      list_codepageterms = []
      list_codepage = Ops.get(Responsefile.list_codepage, countryCode, [])
      Builtins.y2milestone(
        "Responsefile::ChangeCodePage() countryCode %1  list_codepage %2",
        countryCode,
        list_codepage
      )
      pos = 0
      Builtins.foreach(list_codepage) do |val|
        code = Ops.get_string(
          Responsefile.map_codepage,
          Ops.get(list_codepage, pos, ""),
          ""
        )
        element = Item(Id(pos), code)
        list_codepageterms = Builtins.add(list_codepageterms, element)
        pos = Ops.add(pos, 1)
      end
      Builtins.y2milestone(
        "Responsefile::ChangeCodePage() list_codepageterms %1",
        list_codepageterms
      )
      deep_copy(list_codepageterms)
    end

    # Destination Address dialog
    # @return dialog result
    def LanguageDialog
      Builtins.y2milestone("Responsefile::LanguageDialog() started")
      # Language dialog caption
      caption = _("Language Properties")

      destLanguage = Ops.get_integer(Responsefile.map_dest_lang, "language", 0)
      destCountryCode = Ops.get_integer(
        Responsefile.map_dest_lang,
        "countrycode",
        0
      )
      destCodePage = Ops.get_integer(Responsefile.map_dest_lang, "codepage", 0)
      destKeyboardCode = Ops.get_integer(
        Responsefile.map_dest_lang,
        "keyboardcode",
        0
      )

      list_langterms = []
      pos = 0
      Builtins.foreach(Responsefile.list_language) do |val|
        element = Item(Id(pos), Ops.get(Responsefile.list_language, pos, ""))
        list_langterms = Builtins.add(list_langterms, element)
        pos = Ops.add(pos, 1)
      end
      list_countryterms = []
      pos = 0
      Builtins.foreach(Responsefile.list_countrycode_name) do |val|
        element = Item(
          Id(pos),
          Ops.get(Responsefile.list_countrycode_name, pos, "")
        )
        list_countryterms = Builtins.add(list_countryterms, element)
        pos = Ops.add(pos, 1)
      end
      list_codepageterms = ChangeCodePage(destCountryCode)
      list_keyboardterms = []
      pos = 0
      Builtins.foreach(Responsefile.list_keyboardcode) do |val|
        element = Item(
          Id(pos),
          Ops.get(Responsefile.list_keyboardcode, pos, "")
        )
        list_keyboardterms = Builtins.add(list_keyboardterms, element)
        pos = Ops.add(pos, 1)
      end

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the language and code page information."))),
          VSpacing(1),
          Left(ComboBox(Id(:language), _("Language"), list_langterms)),
          VSpacing(0.8),
          Left(ComboBox(Id(:countrycode), _("Country Code"), list_countryterms)),
          VSpacing(0.8),
          Left(
            ReplacePoint(
              Id(:rp),
              ComboBox(Id(:codepage), _("Code Page"), list_codepageterms)
            )
          ),
          VSpacing(0.8),
          Left(
            ComboBox(Id(:keyboardcode), _("Keyboard Code"), list_keyboardterms)
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "language", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))
      UI.ChangeWidget(Id(:language), :Notify, true)
      UI.ChangeWidget(Id(:countrycode), :Notify, true)

      UI.ChangeWidget(Id(:language), :Value, destLanguage)
      UI.ChangeWidget(Id(:countrycode), :Value, destCountryCode)
      UI.ChangeWidget(Id(:codepage), :Value, destCodePage)
      UI.ChangeWidget(Id(:keyboardcode), :Value, destKeyboardCode)

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          break
        elsif ret == :back
          break
        elsif ret == :language
          destLanguage = Convert.to_integer(
            UI.QueryWidget(Id(:language), :Value)
          )
          Builtins.y2milestone(
            "Responsefile::LanguageDialog() returned 'language %1",
            destLanguage
          )
          destCountryCode = Ops.get_integer(
            Responsefile.list_language_defaults,
            [destLanguage, 1],
            0
          )
          destCodePage = Ops.get_integer(
            Responsefile.list_language_defaults,
            [destLanguage, 2],
            0
          )
          destKeyboardCode = Ops.get_integer(
            Responsefile.list_language_defaults,
            [destLanguage, 3],
            0
          )
          UI.ChangeWidget(Id(:language), :Value, destLanguage)
          UI.ChangeWidget(Id(:countrycode), :Value, destCountryCode)
          list_codepageterms = ChangeCodePage(destCountryCode)
          UI.ReplaceWidget(
            Id(:rp),
            ComboBox(Id(:codepage), _("Code Page"), list_codepageterms)
          )
          UI.ChangeWidget(Id(:codepage), :Value, destCodePage)
          UI.ChangeWidget(Id(:keyboardcode), :Value, destKeyboardCode)
        elsif ret == :countrycode
          destCountryCode = Convert.to_integer(
            UI.QueryWidget(Id(:countrycode), :Value)
          )
          Builtins.y2milestone(
            "Responsefile::LanguageDialog() returned 'countrycode %1",
            destCountryCode
          )
          list_codepageterms = ChangeCodePage(destCountryCode)
          UI.ReplaceWidget(
            Id(:rp),
            ComboBox(Id(:codepage), _("Code Page"), list_codepageterms)
          )
          destCodePage = 0
          UI.ChangeWidget(Id(:codepage), :Value, destCodePage)
        else
          Builtins.y2error(
            "Responsefile::LanguageDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_dest_lang,
          "language",
          UI.QueryWidget(Id(:language), :Value)
        )
        Ops.set(
          Responsefile.map_dest_lang,
          "countrycode",
          UI.QueryWidget(Id(:countrycode), :Value)
        )
        Ops.set(
          Responsefile.map_dest_lang,
          "codepage",
          UI.QueryWidget(Id(:codepage), :Value)
        )
        Ops.set(
          Responsefile.map_dest_lang,
          "keyboardcode",
          UI.QueryWidget(Id(:keyboardcode), :Value)
        )
        Ops.set(Responsefile.map_dest_lang, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # SLP Config dialog
    # @return dialog result
    def SLPConfigDialog
      Builtins.y2milestone("Responsefile::SLPConfigDialog() started")

      caption = _("SLP Properties")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the SLP information."))),
          VSpacing(0.8),
          Left(CheckBox(Id(:skipslp), _("Do not install SLP"))),
          VSpacing(0.8),
          Left(
            TextEntry(
              Id(:daserver1),
              Opt(:hstretch),
              _("DA Server 1 IP Address")
            )
          ),
          VSpacing(0.8),
          Left(
            TextEntry(
              Id(:daserver2),
              Opt(:hstretch),
              _("DA Server 2 IP Address")
            )
          ),
          VSpacing(0.8),
          Left(
            TextEntry(
              Id(:daserver3),
              Opt(:hstretch),
              _("DA Server 3 IP Address")
            )
          ),
          VSpacing(0.8),
          Left(
            TextEntry(Id(:slpscopelist), Opt(:hstretch), _("SLP Scope List"))
          ),
          VSpacing(0.8),
          Left(
            CheckBox(
              Id(:diragent),
              _("Configure server as Directory Agent (DA)")
            )
          ),
          VSpacing(0.8),
          Left(
            CheckBox(
              Id(:multicast),
              _("Enable multicast routing on this server")
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "slpconfig", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))
      UI.ChangeWidget(Id(:skipslp), :Notify, true)

      UI.ChangeWidget(
        Id(:daserver1),
        :Value,
        Ops.get_string(Responsefile.map_dest_slp, "daserver1", "")
      )
      UI.ChangeWidget(
        Id(:daserver2),
        :Value,
        Ops.get_string(Responsefile.map_dest_slp, "daserver2", "")
      )
      UI.ChangeWidget(
        Id(:daserver3),
        :Value,
        Ops.get_string(Responsefile.map_dest_slp, "daserver3", "")
      )
      UI.ChangeWidget(
        Id(:slpscopelist),
        :Value,
        Ops.get_string(Responsefile.map_dest_slp, "slpscopelist", "")
      )
      UI.ChangeWidget(
        Id(:diragent),
        :Value,
        Ops.get_boolean(Responsefile.map_dest_slp, "diragent", false)
      )
      UI.ChangeWidget(
        Id(:multicast),
        :Value,
        Ops.get_boolean(Responsefile.map_dest_slp, "multicast", false)
      )

      if Ops.get_boolean(Responsefile.map_dest_slp, "skipslp", false)
        UI.ChangeWidget(Id(:skipslp), :Value, true)
        UI.ChangeWidget(Id(:daserver1), :Enabled, false)
        UI.ChangeWidget(Id(:daserver2), :Enabled, false)
        UI.ChangeWidget(Id(:daserver3), :Enabled, false)
        UI.ChangeWidget(Id(:slpscopelist), :Enabled, false)
        UI.ChangeWidget(Id(:diragent), :Enabled, false)
        UI.ChangeWidget(Id(:multicast), :Enabled, false)
      else
        UI.ChangeWidget(Id(:skipslp), :Value, false)
        UI.ChangeWidget(Id(:daserver1), :Enabled, true)
        UI.ChangeWidget(Id(:daserver2), :Enabled, true)
        UI.ChangeWidget(Id(:daserver3), :Enabled, true)
        UI.ChangeWidget(Id(:slpscopelist), :Enabled, true)
        UI.ChangeWidget(Id(:diragent), :Enabled, true)
        UI.ChangeWidget(Id(:multicast), :Enabled, true)
      end

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:daserver1), :Value)) != "" &&
              !Responsefile.ValidateIPMsg(
                Convert.to_string(UI.QueryWidget(Id(:daserver1), :Value)),
                _("Invalid IP address.")
              ) ||
              Convert.to_string(UI.QueryWidget(Id(:daserver2), :Value)) != "" &&
                !Responsefile.ValidateIPMsg(
                  Convert.to_string(UI.QueryWidget(Id(:daserver2), :Value)),
                  _("Invalid IP address.")
                ) ||
              Convert.to_string(UI.QueryWidget(Id(:daserver3), :Value)) != "" &&
                !Responsefile.ValidateIPMsg(
                  Convert.to_string(UI.QueryWidget(Id(:daserver3), :Value)),
                  _("Invalid IP address.")
                )
            next
          end
          break
        elsif ret == :back
          break
        elsif ret == :skipslp
          Builtins.y2milestone("Responsefile::SLPConfigDialog() toggle")
          if Convert.to_boolean(UI.QueryWidget(Id(:skipslp), :Value))
            UI.ChangeWidget(Id(:daserver1), :Enabled, false)
            UI.ChangeWidget(Id(:daserver2), :Enabled, false)
            UI.ChangeWidget(Id(:daserver3), :Enabled, false)
            UI.ChangeWidget(Id(:slpscopelist), :Enabled, false)
            UI.ChangeWidget(Id(:diragent), :Enabled, false)
            UI.ChangeWidget(Id(:multicast), :Enabled, false)
          else
            UI.ChangeWidget(Id(:daserver1), :Enabled, true)
            UI.ChangeWidget(Id(:daserver2), :Enabled, true)
            UI.ChangeWidget(Id(:daserver3), :Enabled, true)
            UI.ChangeWidget(Id(:slpscopelist), :Enabled, true)
            UI.ChangeWidget(Id(:diragent), :Enabled, true)
            UI.ChangeWidget(Id(:multicast), :Enabled, true)
          end
        else
          Builtins.y2error(
            "Responsefile::DestinationAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_dest_slp,
          "skipslp",
          Convert.to_boolean(UI.QueryWidget(Id(:skipslp), :Value))
        )
        if Convert.to_boolean(UI.QueryWidget(Id(:skipslp), :Value))
          Ops.set(Responsefile.map_dest_slp, "daserver1", "")
          Ops.set(Responsefile.map_dest_slp, "daserver2", "")
          Ops.set(Responsefile.map_dest_slp, "daserver3", "")
          Ops.set(Responsefile.map_dest_slp, "slpscopelist", "")
          Ops.set(Responsefile.map_dest_slp, "diragent", false)
          Ops.set(Responsefile.map_dest_slp, "multicast", false)
        else
          Ops.set(
            Responsefile.map_dest_slp,
            "daserver1",
            Convert.to_string(UI.QueryWidget(Id(:daserver1), :Value))
          )
          Ops.set(
            Responsefile.map_dest_slp,
            "daserver2",
            Convert.to_string(UI.QueryWidget(Id(:daserver2), :Value))
          )
          Ops.set(
            Responsefile.map_dest_slp,
            "daserver3",
            Convert.to_string(UI.QueryWidget(Id(:daserver3), :Value))
          )
          Ops.set(
            Responsefile.map_dest_slp,
            "slpscopelist",
            Convert.to_string(UI.QueryWidget(Id(:slpscopelist), :Value))
          )
          Ops.set(
            Responsefile.map_dest_slp,
            "diragent",
            Convert.to_boolean(UI.QueryWidget(Id(:diragent), :Value))
          )
          Ops.set(
            Responsefile.map_dest_slp,
            "multicast",
            Convert.to_boolean(UI.QueryWidget(Id(:multicast), :Value))
          )
        end
        Ops.set(Responsefile.map_dest_slp, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # DNS Config dialog
    # @return dialog result
    def DNSConfigDialog
      Builtins.y2milestone("Responsefile::DNSConfigDialog() started")

      caption = _("DNS Properties")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the Domain Name Service information."))),
          VSpacing(0.8),
          Left(
            HBox(
              TextEntry(Id(:hostname), Opt(:hstretch), _("Host Name")),
              VBox(Label(""), Label(".")),
              TextEntry(Id(:domain), Opt(:hstretch), _("Domain"))
            )
          ),
          Left(
            TextEntry(
              Id(:dnserver1),
              Opt(:hstretch),
              _("Name Server 1 IP Address")
            )
          ),
          VSpacing(0.8),
          Left(
            TextEntry(
              Id(:dnserver2),
              Opt(:hstretch),
              _("Name Server 2 IP Address")
            )
          ),
          VSpacing(0.8),
          Left(
            TextEntry(
              Id(:dnserver3),
              Opt(:hstretch),
              _("Name Server 3 IP Address")
            )
          ),
          VSpacing(0.8),
          Left(CheckBox(Id(:verifydns), _("Verify the DNS information")))
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "dnsconfig", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      hostName = Ops.get_string(Responsefile.map_dest_dns, "hostname", "")
      domainName = Ops.get_string(Responsefile.map_dest_dns, "domain", "")
      dnServer1 = Ops.get_string(Responsefile.map_dest_dns, "dnserver1", "")
      dnServer2 = Ops.get_string(Responsefile.map_dest_dns, "dnserver2", "")
      dnServer3 = Ops.get_string(Responsefile.map_dest_dns, "dnserver3", "")
      verifyDNS = Ops.get_boolean(Responsefile.map_dest_dns, "verifydns", false)

      UI.ChangeWidget(Id(:hostname), :Value, hostName)
      UI.ChangeWidget(Id(:domain), :Value, domainName)
      UI.ChangeWidget(Id(:dnserver1), :Value, dnServer1)
      UI.ChangeWidget(Id(:dnserver2), :Value, dnServer2)
      UI.ChangeWidget(Id(:dnserver3), :Value, dnServer3)
      UI.ChangeWidget(Id(:verifydns), :Value, verifyDNS)

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:hostname), :Value)) != "" &&
              !Responsefile.ValidateName(
                Convert.to_string(UI.QueryWidget(Id(:hostname), :Value))
              ) ||
              Convert.to_string(UI.QueryWidget(Id(:dnserver1), :Value)) != "" &&
                !Responsefile.ValidateIPMsg(
                  Convert.to_string(UI.QueryWidget(Id(:dnserver1), :Value)),
                  _("Invalid IP address.")
                ) ||
              Convert.to_string(UI.QueryWidget(Id(:dnserver2), :Value)) != "" &&
                !Responsefile.ValidateIPMsg(
                  Convert.to_string(UI.QueryWidget(Id(:dnserver2), :Value)),
                  _("Invalid IP address.")
                ) ||
              Convert.to_string(UI.QueryWidget(Id(:dnserver3), :Value)) != "" &&
                !Responsefile.ValidateIPMsg(
                  Convert.to_string(UI.QueryWidget(Id(:dnserver3), :Value)),
                  _("Invalid IP address.")
                )
            next
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error(
            "Responsefile::DestinationAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_dest_dns,
          "hostname",
          Convert.to_string(UI.QueryWidget(Id(:hostname), :Value))
        )
        Ops.set(
          Responsefile.map_dest_dns,
          "domain",
          Convert.to_string(UI.QueryWidget(Id(:domain), :Value))
        )
        Ops.set(
          Responsefile.map_dest_dns,
          "dnserver1",
          Convert.to_string(UI.QueryWidget(Id(:dnserver1), :Value))
        )
        Ops.set(
          Responsefile.map_dest_dns,
          "dnserver2",
          Convert.to_string(UI.QueryWidget(Id(:dnserver2), :Value))
        )
        Ops.set(
          Responsefile.map_dest_dns,
          "dnserver3",
          Convert.to_string(UI.QueryWidget(Id(:dnserver3), :Value))
        )
        Ops.set(
          Responsefile.map_dest_dns,
          "verifydns",
          Convert.to_boolean(UI.QueryWidget(Id(:verifydns), :Value))
        )
        Ops.set(Responsefile.map_dest_dns, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # DNS Config dialog
    # @return dialog result
    def TimeZoneConfigDialog
      Builtins.y2milestone("Responsefile::TimeZoneConfigDialog() started")

      caption = _("Time Zone Properties")

      list_timezones = []
      pos = 0
      Builtins.foreach(Responsefile.list_timezones_name) do |val|
        element = Item(
          Id(pos),
          Ops.get(Responsefile.list_timezones_name, pos, "")
        )
        list_timezones = Builtins.add(list_timezones, element)
        pos = Ops.add(pos, 1)
      end

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the Time Zone information."))),
          VSpacing(0.8),
          Left(
            MinWidth(
              50,
              MinHeight(
                10,
                Frame(
                  _("Time Zone"),
                  MarginBox(
                    1.5,
                    0.2,
                    SelectionBox(
                      Id(:timezone),
                      Opt(:notify),
                      "",
                      list_timezones
                    )
                  )
                )
              )
            )
          ),
          VSpacing(0.8),
          Left(
            Frame(
              _("Daylight Saving Time"),
              MarginBox(
                1.5,
                0.2,
                CheckBox(
                  Id(:daylightsaving),
                  _("Allow system to adjust for Daylight Saving Time.")
                )
              )
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "timezone", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      timeZone = Ops.get_integer(Responsefile.map_timezones, "timezone", 6)
      daylightSaving = false
      if Ops.get(Responsefile.list_timezones_dst, timeZone, false)
        daylightSaving = Ops.get_boolean(
          Responsefile.map_timezones,
          "daylightsaving",
          false
        )
        UI.ChangeWidget(Id(:daylightsaving), :Enabled, true)
      else
        UI.ChangeWidget(Id(:daylightsaving), :Enabled, false)
      end

      Builtins.y2milestone(
        "Responsefile::TimeZoneConfigDialog() in timezone %1",
        timeZone
      )
      UI.ChangeWidget(Id(:timezone), :CurrentItem, timeZone)
      UI.ChangeWidget(Id(:daylightsaving), :Value, daylightSaving)

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          break
        elsif ret == :back
          break
        elsif ret == :timezone
          timeZone = Convert.to_integer(
            UI.QueryWidget(Id(:timezone), :CurrentItem)
          )
          Builtins.y2milestone(
            "Responsefile::TimeZoneConfigDialog() in timezone %1",
            timeZone
          )
          if Ops.get(Responsefile.list_timezones_dst, timeZone, false)
            UI.ChangeWidget(Id(:daylightsaving), :Enabled, true)
            UI.ChangeWidget(Id(:daylightsaving), :Value, true)
          else
            UI.ChangeWidget(Id(:daylightsaving), :Enabled, false)
            UI.ChangeWidget(Id(:daylightsaving), :Value, false)
          end
        else
          Builtins.y2error(
            "Responsefile::DestinationAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_timezones,
          "timezone",
          Convert.to_integer(UI.QueryWidget(Id(:timezone), :CurrentItem))
        )
        Ops.set(
          Responsefile.map_timezones,
          "daylightsaving",
          Convert.to_boolean(UI.QueryWidget(Id(:daylightsaving), :Value))
        )
        Ops.set(Responsefile.map_timezones, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # Time Sync Config dialog
    # @return dialog result
    def TimeSyncConfigDialog
      Builtins.y2milestone("Responsefile::TimeSynchConfigDialog() started")

      caption = _("Time Synchronization Properties")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter time synchronization information."))),
          VSpacing(1),
          Left(
            HBox(
              Label(_("Time Server Protocol")),
              HSpacing(1.5),
              RadioButtonGroup(
                Id(:serverprotocol),
                HBox(
                  Left(RadioButton(Id(0), Opt(:notify), _("NTPv3"))),
                  Left(RadioButton(Id(1), Opt(:notify), _("TimeSync")))
                )
              )
            )
          ),
          VSpacing(1),
          Left(
            Frame(
              _("Time server type"),
              MarginBox(
                1.5,
                0.2,
                RadioButtonGroup(
                  Id(:servertype),
                  HBox(
                    VBox(
                      Left(RadioButton(Id(2), _("Single"))),
                      Left(RadioButton(Id(3), _("Reference")))
                    ),
                    VBox(
                      Left(RadioButton(Id(4), _("Primary"))),
                      Left(RadioButton(Id(5), _("Secondary")))
                    )
                  )
                )
              )
            )
          ),
          VSpacing(0.8),
          Left(CheckBox(Id(:timesync), _("Use TIMESYNC Configured Sources"))),
          VSpacing(0.8),
          HBox(
            TextEntry(Id(:timesource1), _("Time Source 1 IP Address")),
            HSpacing(0.8),
            VBox(Label(""), CheckBox(Id(:timesrcntp1), _("NTP")))
          ),
          VSpacing(0.5),
          HBox(
            TextEntry(Id(:timesource2), _("Time Source 2 IP Address")),
            HSpacing(0.8),
            VBox(Label(""), CheckBox(Id(:timesrcntp2), _("NTP")))
          ),
          VSpacing(0.5),
          HBox(
            TextEntry(Id(:timesource3), _("Time Source 3 IP Address")),
            HSpacing(0.8),
            VBox(Label(""), CheckBox(Id(:timesrcntp3), _("NTP")))
          )
        )
      )
      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "timesync", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))
      UI.ChangeWidget(Id(:timesync), :Notify, true)

      UI.ChangeWidget(
        Id(:serverprotocol),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_timesync, "protocol", 0)
      )

      if Ops.get_integer(Responsefile.map_timesync, "protocol", 0) == 1 # mode = TimeSync
        Builtins.y2milestone(
          "Responsefile::TimeZoneConfigDialog() protocol is 1"
        )
        UI.ChangeWidget(Id(:servertype), :Enabled, true)
        UI.ChangeWidget(Id(:timesync), :Enabled, true)
        Builtins.y2milestone(
          "Responsefile::TimeZoneConfigDialog() timesync is %1",
          Ops.get_boolean(Responsefile.map_timesync, "timesync", false)
        )
        UI.ChangeWidget(
          Id(:servertype),
          :CurrentButton,
          Ops.get_integer(Responsefile.map_timesync, "servertype", 0)
        )
        if Ops.get_boolean(Responsefile.map_timesync, "timesync", false)
          Builtins.y2milestone(
            "Responsefile::TimeZoneConfigDialog() timesync is in true"
          )
          UI.ChangeWidget(Id(:timesync), :Value, true)
          UI.ChangeWidget(Id(:timesource1), :Enabled, true)
          UI.ChangeWidget(Id(:timesource2), :Enabled, true)
          UI.ChangeWidget(Id(:timesource3), :Enabled, true)
          UI.ChangeWidget(Id(:timesrcntp1), :Enabled, true)
          UI.ChangeWidget(Id(:timesrcntp2), :Enabled, true)
          UI.ChangeWidget(Id(:timesrcntp3), :Enabled, true)
          UI.ChangeWidget(
            Id(:timesrcntp1),
            :Value,
            Ops.get_boolean(Responsefile.map_timesync, "timesrcntp1", false)
          )
          UI.ChangeWidget(
            Id(:timesrcntp2),
            :Value,
            Ops.get_boolean(Responsefile.map_timesync, "timesrcntp2", false)
          )
          UI.ChangeWidget(
            Id(:timesrcntp3),
            :Value,
            Ops.get_boolean(Responsefile.map_timesync, "timesrcntp3", false)
          )
          UI.ChangeWidget(
            Id(:timesource1),
            :Value,
            Ops.get_string(Responsefile.map_timesync, "timesource1", "")
          )
          UI.ChangeWidget(
            Id(:timesource2),
            :Value,
            Ops.get_string(Responsefile.map_timesync, "timesource2", "")
          )
          UI.ChangeWidget(
            Id(:timesource3),
            :Value,
            Ops.get_string(Responsefile.map_timesync, "timesource3", "")
          )
        else
          Builtins.y2milestone(
            "Responsefile::TimeZoneConfigDialog() timesync is in false"
          )
          UI.ChangeWidget(Id(:timesync), :Value, false)
          UI.ChangeWidget(Id(:timesource1), :Enabled, false)
          UI.ChangeWidget(Id(:timesource2), :Enabled, false)
          UI.ChangeWidget(Id(:timesource3), :Enabled, false)
          UI.ChangeWidget(Id(:timesrcntp1), :Enabled, false)
          UI.ChangeWidget(Id(:timesrcntp2), :Enabled, false)
          UI.ChangeWidget(Id(:timesrcntp3), :Enabled, false)
          UI.ChangeWidget(Id(:timesrcntp1), :Value, false)
          UI.ChangeWidget(Id(:timesrcntp2), :Value, false)
          UI.ChangeWidget(Id(:timesrcntp3), :Value, false)
          UI.ChangeWidget(Id(:timesource1), :Value, "")
          UI.ChangeWidget(Id(:timesource2), :Value, "")
          UI.ChangeWidget(Id(:timesource3), :Value, "")
        end # mode == NTPv3
      else
        Builtins.y2milestone(
          "Responsefile::TimeZoneConfigDialog() protocol is 0"
        )
        UI.ChangeWidget(Id(:servertype), :Enabled, false)
        UI.ChangeWidget(Id(:timesync), :Value, true)
        UI.ChangeWidget(Id(:servertype), :CurrentButton, 0)
        UI.ChangeWidget(Id(:timesync), :Enabled, false)
        UI.ChangeWidget(Id(:timesrcntp1), :Value, true)
        UI.ChangeWidget(Id(:timesrcntp2), :Value, true)
        UI.ChangeWidget(Id(:timesrcntp3), :Value, true)
        UI.ChangeWidget(Id(:timesrcntp1), :Enabled, false)
        UI.ChangeWidget(Id(:timesrcntp2), :Enabled, false)
        UI.ChangeWidget(Id(:timesrcntp3), :Enabled, false)
        UI.ChangeWidget(Id(:timesource1), :Enabled, true)
        UI.ChangeWidget(Id(:timesource2), :Enabled, true)
        UI.ChangeWidget(Id(:timesource3), :Enabled, true)
        UI.ChangeWidget(Id(:timesrcntp1), :Value, true)
        UI.ChangeWidget(Id(:timesrcntp2), :Value, true)
        UI.ChangeWidget(Id(:timesrcntp3), :Value, true)
        UI.ChangeWidget(
          Id(:timesource1),
          :Value,
          Ops.get_string(Responsefile.map_timesync, "timesource1", "")
        )
        UI.ChangeWidget(
          Id(:timesource2),
          :Value,
          Ops.get_string(Responsefile.map_timesync, "timesource2", "")
        )
        UI.ChangeWidget(
          Id(:timesource3),
          :Value,
          Ops.get_string(Responsefile.map_timesync, "timesource3", "")
        )
      end

      ret = nil
      while true
        ret = UI.UserInput

        Builtins.y2milestone(
          "Responsefile::TimeZoneConfigDialog() pressed %1",
          ret
        )
        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_boolean(UI.QueryWidget(Id(:timesync), :Value))
            if Convert.to_string(UI.QueryWidget(Id(:timesource1), :Value)) == "" &&
                Convert.to_string(UI.QueryWidget(Id(:timesource2), :Value)) == "" &&
                Convert.to_string(UI.QueryWidget(Id(:timesource3), :Value)) == ""
              Report.Error(_("Missing a time source."))
              Builtins.y2error("TimeSyncConfigDialog Missing a time source.")
              next
            end
            if Convert.to_string(UI.QueryWidget(Id(:timesource1), :Value)) != "" &&
                !Responsefile.ValidateIPMsg(
                  Convert.to_string(UI.QueryWidget(Id(:timesource1), :Value)),
                  _("Invalid IP address.")
                ) ||
                Convert.to_string(UI.QueryWidget(Id(:timesource2), :Value)) != "" &&
                  !Responsefile.ValidateIPMsg(
                    Convert.to_string(UI.QueryWidget(Id(:timesource2), :Value)),
                    _("Invalid IP address.")
                  ) ||
                Convert.to_string(UI.QueryWidget(Id(:timesource3), :Value)) != "" &&
                  !Responsefile.ValidateIPMsg(
                    Convert.to_string(UI.QueryWidget(Id(:timesource3), :Value)),
                    _("Invalid IP address.")
                  )
              next
            end
          end
          break
        elsif ret == :back
          break
        elsif ret == :timesync
          if Convert.to_boolean(UI.QueryWidget(Id(:timesync), :Value))
            UI.ChangeWidget(Id(:timesource1), :Enabled, true)
            UI.ChangeWidget(Id(:timesource2), :Enabled, true)
            UI.ChangeWidget(Id(:timesource3), :Enabled, true)
            UI.ChangeWidget(Id(:timesrcntp1), :Enabled, true)
            UI.ChangeWidget(Id(:timesrcntp2), :Enabled, true)
            UI.ChangeWidget(Id(:timesrcntp3), :Enabled, true)
          else
            UI.ChangeWidget(Id(:timesource1), :Enabled, false)
            UI.ChangeWidget(Id(:timesource2), :Enabled, false)
            UI.ChangeWidget(Id(:timesource3), :Enabled, false)
            UI.ChangeWidget(Id(:timesrcntp1), :Enabled, false)
            UI.ChangeWidget(Id(:timesrcntp2), :Enabled, false)
            UI.ChangeWidget(Id(:timesrcntp3), :Enabled, false)
            UI.ChangeWidget(Id(:timesource1), :Value, "")
            UI.ChangeWidget(Id(:timesource2), :Value, "")
            UI.ChangeWidget(Id(:timesource3), :Value, "")
          end
        elsif Ops.is_integer?(ret)
          UI.ChangeWidget(Id(:timesource1), :Value, "")
          UI.ChangeWidget(Id(:timesource2), :Value, "")
          UI.ChangeWidget(Id(:timesource3), :Value, "")
          if ret == 0 # mode == NTPv3
            Builtins.y2milestone(
              "Responsefile::TimeZoneConfigDialog() pressed %1",
              ret
            )
            UI.ChangeWidget(Id(:servertype), :Enabled, false)
            UI.ChangeWidget(Id(:servertype), :CurrentButton, 2)
            UI.ChangeWidget(Id(:timesync), :Value, true)
            UI.ChangeWidget(Id(:timesync), :Enabled, false)
            UI.ChangeWidget(Id(:timesrcntp1), :Value, true)
            UI.ChangeWidget(Id(:timesrcntp2), :Value, true)
            UI.ChangeWidget(Id(:timesrcntp3), :Value, true)
            UI.ChangeWidget(Id(:timesrcntp1), :Enabled, false)
            UI.ChangeWidget(Id(:timesrcntp2), :Enabled, false)
            UI.ChangeWidget(Id(:timesrcntp3), :Enabled, false)
            UI.ChangeWidget(Id(:timesource1), :Enabled, true)
            UI.ChangeWidget(Id(:timesource2), :Enabled, true)
            UI.ChangeWidget(Id(:timesource3), :Enabled, true) # mode == TimeSync
          else
            Builtins.y2milestone(
              "Responsefile::TimeZoneConfigDialog() pressed %1",
              ret
            )
            UI.ChangeWidget(Id(:servertype), :Enabled, true)
            UI.ChangeWidget(Id(:timesync), :Enabled, false)
            UI.ChangeWidget(Id(:timesrcntp1), :Value, false)
            UI.ChangeWidget(Id(:timesrcntp2), :Value, false)
            UI.ChangeWidget(Id(:timesrcntp3), :Value, false)

            if Convert.to_boolean(UI.QueryWidget(Id(:timesync), :Value))
              Builtins.y2milestone(
                "Responsefile::TimeZoneConfigDialog() pressed %1  timesync is true",
                ret
              )
              UI.ChangeWidget(Id(:timesource1), :Enabled, true)
              UI.ChangeWidget(Id(:timesource2), :Enabled, true)
              UI.ChangeWidget(Id(:timesource3), :Enabled, true)
              UI.ChangeWidget(Id(:timesrcntp1), :Enabled, true)
              UI.ChangeWidget(Id(:timesrcntp2), :Enabled, true)
              UI.ChangeWidget(Id(:timesrcntp3), :Enabled, true)
            else
              Builtins.y2milestone(
                "Responsefile::TimeZoneConfigDialog() pressed %1  timesync is false",
                ret
              )
              UI.ChangeWidget(Id(:timesource1), :Enabled, false)
              UI.ChangeWidget(Id(:timesource2), :Enabled, false)
              UI.ChangeWidget(Id(:timesource3), :Enabled, false)
              UI.ChangeWidget(Id(:timesrcntp1), :Enabled, false)
              UI.ChangeWidget(Id(:timesrcntp2), :Enabled, false)
              UI.ChangeWidget(Id(:timesrcntp3), :Enabled, false)
            end
          end
        else
          Builtins.y2error(
            "TimeSyncConfigDialog Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_timesync,
          "protocol",
          Convert.to_integer(
            UI.QueryWidget(Id(:serverprotocol), :CurrentButton)
          )
        )
        Ops.set(
          Responsefile.map_timesync,
          "servertype",
          Convert.to_integer(UI.QueryWidget(Id(:servertype), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_timesync,
          "timesync",
          Convert.to_boolean(UI.QueryWidget(Id(:timesync), :Value))
        )
        Ops.set(
          Responsefile.map_timesync,
          "timesource1",
          Convert.to_string(UI.QueryWidget(Id(:timesource1), :Value))
        )
        Ops.set(
          Responsefile.map_timesync,
          "timesource2",
          Convert.to_string(UI.QueryWidget(Id(:timesource2), :Value))
        )
        Ops.set(
          Responsefile.map_timesync,
          "timesource3",
          Convert.to_string(UI.QueryWidget(Id(:timesource3), :Value))
        )
        Ops.set(
          Responsefile.map_timesync,
          "timesrcntp1",
          Convert.to_boolean(UI.QueryWidget(Id(:timesrcntp1), :Value))
        )
        Ops.set(
          Responsefile.map_timesync,
          "timesrcntp2",
          Convert.to_boolean(UI.QueryWidget(Id(:timesrcntp2), :Value))
        )
        Ops.set(
          Responsefile.map_timesync,
          "timesrcntp3",
          Convert.to_boolean(UI.QueryWidget(Id(:timesrcntp3), :Value))
        )
        Ops.set(Responsefile.map_timesync, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # SetParametersDialog
    #
    #
    # @return dialog result
    def SetParametersDialog
      Builtins.y2milestone("Entering SetParametersDialog()")
      Wizard.CreateDialog

      # LDAPDefaultServerDialog caption
      caption = _("Add Set Parameters")

      #set entries in the table
      id_num = 0
      list_params = []

      # Add other servers previously configured
      sl = Ops.get_list(Responsefile.map_install, "setparam", [])
      #list<string> sl = ["set developer option=on", "set bell=on", "set options=true"];

      Builtins.y2milestone("Parameter list = %1", Builtins.tostring(sl))
      Builtins.foreach(sl) do |sd|
        item = Item(Id(id_num), sd)
        list_params = Builtins.add(list_params, item)
        id_num = Ops.add(id_num, 1)
      end
      Builtins.y2milestone("list_params = %1", list_params)

      # LDAP Servers Dialog
      contents = HVCenter(
        VBox(
          HSpacing(Opt(:hstretch), 45),
          Left(Label(_("Set Parameters"))),
          HBox(MinSize(35, 5, SelectionBox(Id(:param_list), "", list_params))),
          # button label
          HBox(
            PushButton(Id(:addBttn), _("&Add")),
            #`PushButton(`id(`editBttn), _("&Edit")),
            PushButton(Id(:delBttn), _("&Delete"))
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "setparams", "FIXME"),
        Label.BackButton,
        Label.NextButton
      )

      ret = nil

      # Begin main dailog processing
      while true
        ret = UI.UserInput
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          Ops.set(Responsefile.map_install, "setparam", sl)
          break
        elsif ret == :back
          break
        elsif ret == :addBttn
          Builtins.y2milestone("In addBttn")
          setparameter = ""
          contents2 = HVSquash(
            VBox(
              HSpacing(Opt(:hstretch), 45),
              TextEntry(Id(:setparam), _("Set Parameter")),
              HBox(
                Left(PushButton(Id(:add), Opt(:default, :key_F3), _("&Add"))),
                Left(PushButton(Id(:cancel), Opt(:key_F4), _("&Cancel")))
              )
            )
          )
          UI.OpenDialog(Opt(:decorated), contents2)
          UI.SetFocus(Id(:setparam))
          while true
            ret2 = UI.UserInput
            if ret2 == :cancel
              UI.CloseDialog
              break
            elsif ret2 == :add
              setparameter = Convert.to_string(
                UI.QueryWidget(Id(:setparam), :Value)
              )
              sl = Builtins.add(sl, setparameter)
              Builtins.y2milestone(
                "Parameter list after add = %1",
                Builtins.tostring(sl)
              )
              UI.CloseDialog
              # Add the new ip to the servers in table list
              new_item = Item(Id(id_num), setparameter)
              Builtins.y2milestone(
                "Adding %1 to list as item %2",
                new_item,
                id_num
              )
              Builtins.y2debug("Adding %1 to list as item %2", new_item, id_num)
              list_params = Builtins.add(list_params, new_item)
              id_num = Ops.add(id_num, 1)
              UI.ChangeWidget(Id(:param_list), :Items, list_params)
              break
            else
              Builtins.y2debug(
                "Invalid value %1 returned from Add SetParameter dialog",
                ret2
              )
              next
            end
          end
        elsif ret == :delBttn
          num = -1
          num = Convert.to_integer(
            UI.QueryWidget(Id(:param_list), :CurrentItem)
          )
          Builtins.y2milestone("Entry to delete num = %1", num)
          next if num == nil
          if Ops.greater_or_equal(num, 0)
            sl = Builtins.remove(sl, num)
            Builtins.y2milestone(
              "Parameter list after delete = %1",
              Builtins.tostring(sl)
            )
            #Recompress the list
            id_num = 0
            new_list = []
            new_list = []
            Builtins.foreach(sl) do |sd|
              item = Item(Id(id_num), sd)
              new_list = Builtins.add(new_list, item)
              id_num = Ops.add(id_num, 1)
            end
            list_params = deep_copy(new_list)
            Builtins.y2milestone("list_params after delete = %1", list_params)
            UI.ChangeWidget(Id(:param_list), :Items, list_params)
            next
          end
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
      Builtins.y2milestone(
        "Responsefile::SetParameterDialog() returning %1",
        ret
      )
      UI.CloseDialog
      deep_copy(ret)
    end

    # Install Settings dialog
    # @return dialog result
    def InstallSettingsDialog
      Builtins.y2milestone("Responsefile::InstallSettingsDialog() started")

      caption = _("Install Settings")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter install settings."))),
          VSpacing(0.8),
          Left(Label(_("Automatic Install Reboot"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:reboot),
            VBox(
              Left(RadioButton(Id(2), _("Yes"))),
              Left(RadioButton(Id(3), _("No")))
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Unsupported Drivers"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:drivers),
            VBox(
              Left(RadioButton(Id(4), _("Yes"))),
              Left(RadioButton(Id(5), _("No")))
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Driver Install Type"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:express),
            VBox(
              Left(RadioButton(Id(6), _("Default"))),
              Left(RadioButton(Id(7), _("Manual")))
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Server Up On Reboot"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:autoexec),
            VBox(
              Left(RadioButton(Id(8), _("Yes"))),
              Left(RadioButton(Id(9), _("No")))
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Server Set Parameters"))),
          VSpacing(0.2),
          Left(PushButton(Id(:setparam), _("Edit")))
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "install", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:reboot),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_install, "reboot", 0)
      )
      UI.ChangeWidget(
        Id(:drivers),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_install, "drivers", 1)
      )
      UI.ChangeWidget(
        Id(:express),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_install, "express", 0)
      )
      if Ops.get_integer(Responsefile.map_select_type, "hardwaretype", 0) == 1
        UI.ChangeWidget(
          Id(:autoexec),
          :CurrentButton,
          Ops.get_integer(Responsefile.map_install, "autoexec", 0)
        )
        UI.ChangeWidget(Id(:autoexec), :Enabled, true)
      else
        UI.ChangeWidget(Id(:autoexec), :CurrentButton, 9)
        UI.ChangeWidget(Id(:autoexec), :Enabled, false)
      end

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next || ret == :back
          break
        elsif ret == :setparam
          ret = SetParametersDialog()
          if ret == :abort || ret == :cancel
            if ReallyAbort()
              break
            else
              next
            end
          else
            next
          end
        else
          Builtins.y2error(
            "Responsefile::DestinationAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_install,
          "reboot",
          Convert.to_integer(UI.QueryWidget(Id(:reboot), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_install,
          "drivers",
          Convert.to_integer(UI.QueryWidget(Id(:drivers), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_install,
          "express",
          Convert.to_integer(UI.QueryWidget(Id(:express), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_install,
          "autoexec",
          Convert.to_integer(UI.QueryWidget(Id(:autoexec), :CurrentButton))
        )
        Ops.set(Responsefile.map_install, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # Storage Configuration dialog
    # @return dialog result
    def StorageConfigDialog
      Builtins.y2milestone("Responsefile::StorageConfigDialog() started")
      # SourceAddr dialog caption
      caption = _("Storage Configuration")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the storage configuration."))),
          VSpacing(1),
          Left(
            MinWidth(
              20,
              IntField(
                Id(:fatpart),
                Opt(:hstretch),
                _("Fat Partition Size (MB)"),
                Ops.get_integer(Responsefile.map_storage, "fatmin", 0),
                Ops.get_integer(Responsefile.map_storage, "fatmax", 0),
                Ops.get_integer(Responsefile.map_storage, "fatpart", 0)
              )
            )
          ),
          VSpacing(1),
          Left(
            MinWidth(
              20,
              IntField(
                Id(:sysvol),
                Opt(:hstretch),
                _("Sys Volume Size (MB)"),
                Ops.get_integer(Responsefile.map_storage, "sysmin", 0),
                Ops.get_integer(Responsefile.map_storage, "sysmax", 0),
                Ops.get_integer(Responsefile.map_storage, "sysvol", 0)
              )
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "storeconfig", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          break
        elsif ret == :back
          break
        else
          Builtins.y2error(
            "Responsefile::DestinationAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_storage,
          "fatpart",
          Convert.to_integer(UI.QueryWidget(Id(:fatpart), :Value))
        )
        Ops.set(
          Responsefile.map_storage,
          "sysvol",
          Convert.to_integer(UI.QueryWidget(Id(:sysvol), :Value))
        )
        if Ops.get_integer(Responsefile.map_storage, "fatpart", 0) !=
            Ops.get_integer(Responsefile.map_storage, "fatdef", 0)
          Ops.set(Responsefile.map_storage, "complete", true)
        end
        if Ops.get_integer(Responsefile.map_storage, "sysvol", 0) !=
            Ops.get_integer(Responsefile.map_storage, "sysdef", 0)
          Ops.set(Responsefile.map_storage, "complete1", true)
        end
      end

      Convert.to_symbol(ret)
    end

    # Upgrade Settings dialog
    # @return dialog result
    def UpgradeSettingsDialog
      Builtins.y2milestone("Responsefile::UpgradeSettingsDialog() started")

      caption = _("Upgrade Settings")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter upgrade settings."))),
          VSpacing(1),
          Left(Label(_("Backup server boot directory"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:backup),
            VBox(
              Left(RadioButton(Id(0), Opt(:notify), _("Yes"))),
              Left(RadioButton(Id(1), Opt(:notify), _("No")))
            )
          ),
          VSpacing(0.6),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:backuploc),
                Opt(:hstretch),
                _("Backup Location Path")
              )
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Automatic Upgrade Reboot"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:reboot),
            VBox(
              Left(RadioButton(Id(2), _("Yes"))),
              Left(RadioButton(Id(3), _("No")))
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Unsupported Drivers"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:drivers),
            VBox(
              Left(RadioButton(Id(4), _("Yes"))),
              Left(RadioButton(Id(5), _("No")))
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Driver Upgrade Type"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:express),
            VBox(
              Left(RadioButton(Id(6), _("Default"))),
              Left(RadioButton(Id(7), _("Manual")))
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Server Up On Reboot"))),
          VSpacing(0.2),
          RadioButtonGroup(
            Id(:autoexec),
            VBox(
              Left(RadioButton(Id(8), _("Yes"))),
              Left(RadioButton(Id(9), _("No")))
            )
          ),
          VSpacing(0.8),
          Left(Label(_("Server Set Parameters"))),
          VSpacing(0.2),
          Left(PushButton(Id(:setparam), _("Edit")))
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "upgrade", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:backup),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_install, "backup", 0)
      )
      UI.ChangeWidget(
        Id(:backuploc),
        :Value,
        Ops.get_string(Responsefile.map_install, "backuploc", "")
      )
      UI.ChangeWidget(
        Id(:reboot),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_install, "reboot", 0)
      )
      UI.ChangeWidget(
        Id(:drivers),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_install, "drivers", 1)
      )
      UI.ChangeWidget(
        Id(:express),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_install, "express", 0)
      )

      if Convert.to_integer(UI.QueryWidget(Id(:backup), :CurrentButton)) == 0
        UI.ChangeWidget(Id(:backuploc), :Enabled, true)
      else
        UI.ChangeWidget(Id(:backuploc), :Enabled, false)
      end
      if Ops.get_integer(Responsefile.map_select_type, "hardwaretype", 0) == 1
        UI.ChangeWidget(
          Id(:autoexec),
          :CurrentButton,
          Ops.get_integer(Responsefile.map_install, "autoexec", 0)
        )
        UI.ChangeWidget(Id(:autoexec), :Enabled, true)
      else
        UI.ChangeWidget(Id(:autoexec), :CurrentButton, 9)
        UI.ChangeWidget(Id(:autoexec), :Enabled, false)
      end

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next || ret == :back
          break
        elsif Ops.is_integer?(ret)
          if ret == 0 # mode == backup
            UI.ChangeWidget(Id(:backuploc), :Enabled, true)
          else
            UI.ChangeWidget(Id(:backuploc), :Enabled, false)
          end
        elsif ret == :setparam
          ret = SetParametersDialog()
          if ret == :abort || ret == :cancel
            if ReallyAbort()
              break
            else
              next
            end
          else
            next
          end
        else
          Builtins.y2error(
            "Responsefile::DestinationAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_install,
          "backup",
          Convert.to_integer(UI.QueryWidget(Id(:backup), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_install,
          "backuploc",
          Convert.to_string(UI.QueryWidget(Id(:backuploc), :Value))
        )
        Ops.set(
          Responsefile.map_install,
          "reboot",
          Convert.to_integer(UI.QueryWidget(Id(:reboot), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_install,
          "drivers",
          Convert.to_integer(UI.QueryWidget(Id(:drivers), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_install,
          "express",
          Convert.to_integer(UI.QueryWidget(Id(:express), :CurrentButton))
        )
        Ops.set(
          Responsefile.map_install,
          "autoexec",
          Convert.to_integer(UI.QueryWidget(Id(:autoexec), :CurrentButton))
        )
        Ops.set(Responsefile.map_install, "complete1", true)
      end

      Convert.to_symbol(ret)
    end


    # Patterns dialog
    # @return dialog result
    def SelectPatternDialog
      Builtins.y2milestone("Responsefile::SelectPatternDialog() started")
      # SelectType dialog caption
      caption = _("Select Pattern")

      vbox_patterns = VBox()
      pos = 0
      Builtins.foreach(Responsefile.list_patterns_name) do |val|
        vbox_patterns = Builtins.add(
          vbox_patterns,
          Left(
            RadioButton(
              Id(pos),
              Opt(:notify),
              Ops.get(Responsefile.list_patterns_name, pos, "")
            )
          )
        )
        if pos == 2
          vbox_patterns = Builtins.add(
            vbox_patterns,
            Left(Label(_("Pre-Configured servers")))
          )
        end
        pos = Ops.add(pos, 1)
      end

      contents = HVCenter(
        VBox(
          Left(Heading(_("Select the pattern to install."))),
          VSpacing(1),
          Left(
            Frame(
              "",
              MarginBox(1.5, 0.2, RadioButtonGroup(Id(:rb), vbox_patterns))
            )
          ),
          RichText(Id(:infoText), "")
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "patterns", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:rb),
        :CurrentButton,
        Ops.get_integer(Responsefile.map_patterns, "pattern", 0)
      )
      UI.ChangeWidget(
        Id(:infoText),
        :Value,
        Ops.get(
          Responsefile.list_patterns_desc,
          Convert.to_integer(UI.QueryWidget(Id(:rb), :CurrentButton)),
          ""
        )
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next || ret == :back
          break
        elsif Ops.is_integer?(ret)
          UI.ChangeWidget(
            Id(:infoText),
            :Value,
            Ops.get(
              Responsefile.list_patterns_desc,
              Convert.to_integer(ret),
              ""
            )
          )
        else
          Builtins.y2error(
            "Responsefile::SelectTypeDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_patterns,
          "pattern",
          Convert.to_integer(UI.QueryWidget(Id(:rb), :CurrentButton))
        )
        Ops.set(Responsefile.map_patterns, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # Products dialog
    # @return dialog result
    def SelectProductsDialog
      Builtins.y2milestone("Responsefile::SelectProductsDialog() started")
      # SelectType dialog caption
      caption = _("Select Products")

      list_prodterms = []
      pos = 0
      Builtins.foreach(Responsefile.list_products_name) do |val|
        element = Item(
          Id(pos),
          Ops.get(Responsefile.list_products_name, pos, ""),
          Ops.get(Responsefile.list_products_select, pos, false)
        )
        if !Ops.get(Responsefile.list_products_nc, pos, false)
          list_prodterms = Builtins.add(list_prodterms, element)
        end
        pos = Ops.add(pos, 1)
      end

      contents = HBox(
        HStretch(),
        VBox(
          VWeight(
            61,
            VBox(
              Left(Heading(_("Select the components to install."))),
              MultiSelectionBox(Id(:prod), Opt(:vstretch), "", list_prodterms)
            )
          ),
          VWeight(15, RichText(Id(:infoText), "")),
          HBox(
            PushButton(Id(:set), _("Set All")),
            PushButton(Id(:clear), _("Clear All"))
          ),
          VStretch()
        ),
        HStretch()
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "products", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.ChangeWidget(Id(:prod), :Notify, true)
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:infoText),
        :Value,
        Ops.get(
          Responsefile.list_products_desc,
          Convert.to_integer(UI.QueryWidget(Id(:prod), :CurrentItem)),
          ""
        )
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          break
        elsif ret == :back
          break
        elsif ret == :prod
          tempint = Convert.to_integer(UI.QueryWidget(Id(:prod), :CurrentItem))
          tempint1 = 0
          templist = Convert.convert(
            UI.QueryWidget(Id(:prod), :SelectedItems),
            :from => "any",
            :to   => "list <integer>"
          )
          if Builtins.contains(templist, tempint)
            templist1 = Ops.get(Responsefile.list_products_depend, tempint, [])
            if Ops.greater_than(Builtins.size(templist1), 0)
              Builtins.foreach(templist1) do |val|
                if !Builtins.contains(templist, val)
                  templist = Builtins.add(templist, val)
                  tempint1 = Ops.add(tempint1, 1)
                end
              end
              if Ops.greater_than(tempint1, 0)
                UI.ChangeWidget(Id(:prod), :SelectedItems, templist)
              end
            end
          end
          UI.ChangeWidget(
            Id(:infoText),
            :Value,
            Ops.get(Responsefile.list_products_desc, tempint, "")
          )
        elsif ret == :set
          set_list = []
          pos2 = 0
          Builtins.foreach(Responsefile.list_products_nc) do |val|
            set_list = Builtins.add(set_list, pos2) if !val
            pos2 = Ops.add(pos2, 1)
          end
          #			y2milestone("Responsefile::SelectTypeDialog() set_list: %1", set_list);
          UI.ChangeWidget(Id(:prod), :SelectedItems, set_list)
        elsif ret == :clear
          UI.ChangeWidget(Id(:prod), :SelectedItems, [])
        else
          Builtins.y2error(
            "Responsefile::SelectTypeDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        pos2 = 0
        Builtins.foreach(Responsefile.list_products_select) do |val|
          Ops.set(Responsefile.list_products_select, pos2, false)
          pos2 = Ops.add(pos2, 1)
        end
        selected_items = Convert.convert(
          UI.QueryWidget(Id(:prod), :SelectedItems),
          :from => "any",
          :to   => "list <integer>"
        )
        Builtins.foreach(selected_items) do |val|
          Ops.set(Responsefile.list_products_select, val, true)
        end
        Ops.set(Responsefile.map_products, "complete", true)
      end

      Convert.to_symbol(ret)
    end

    # NMAS Configuration dialog
    # @return dialog result
    def NMASConfigDialog
      Builtins.y2milestone("Responsefile::NMASConfigDialog() started")
      caption = _("NMAS - Login Method")


      list_nmasterms = []
      pos = 0
      Builtins.foreach(Responsefile.list_nmas_name) do |val|
        element = Item(
          Id(pos),
          Ops.get(Responsefile.list_nmas_name, pos, ""),
          Ops.get(Responsefile.list_nmas_select, pos, false)
        )
        list_nmasterms = Builtins.add(list_nmasterms, element)
        pos = Ops.add(pos, 1)
      end

      contents = HBox(
        HStretch(),
        VBox(
          VWeight(
            30,
            VBox(
              Left(Heading(_("Select the components to install."))),
              MultiSelectionBox(Id(:nmas), Opt(:vstretch), "", list_nmasterms)
            )
          ),
          VWeight(30, RichText(Id(:infoText), "")),
          HBox(
            PushButton(Id(:set), _("Set All")),
            PushButton(Id(:clear), _("Clear All"))
          ),
          VWeight(20, VStretch())
        ),
        HStretch()
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "nmas", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.ChangeWidget(Id(:nmas), :Notify, true)
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:infoText),
        :Value,
        Ops.get(
          Responsefile.list_nmas_desc,
          Convert.to_integer(UI.QueryWidget(Id(:nmas), :CurrentItem)),
          ""
        )
      )

      UI.ChangeWidget(Id(3), :Value, true)
      UI.ChangeWidget(Id(3), :Enabled, false)

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          break
        elsif ret == :back
          break
        elsif ret == :nmas
          tempint = Convert.to_integer(UI.QueryWidget(Id(:nmas), :CurrentItem))
          UI.ChangeWidget(
            Id(:infoText),
            :Value,
            Ops.get(Responsefile.list_nmas_desc, tempint, "")
          )
          if tempint == 3
            fix_list = Convert.convert(
              UI.QueryWidget(Id(:nmas), :SelectedItems),
              :from => "any",
              :to   => "list <integer>"
            )
            fix_list = Builtins.add(fix_list, 3)
            UI.ChangeWidget(Id(:nmas), :SelectedItems, fix_list)
          end
        elsif ret == :set
          set_list = []
          pos2 = 0
          Builtins.foreach(Responsefile.list_nmas_select) do |val|
            set_list = Builtins.add(set_list, pos2)
            pos2 = Ops.add(pos2, 1)
          end
          UI.ChangeWidget(Id(:nmas), :SelectedItems, set_list)
        elsif ret == :clear
          UI.ChangeWidget(Id(:nmas), :SelectedItems, [3])
        else
          Builtins.y2error(
            "Responsefile::NMASConfigurationDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        pos2 = 0
        Builtins.foreach(Responsefile.list_nmas_select) do |val|
          Ops.set(Responsefile.list_nmas_select, pos2, false)
          pos2 = Ops.add(pos2, 1)
        end
        selected_items = Convert.convert(
          UI.QueryWidget(Id(:nmas), :SelectedItems),
          :from => "any",
          :to   => "list <integer>"
        )
        Builtins.foreach(selected_items) do |val|
          Ops.set(Responsefile.list_nmas_select, val, true)
        end
        Ops.set(Responsefile.map_nmas, "complete", true)
        Ops.set(Responsefile.map_nmas, "skip", false)
      end

      Convert.to_symbol(ret)
    end

    # DNS/DHCP Configuration dialog
    # @return dialog result
    def DNS_DHCPConfigDialog
      Builtins.y2milestone("Responsefile::DNS_DHCPConfigDialog() started")
      caption = _("DNS/DHCP Configuration")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the DNS/DHCP configuration information."))),
          VSpacing(1),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:locator),
                Opt(:hstretch),
                _("DNS/DHCP Locator Object eDirectory Context")
              )
            )
          ),
          VSpacing(1),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:group),
                Opt(:hstretch),
                _("DNS/DHCP Group Object eDirectory Context")
              )
            )
          ),
          VSpacing(1),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:zone),
                Opt(:hstretch),
                _("RootSrvr Zone eDirectory Context")
              )
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "dnsdhcp", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      tempstr = ""
      if Ops.get_string(Responsefile.map_dns_dhcp, "locator", "") == ""
        tempstr = Ops.get_string(Responsefile.map_dest_edir, "destsrvctx", "")
      else
        tempstr = Ops.get_string(Responsefile.map_dns_dhcp, "locator", "")
      end
      UI.ChangeWidget(Id(:locator), :Value, tempstr)
      if Ops.get_string(Responsefile.map_dns_dhcp, "group", "") == ""
        tempstr = Ops.get_string(Responsefile.map_dest_edir, "destsrvctx", "")
      else
        tempstr = Ops.get_string(Responsefile.map_dns_dhcp, "group", "")
      end
      UI.ChangeWidget(Id(:group), :Value, tempstr)
      if Ops.get_string(Responsefile.map_dns_dhcp, "zone", "") == ""
        tempstr = Ops.get_string(Responsefile.map_dest_edir, "destsrvctx", "")
      else
        tempstr = Ops.get_string(Responsefile.map_dns_dhcp, "zone", "")
      end
      UI.ChangeWidget(Id(:zone), :Value, tempstr)

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:locator), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:group), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:zone), :Value)) == ""
            Report.Error(_("All entries must be filled in."))
            next
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error(
            "Responsefile::SourceAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_dns_dhcp,
          "locator",
          Convert.to_string(UI.QueryWidget(Id(:locator), :Value))
        )
        Ops.set(
          Responsefile.map_dns_dhcp,
          "group",
          Convert.to_string(UI.QueryWidget(Id(:group), :Value))
        )
        Ops.set(
          Responsefile.map_dns_dhcp,
          "zone",
          Convert.to_string(UI.QueryWidget(Id(:zone), :Value))
        )
        Ops.set(Responsefile.map_dns_dhcp, "complete", true)
        Ops.set(Responsefile.map_dns_dhcp, "skip", false)
      end
      Convert.to_symbol(ret)
    end

    # iFolder Configuration dialog
    # @return dialog result
    def IFolderConfigDialog
      Builtins.y2milestone("Responsefile::IFolderConfigDialog() started")
      caption = _("iFolder Server Configuration")

      contents = HVCenter(
        VBox(
          Left(
            Heading(_("Enter the iFolder Server configuration information."))
          ),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:ldaphost),
                Opt(:hstretch),
                _("LDAP Host Name or IP")
              )
            )
          ),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:ldapport),
                Opt(:hstretch),
                _("LDAP Port - 389 (Clear Text) or 636 (SSL)")
              )
            )
          ),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:ldapcontext),
                Opt(:hstretch),
                _("LDAP Context for Admin")
              )
            )
          ),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:ifolderhost),
                Opt(:hstretch),
                _("iFolder Server Host Name or IP")
              )
            )
          ),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              TextEntry(
                Id(:ifolderadmin),
                Opt(:hstretch),
                _("iFolder Admin Name")
              )
            )
          ),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              TextEntry(Id(:dbpath), Opt(:hstretch), _("User Database Path"))
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "ifolder", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      tempstr = ""
      if Ops.get_string(Responsefile.map_ifolder, "ldaphost", "") == ""
        if Ops.get_string(Responsefile.map_dest_dns, "hostname", "") != "" &&
            Ops.get_string(Responsefile.map_dest_dns, "domain", "") != ""
          tempstr = Ops.add(
            Ops.add(
              Ops.get_string(Responsefile.map_dest_dns, "hostname", ""),
              "."
            ),
            Ops.get_string(Responsefile.map_dest_dns, "domain", "")
          )
        else
          tempstr = ""
        end
      else
        tempstr = Ops.get_string(Responsefile.map_ifolder, "ldaphost", "")
      end
      UI.ChangeWidget(Id(:ldaphost), :Value, tempstr)
      UI.ChangeWidget(
        Id(:ldapport),
        :Value,
        Ops.get_string(Responsefile.map_ifolder, "ldapport", "")
      )
      if Ops.get_string(Responsefile.map_ifolder, "ldacontext", "") == ""
        tempstr = Ops.get_string(Responsefile.map_dest_edir, "destadminctx", "")
      else
        tempstr = Ops.get_string(Responsefile.map_ifolder, "ldacontext", "")
      end
      UI.ChangeWidget(Id(:ldapcontext), :Value, tempstr)
      if Ops.get_string(Responsefile.map_ifolder, "ifolderhost", "") == ""
        if Ops.get_string(Responsefile.map_dest_dns, "hostname", "") != "" &&
            Ops.get_string(Responsefile.map_dest_dns, "domain", "") != ""
          tempstr = Ops.add(
            Ops.add(
              Ops.get_string(Responsefile.map_dest_dns, "hostname", ""),
              "."
            ),
            Ops.get_string(Responsefile.map_dest_dns, "domain", "")
          )
        else
          tempstr = ""
        end
      else
        tempstr = Ops.get_string(Responsefile.map_ifolder, "ifolderhost", "")
      end
      UI.ChangeWidget(Id(:ifolderhost), :Value, tempstr)
      if Ops.get_string(Responsefile.map_ifolder, "ifolderadmin", "") == ""
        tempstr = Ops.get_string(
          Responsefile.map_dest_edir,
          "destadminname",
          ""
        )
      else
        tempstr = Ops.get_string(Responsefile.map_ifolder, "ifolderadmin", "")
      end
      UI.ChangeWidget(Id(:ifolderadmin), :Value, tempstr)
      UI.ChangeWidget(
        Id(:dbpath),
        :Value,
        Ops.get_string(Responsefile.map_ifolder, "dbpath", "")
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:ldaphost), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:ldapport), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:ldapcontext), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:ifolderhost), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:ifolderadmin), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:dbpath), :Value)) == ""
            Report.Error(_("All entries must be filled in."))
            next
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error(
            "Responsefile::SourceAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_ifolder,
          "ldaphost",
          Convert.to_string(UI.QueryWidget(Id(:ldaphost), :Value))
        )
        Ops.set(
          Responsefile.map_ifolder,
          "ldapport",
          Convert.to_string(UI.QueryWidget(Id(:ldapport), :Value))
        )
        Ops.set(
          Responsefile.map_ifolder,
          "ldapcontext",
          Convert.to_string(UI.QueryWidget(Id(:ldapcontext), :Value))
        )
        Ops.set(
          Responsefile.map_ifolder,
          "ifolderhost",
          Convert.to_string(UI.QueryWidget(Id(:ifolderhost), :Value))
        )
        Ops.set(
          Responsefile.map_ifolder,
          "ifolderadmin",
          Convert.to_string(UI.QueryWidget(Id(:ifolderadmin), :Value))
        )
        Ops.set(
          Responsefile.map_ifolder,
          "dbpath",
          Convert.to_string(UI.QueryWidget(Id(:dbpath), :Value))
        )
        Ops.set(Responsefile.map_ifolder, "complete", true)
        Ops.set(Responsefile.map_ifolder, "skip", false)
      end
      Convert.to_symbol(ret)
    end

    # MySQL Database Configuration dialog
    # @return dialog result
    def MySQLConfigDialog
      Builtins.y2milestone("Responsefile::MySQLConfigDialog() started")
      caption = _("MySQL Database Configuration")

      contents = HVCenter(
        VBox(
          Left(Heading(_("Enter the MySQL configuration information."))),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              TextEntry(Id(:datadir), Opt(:hstretch), _("Data Directory"))
            )
          ),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              Password(Id(:password), Opt(:hstretch), _("Root Password"))
            )
          ),
          VSpacing(0.8),
          Left(
            MinWidth(
              35,
              Password(
                Id(:rpassword),
                Opt(:hstretch),
                _("Confirm Root Password")
              )
            )
          ),
          VSpacing(0.8),
          Left(CheckBox(Id(:secure), _("Secure Installation")))
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "mysql", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:datadir),
        :Value,
        Ops.get_string(Responsefile.map_mysql, "datadir", "")
      )
      UI.ChangeWidget(
        Id(:password),
        :Value,
        Ops.get_string(Responsefile.map_mysql, "password", "")
      )
      UI.ChangeWidget(
        Id(:rpassword),
        :Value,
        Ops.get_string(Responsefile.map_mysql, "password", "")
      )
      UI.ChangeWidget(
        Id(:secure),
        :Value,
        Ops.get_boolean(Responsefile.map_mysql, "secure", false)
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:datadir), :Value)) == ""
            Report.Error(_("All entries must be filled in."))
            next
          end
          if !Responsefile.ValidatePasswords(
              Convert.to_string(UI.QueryWidget(Id(:password), :Value)),
              Convert.to_string(UI.QueryWidget(Id(:rpassword), :Value))
            )
            next
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error(
            "Responsefile::SourceAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_mysql,
          "datadir",
          Convert.to_string(UI.QueryWidget(Id(:datadir), :Value))
        )
        Ops.set(
          Responsefile.map_mysql,
          "password",
          Convert.to_string(UI.QueryWidget(Id(:password), :Value))
        )
        Ops.set(
          Responsefile.map_mysql,
          "secure",
          Convert.to_boolean(UI.QueryWidget(Id(:secure), :Value))
        )
        Ops.set(Responsefile.map_mysql, "complete", true)
        Ops.set(Responsefile.map_mysql, "skip", false)
      end
      Convert.to_symbol(ret)
    end

    # exteNd Application Server Configuration dialog
    # @return dialog result
    def ExteNdConfigDialog
      Builtins.y2milestone("Responsefile::ExteNdConfigConfigDialog() started")
      caption = _("exteNd Application Server Configuration")

      contents = HVCenter(
        VBox(
          Left(
            Heading(
              _(
                "Enter the exteNd application server configuration information."
              )
            )
          ),
          VSpacing(1),
          HBox(
            VBox(
              Left(
                MinWidth(
                  35,
                  TextEntry(Id(:adminname), Opt(:hstretch), _("Admin Name"))
                )
              ),
              VSpacing(0.5),
              Left(
                MinWidth(
                  35,
                  Password(Id(:password), Opt(:hstretch), _("Admin Password"))
                )
              ),
              VSpacing(0.5),
              Left(
                MinWidth(
                  35,
                  Password(
                    Id(:rpassword),
                    Opt(:hstretch),
                    _("Confirm Admin Password")
                  )
                )
              ),
              VSpacing(0.5),
              Left(
                MinWidth(
                  35,
                  TextEntry(Id(:httpport), Opt(:hstretch), _("HTTP Port"))
                )
              ),
              VSpacing(0.5),
              Left(
                MinWidth(
                  35,
                  TextEntry(
                    Id(:dbhost),
                    Opt(:hstretch),
                    _("MySQL Database Host")
                  )
                )
              ),
              VSpacing(0.5),
              Left(CheckBox(Id(:restrict), _("Restrict Access")))
            ),
            VBox(
              Left(
                MinWidth(
                  35,
                  TextEntry(Id(:mysqlport), Opt(:hstretch), _("DB Port"))
                )
              ),
              VSpacing(0.5),
              Left(
                MinWidth(
                  35,
                  TextEntry(Id(:dbusername), Opt(:hstretch), _("DB User Name"))
                )
              ),
              VSpacing(0.5),
              Left(
                MinWidth(
                  35,
                  Password(
                    Id(:dbpassword),
                    Opt(:hstretch),
                    _("DB User Password")
                  )
                )
              ),
              VSpacing(0.5),
              Left(
                MinWidth(
                  35,
                  Password(
                    Id(:rdbpassword),
                    Opt(:hstretch),
                    _("Confirm DB User Password")
                  )
                )
              ),
              VSpacing(0.5),
              Left(
                MinWidth(
                  35,
                  TextEntry(Id(:smname), Opt(:hstretch), _("SilverMaster Name"))
                )
              ),
              VSpacing(0.5),
              Left(CheckBox(Id(:executesmi), _("Execute SilverMasterInit")))
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "extend", ""),
        Label.BackButton,
        Label.NextButton
      )
      UI.SetFocus(Id(:next))

      tempstr = ""
      if Ops.get_string(Responsefile.map_extend, "adminname", "") == ""
        tempstr = Ops.get_string(
          Responsefile.map_dest_edir,
          "destadminname",
          ""
        )
      else
        tempstr = Ops.get_string(Responsefile.map_extend, "adminname", "")
      end
      UI.ChangeWidget(Id(:adminname), :Value, tempstr)
      UI.ChangeWidget(
        Id(:password),
        :Value,
        Ops.get_string(Responsefile.map_extend, "password", "")
      )
      UI.ChangeWidget(
        Id(:rpassword),
        :Value,
        Ops.get_string(Responsefile.map_extend, "password", "")
      )
      UI.ChangeWidget(
        Id(:httpport),
        :Value,
        Ops.get_string(Responsefile.map_extend, "httpport", "")
      )
      UI.ChangeWidget(
        Id(:restrict),
        :Value,
        Ops.get_boolean(Responsefile.map_extend, "restrict", false)
      )
      UI.ChangeWidget(
        Id(:dbhost),
        :Value,
        Ops.get_string(Responsefile.map_extend, "dbhost", "")
      )
      UI.ChangeWidget(
        Id(:mysqlport),
        :Value,
        Ops.get_string(Responsefile.map_extend, "mysqlport", "")
      )
      UI.ChangeWidget(
        Id(:dbusername),
        :Value,
        Ops.get_string(Responsefile.map_extend, "dbusername", "")
      )
      UI.ChangeWidget(
        Id(:dbpassword),
        :Value,
        Ops.get_string(Responsefile.map_extend, "dbpassword", "")
      )
      UI.ChangeWidget(
        Id(:rdbpassword),
        :Value,
        Ops.get_string(Responsefile.map_extend, "dbpassword", "")
      )
      UI.ChangeWidget(
        Id(:smname),
        :Value,
        Ops.get_string(Responsefile.map_extend, "smname", "")
      )
      UI.ChangeWidget(
        Id(:executesmi),
        :Value,
        Ops.get_boolean(Responsefile.map_extend, "executesmi", false)
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        # next
        elsif ret == :next
          if Convert.to_string(UI.QueryWidget(Id(:adminname), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:httpport), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:dbhost), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:mysqlport), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:dbusername), :Value)) == "" ||
              Convert.to_string(UI.QueryWidget(Id(:smname), :Value)) == ""
            Report.Error(_("All entries must be filled in."))
            next
          end
          if !Responsefile.ValidatePasswords(
              Convert.to_string(UI.QueryWidget(Id(:password), :Value)),
              Convert.to_string(UI.QueryWidget(Id(:rpassword), :Value))
            )
            next
          end
          if !Responsefile.ValidatePasswords(
              Convert.to_string(UI.QueryWidget(Id(:dbpassword), :Value)),
              Convert.to_string(UI.QueryWidget(Id(:rdbpassword), :Value))
            )
            next
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error(
            "Responsefile::SourceAddrDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      if ret == :next
        Ops.set(
          Responsefile.map_extend,
          "adminname",
          Convert.to_string(UI.QueryWidget(Id(:adminname), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "password",
          Convert.to_string(UI.QueryWidget(Id(:password), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "httpport",
          Convert.to_string(UI.QueryWidget(Id(:httpport), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "restrict",
          Convert.to_boolean(UI.QueryWidget(Id(:restrict), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "dbhost",
          Convert.to_string(UI.QueryWidget(Id(:dbhost), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "mysqlport",
          Convert.to_string(UI.QueryWidget(Id(:mysqlport), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "dbusername",
          Convert.to_string(UI.QueryWidget(Id(:dbusername), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "dbpassword",
          Convert.to_string(UI.QueryWidget(Id(:dbpassword), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "smname",
          Convert.to_string(UI.QueryWidget(Id(:smname), :Value))
        )
        Ops.set(
          Responsefile.map_extend,
          "executesmi",
          Convert.to_boolean(UI.QueryWidget(Id(:executesmi), :Value))
        )
        Ops.set(Responsefile.map_extend, "complete", true)
        Ops.set(Responsefile.map_extend, "skip", false)
      end
      Convert.to_symbol(ret)
    end

    # Save Response File dialog
    # @return dialog result
    def SaveResponseFileDialog
      Builtins.y2milestone("Responsefile::SaveResponseFileDialog() started")
      # SelectType dialog caption
      caption = ""
      caption = _("Save Response File")

      contents = nil
      hardwaretype = Ops.get_integer(
        Responsefile.map_select_type,
        "hardwaretype",
        0
      )
      installtype = Ops.get_integer(
        Responsefile.map_select_type,
        "installtype",
        0
      )
      Builtins.y2milestone(
        "Responsefile::SaveResponseFileDialog() harwaretype %1",
        hardwaretype
      )
      help_type = ""
      dom0_running = false
      #dom0_running = true;  // test code
      if hardwaretype == 1
        help_type = "rfsavephy"
        contents = HVCenter(
          VBox(
            Left(
              MinWidth(
                35,
                TextEntry(Id(:rfname), Opt(:hstretch), _("Response file name"))
              )
            ),
            VSpacing(0.8),
            Left(
              MinWidth(
                45,
                HBox(
                  TextEntry(Id(:tmprfpath), Opt(:hstretch), _("Save directory")),
                  VBox(Label(""), PushButton(Id(:browse), _("Browse")))
                )
              )
            ),
            Left(
              MinWidth(
                35,
                TextEntry(
                  Id(:rfpath),
                  Opt(:hstretch),
                  _("Response file and license install location")
                )
              )
            )
          )
        )
      else
        help_type = "rfsave"
        contents = HVCenter(
          VBox(
            Left(
              MinWidth(
                35,
                TextEntry(Id(:rfname), Opt(:hstretch), _("Response file name"))
              )
            ),
            VSpacing(0.3),
            Left(
              MinWidth(
                45,
                HBox(
                  TextEntry(Id(:tmprfpath), Opt(:hstretch), _("Save directory")),
                  VBox(Label(""), PushButton(Id(:browse), _("Browse")))
                )
              )
            ),
            VSpacing(0.5),
            Left(CheckBox(Id(:launchvm), _("Start the VM Creation wizard"))),
            VSpacing(0.3),
            Left(
              MinWidth(
                35,
                TextEntry(Id(:vmname), Opt(:hstretch), _("Custom VM name"))
              )
            ),
            VSpacing(0.3),
            Left(
              MinWidth(
                45,
                HBox(
                  TextEntry(
                    Id(:srclocpath),
                    Opt(:hstretch),
                    _("Installation Source Location")
                  ),
                  VBox(Label(""), PushButton(Id(:sbrowse), _("Browse")))
                )
              )
            ),
            VSpacing(0.3),
            Left(
              MinWidth(
                45,
                HBox(
                  TextEntry(
                    Id(:destlocpath),
                    Opt(:hstretch),
                    _("Installation Destination Location")
                  ),
                  VBox(Label(""), PushButton(Id(:dbrowse), _("Browse")))
                )
              )
            )
          )
        )
      end

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, help_type, ""),
        Label.BackButton,
        Label.FinishButton
      )
      UI.SetFocus(Id(:next))

      UI.ChangeWidget(
        Id(:rfname),
        :Value,
        Ops.get_string(Responsefile.map_save_rf, "rfname", "")
      )
      UI.ChangeWidget(
        Id(:tmprfpath),
        :Value,
        Ops.get_string(Responsefile.map_save_rf, "tmprfpath", "")
      )
      if hardwaretype == 1
        UI.ChangeWidget(
          Id(:rfpath),
          :Value,
          Ops.get_string(Responsefile.map_save_rf, "rfpath", "")
        )
      else
        dom0_running = true if SCR.Execute(path(".target.bash"), "xm list") == 0
        UI.ChangeWidget(Id(:launchvm), :Notify, true)
        UI.ChangeWidget(
          Id(:launchvm),
          :Value,
          Ops.get_boolean(Responsefile.map_branch_type, "launchvm", false) && dom0_running
        )
        if Ops.get_string(Responsefile.map_save_rf, "vmname", "") == ""
          UI.ChangeWidget(
            Id(:vmname),
            :Value,
            Ops.get_string(Responsefile.map_dest_addr, "destname", "")
          )
        else
          UI.ChangeWidget(
            Id(:vmname),
            :Value,
            Ops.get_string(Responsefile.map_save_rf, "vmname", "")
          )
        end
        UI.ChangeWidget(
          Id(:srclocpath),
          :Value,
          Ops.get_string(Responsefile.map_save_rf, "srclocpath", "")
        )
        UI.ChangeWidget(
          Id(:destlocpath),
          :Value,
          Ops.get_string(Responsefile.map_save_rf, "destlocpath", "")
        )
        if Convert.to_boolean(UI.QueryWidget(Id(:launchvm), :Value))
          if installtype == 4
            UI.ChangeWidget(Id(:srclocpath), :Enabled, false)
            UI.ChangeWidget(Id(:sbrowse), :Enabled, false)
          elsif installtype != 3
            UI.ChangeWidget(Id(:destlocpath), :Enabled, false)
            UI.ChangeWidget(Id(:dbrowse), :Enabled, false)
          end
        else
          UI.ChangeWidget(Id(:vmname), :Enabled, false)
          UI.ChangeWidget(Id(:srclocpath), :Enabled, false)
          UI.ChangeWidget(Id(:sbrowse), :Enabled, false)
          UI.ChangeWidget(Id(:destlocpath), :Enabled, false)
          UI.ChangeWidget(Id(:dbrowse), :Enabled, false)
        end
        UI.ChangeWidget(Id(:launchvm), :Enabled, false) if !dom0_running
      end

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :launchvm
          if Convert.to_boolean(UI.QueryWidget(Id(:launchvm), :Value))
            UI.ChangeWidget(Id(:vmname), :Enabled, true)
            if installtype == 3
              UI.ChangeWidget(Id(:srclocpath), :Enabled, true)
              UI.ChangeWidget(Id(:sbrowse), :Enabled, true)
              UI.ChangeWidget(Id(:destlocpath), :Enabled, true)
              UI.ChangeWidget(Id(:dbrowse), :Enabled, true)
            elsif installtype == 4
              UI.ChangeWidget(Id(:srclocpath), :Enabled, false)
              UI.ChangeWidget(Id(:sbrowse), :Enabled, false)
              UI.ChangeWidget(Id(:destlocpath), :Enabled, true)
              UI.ChangeWidget(Id(:dbrowse), :Enabled, true)
            else
              UI.ChangeWidget(Id(:srclocpath), :Enabled, true)
              UI.ChangeWidget(Id(:sbrowse), :Enabled, true)
              UI.ChangeWidget(Id(:destlocpath), :Enabled, false)
              UI.ChangeWidget(Id(:dbrowse), :Enabled, false)
            end
          else
            UI.ChangeWidget(Id(:vmname), :Enabled, false)
            UI.ChangeWidget(Id(:srclocpath), :Enabled, false)
            UI.ChangeWidget(Id(:sbrowse), :Enabled, false)
            UI.ChangeWidget(Id(:destlocpath), :Enabled, false)
            UI.ChangeWidget(Id(:dbrowse), :Enabled, false)
          end
        elsif ret == :sbrowse
          srcLocPath = Convert.to_string(
            UI.QueryWidget(Id(:srclocpath), :Value)
          )
          srcLocPath = UI.AskForExistingFile(
            srcLocPath,
            "",
            _("Browse Source Path")
          )
          UI.ChangeWidget(Id(:srclocpath), :Value, srcLocPath)
        elsif ret == :dbrowse
          destLocPath = Convert.to_string(
            UI.QueryWidget(Id(:destlocpath), :Value)
          )
          destLocPath = UI.AskForExistingFile(
            destLocPath,
            "",
            _("Browse Destination Path")
          )
          UI.ChangeWidget(Id(:destlocpath), :Value, destLocPath)
        # next
        elsif ret == :next
          tempstring = ""
          if hardwaretype == 1
            if Convert.to_string(UI.QueryWidget(Id(:rfpath), :Value)) == ""
              Report.Error(
                _("The response file and license install location is missing.")
              )
              next
            else
              Ops.set(
                Responsefile.map_save_rf,
                "rfpath",
                Convert.to_string(UI.QueryWidget(Id(:rfpath), :Value))
              )
            end
          else
            Ops.set(
              Responsefile.map_branch_type,
              "launchvm",
              Convert.to_boolean(UI.QueryWidget(Id(:launchvm), :Value))
            )
            if Convert.to_boolean(UI.QueryWidget(Id(:launchvm), :Value))
              pathname = ""
              if installtype == 3 || installtype == 4
                pathname = Convert.to_string(
                  UI.QueryWidget(Id(:destlocpath), :Value)
                )
                if pathname == ""
                  Report.Error(_("The destination location path is missing."))
                  next
                elsif SCR.Execute(
                    path(".target.bash"),
                    Ops.add("test -f ", pathname)
                  ) == 0 ||
                    SCR.Execute(
                      path(".target.bash"),
                      Ops.add("test -b ", pathname)
                    ) == 0
                  Ops.set(
                    Responsefile.map_save_rf,
                    "destlocpath",
                    Convert.to_string(UI.QueryWidget(Id(:destlocpath), :Value))
                  )
                else
                  Report.Error(
                    Builtins.sformat(_("%1 does not exist"), pathname)
                  )
                  next
                end
              end
              if installtype != 4
                pathname = Convert.to_string(
                  UI.QueryWidget(Id(:srclocpath), :Value)
                )
                if pathname == ""
                  Report.Error(_("The source location path is missing."))
                  next
                elsif SCR.Execute(
                    path(".target.bash"),
                    Ops.add("test -f ", pathname)
                  ) == 0 ||
                    SCR.Execute(
                      path(".target.bash"),
                      Ops.add("test -b ", pathname)
                    ) == 0
                  Ops.set(
                    Responsefile.map_save_rf,
                    "srclocpath",
                    Convert.to_string(UI.QueryWidget(Id(:srclocpath), :Value))
                  )
                else
                  Report.Error(
                    Builtins.sformat(_("%1 does not exist"), pathname)
                  )
                  next
                end
              end
            end
          end
          tempstring = Convert.to_string(UI.QueryWidget(Id(:tmprfpath), :Value))
          tempstring = Responsefile.EndWithSlash(tempstring)
          if Responsefile.ValidatePathName(tempstring, true)
            Ops.set(Responsefile.map_save_rf, "tmprfpath", tempstring)
          else
            next
          end
          Ops.set(
            Responsefile.map_save_rf,
            "rfname",
            Convert.to_string(UI.QueryWidget(Id(:rfname), :Value))
          )
          tempstring = Ops.add(
            Ops.get_string(Responsefile.map_save_rf, "tmprfpath", ""),
            Ops.get_string(Responsefile.map_save_rf, "rfname", "")
          )
          next if !Responsefile.ValidatePathName(tempstring, true)
          if Ops.get_boolean(Responsefile.map_branch_type, "launchvm", false) == true
            if Ops.get_boolean(Responsefile.map_license, "visible", false) &&
                !Ops.get_boolean(Responsefile.map_license, "nolicense", false)
              tempstring = Ops.add(
                Ops.get_string(Responsefile.map_save_rf, "tmprfpath", ""),
                "*.nlf"
              )
              if SCR.Execute(
                  path(".target.bash"),
                  Ops.add("test -f ", tempstring)
                ) != 0
                Report.Error(
                  Builtins.sformat(
                    _("A license file is not in the %1 directory."),
                    Ops.get_string(Responsefile.map_save_rf, "tmprfpath", "")
                  )
                )
                next
              end
            end
            vdisk = ""
            cmd = "--os-type oes2nw "
            if Ops.get_integer(Responsefile.map_select_type, "installtype", 0) == 4
              cmd = Ops.add(cmd, "--no-restart ")
            end
            vdisk = Responsefile.getVirtualDiskName(vdisk)
            if installtype == 1 || installtype == 2 || installtype == 3
              cmd = Ops.add(
                cmd,
                Builtins.sformat(
                  "--disk %1,%2 ",
                  Ops.get_string(Responsefile.map_save_rf, "destlocpath", ""),
                  vdisk
                )
              )
            else
              cmd = Ops.add(
                cmd,
                Builtins.sformat(
                  "--disk ,%1,disk,w,%2 ",
                  vdisk,
                  Ops.add(
                    Ops.get_integer(Responsefile.map_storage, "fatpart", 0),
                    Ops.get_integer(Responsefile.map_storage, "sysvol", 0)
                  )
                )
              )
            end
            #					cmd = cmd + sformat("--disk /var/lib/xen/images/oes2nw/disk0,%1,disk,w,%2 ", vdisk, Responsefile::map_storage["fatpart"]:0 + Responsefile::map_storage["sysvol"]:0);
            if installtype != 4
              vdisk = Responsefile.getVirtualDiskName(vdisk)
              cmd = Ops.add(
                cmd,
                Builtins.sformat(
                  "--disk %1,%2,cdrom ",
                  Ops.get_string(Responsefile.map_save_rf, "srclocpath", ""),
                  vdisk
                )
              )
              cmd = Ops.add(cmd, Builtins.sformat("--source dev:/%1 ", vdisk))
            end
            tempstring = Convert.to_string(UI.QueryWidget(Id(:vmname), :Value))
            Ops.set(Responsefile.map_save_rf, "vmname", tempstring)
            if tempstring != ""
              cmd = Ops.add(cmd, Builtins.sformat("--name %1 ", tempstring))
            end
            cmd = Ops.add(
              cmd,
              Builtins.sformat(
                "--os-settings %1",
                Ops.get_string(Responsefile.map_save_rf, "tmprfpath", "")
              )
            )
            if !Ops.get_boolean(Responsefile.map_license, "visible", false) ||
                Ops.get_boolean(Responsefile.map_license, "nolicense", false)
              cmd = Ops.add(
                cmd,
                Ops.get_string(Responsefile.map_save_rf, "rfname", "")
              )
            end
            Ops.set(Responsefile.map_branch_type, "launchcode", cmd)
            ret = :launch
          end
          Ops.set(Responsefile.map_save_rf, "complete", true)
          next if !Responsefile.ExportRFile
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add("chmod 666 ", Responsefile.homebuildpath),
              "default"
            )
          )
          rfstring = Convert.to_string(
            SCR.Read(
              path(".target.string"),
              Ops.add(Responsefile.homebuildpath, "default")
            )
          )
          rfstring = Responsefile.Substitute(rfstring, "\n\n[[]", "\n[")
          rfstring = Responsefile.Substitute(rfstring, "\n[[]", "\n\n[")
          rfstring = Responsefile.Substitute(rfstring, "\n", " \n")

          tempstring = Ops.get_string(Responsefile.map_save_rf, "tmprfpath", "")
          if SCR.Execute(path(".target.bash"), Ops.add("test -d ", tempstring)) != 0
            if Report.AnyQuestion(
                _("New Directory"),
                Ops.add(
                  Ops.add(_("Create new directory?") + " (", tempstring),
                  ")"
                ),
                _("Yes"),
                _("No"),
                :question
              )
              if SCR.Execute(
                  path(".target.bash"),
                  Ops.add("mkdir -p ", tempstring)
                ) != 0
                Report.Error(
                  Builtins.sformat(
                    _("Failed to create directory %1"),
                    tempstring
                  )
                )
                next
              end
            else
              next
            end
          end
          tempstring = Ops.add(
            Ops.get_string(Responsefile.map_save_rf, "tmprfpath", ""),
            Ops.get_string(Responsefile.map_save_rf, "rfname", "")
          )
          if SCR.Execute(path(".target.bash"), Ops.add("test -f ", tempstring)) == 0 &&
              !Report.AnyQuestion(
                _("File Overwrite"),
                Ops.add(
                  Ops.add(_("Overwrite existing file?") + " (", tempstring),
                  ")"
                ),
                _("Yes"),
                _("No"),
                :question
              )
            next
          elsif !SCR.Write(path(".target.string"), tempstring, rfstring)
            Report.Error(Builtins.sformat(_("%1 failed to copy"), tempstring))
            next
          else
            SCR.Execute(path(".target.bash"), Ops.add("chmod 666 ", tempstring))
          end

          if Ops.get_integer(Responsefile.map_select_type, "installtype", 0) == 1 &&
              Ops.get_integer(Responsefile.map_select_type, "hardwaretype", 0) == 1
            ipsentry = Responsefile.CreateIPSString(
              Ops.get_string(Responsefile.map_save_rf, "rfname", ""),
              Ops.get_string(Responsefile.map_save_rf, "rfpath", "")
            )
            upgradestring = Convert.to_string(
              SCR.Read(
                path(".target.string"),
                Ops.add(Responsefile.homebuildpath, "upgrade")
              )
            )
            upgradestring = Ops.add(Ops.add(upgradestring, ipsentry), "\n")
            SCR.Write(
              path(".target.string"),
              Ops.add(Responsefile.homebuildpath, "upgrade"),
              upgradestring
            )
            upgradestring = Convert.to_string(
              SCR.Read(
                path(".target.string"),
                Ops.add(Responsefile.homebuildpath, "upgrade")
              )
            )

            ncfentry = Responsefile.CreateNCFString(
              Ops.get_string(Responsefile.map_save_rf, "rfname", ""),
              Ops.get_string(Responsefile.map_save_rf, "rfpath", "")
            )
            SCR.Write(
              path(".target.string"),
              Ops.add(Responsefile.homebuildpath, "ncffile"),
              Ops.add(ncfentry, "\n")
            )

            tempstring = Ops.add(
              Ops.get_string(Responsefile.map_save_rf, "tmprfpath", ""),
              Responsefile.CreateFileName(
                Ops.get_string(Responsefile.map_save_rf, "rfname", ""),
                "ips"
              )
            )
            if SCR.Execute(
                path(".target.bash"),
                Ops.add(
                  Ops.add(
                    Ops.add("cp -f ", Responsefile.homebuildpath),
                    "upgrade "
                  ),
                  tempstring
                )
              ) != 0
              Report.Error(Builtins.sformat(_("%1 failed to copy"), tempstring))
              next
            else
              SCR.Execute(
                path(".target.bash"),
                Ops.add("chmod 666 ", tempstring)
              )
            end

            tempstring = Ops.add(
              Ops.get_string(Responsefile.map_save_rf, "tmprfpath", ""),
              Responsefile.CreateFileName(
                Ops.get_string(Responsefile.map_save_rf, "rfname", ""),
                "ncf"
              )
            )
            if SCR.Execute(
                path(".target.bash"),
                Ops.add(
                  Ops.add(
                    Ops.add("cp -f ", Responsefile.homebuildpath),
                    "ncffile "
                  ),
                  tempstring
                )
              ) != 0
              Report.Error(Builtins.sformat(_("%1 failed to copy"), tempstring))
              next
            else
              SCR.Execute(
                path(".target.bash"),
                Ops.add("chmod 666 ", tempstring)
              )
            end
          end
          break
        elsif ret == :back
          break
        elsif ret == :browse
          tmprfPath = Convert.to_string(UI.QueryWidget(Id(:tmprfpath), :Value))
          tmprfPath = UI.AskForExistingDirectory(tmprfPath, _("Browse RF Path"))
          UI.ChangeWidget(Id(:tmprfpath), :Value, tmprfPath)
        else
          Builtins.y2error(
            "Responsefile::SelectTypeDialog() Unexpected return code: %1",
            ret
          )
          next
        end
      end
      Convert.to_symbol(ret)
    end

    # Run the VM launch code
    # @return dialog result
    def LaunchDialog
      Builtins.y2milestone("Responsefile::LaunchDialog() started")
      # SelectType dialog caption
      caption = ""
      caption = _("Launch VM Manager")

      contents = nil

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "rfsave", ""),
        Label.BackButton,
        Label.FinishButton
      )
      UI.SetFocus(Id(:next))
      ret = :next
      if Ops.get_boolean(Responsefile.map_branch_type, "launchvm", false) == true
        # get display mode
        display_info = UI.GetDisplayInfo
        textmode = Ops.get_boolean(display_info, "TextMode", true)

        if textmode
          #error ... cannot do this
          Report.Error(_("Cannot execute command in text mode."))
          ret = :abort
        else
          cmd = Ops.add(
            "vm-install ",
            Ops.get_string(Responsefile.map_branch_type, "launchcode", "")
          )
          Builtins.y2milestone(
            "Responsefile::LaunchDialog() launch code: %1",
            cmd
          )
          SCR.Execute(path(".target.bash_background"), cmd)
        end
      end
      Convert.to_symbol(ret)
    end
  end
end
