# encoding: utf-8

# File:	include/novell-responsefile/wizards.ycp
# Package:	Configuration of responsefile
# Summary:	Wizards definitions
# Authors:	Blair Merrell <bmerrell@novell.com>
#
# $Id: wizards.ycp 13879 2004-02-05 11:29:30Z msvec $
module Yast
  module NovellResponsefileWizardsInclude
    def initialize_novell_responsefile_wizards(include_target)
      Yast.import "UI"

      textdomain "novell-responsefile"

      Yast.import "Sequencer"
      Yast.import "Wizard"

      Yast.include include_target, "novell-responsefile/complex.rb"
    end

    # Main workflow of the responsefile configuration
    # @return sequence result
    def MainSequence
      Builtins.y2milestone("MainSequence() started")
      # FIXME: adapt to your needs
      aliases = {
        "select"     => lambda { SelectTypeDialog() },
        "summary"    => lambda { SummaryDialog() },
        "srcaddr"    => lambda { SourceAddrDialog() },
        "srcedir"    => lambda { SourceDirDialog() },
        "destaddr"   => lambda { DestinationAddrDialog() },
        "destedir"   => lambda { DestinationeDirDialog() },
        "license"    => lambda { LicenseDialog() },
        "protocols"  => lambda { ProtocolsDialog() },
        "language"   => lambda { LanguageDialog() },
        "slpconfig"  => lambda { SLPConfigDialog() },
        "dnsconfig"  => lambda { DNSConfigDialog() },
        "timezone"   => lambda { TimeZoneConfigDialog() },
        "timesync"   => lambda { TimeSyncConfigDialog() },
        "install"    => lambda { InstallSettingsDialog() },
        "upgrade"    => lambda { UpgradeSettingsDialog() },
        "storage"    => lambda { StorageConfigDialog() },
        "patterns"   => lambda { SelectPatternDialog() },
        "products"   => lambda { SelectProductsDialog() },
        "nmas"       => lambda { NMASConfigDialog() },
        "dnsdhcp"    => lambda { DNS_DHCPConfigDialog() },
        "ifolder"    => lambda { IFolderConfigDialog() },
        "mysql"      => lambda { MySQLConfigDialog() },
        "extend"     => lambda { ExteNdConfigDialog() },
        "savefile"   => lambda { SaveResponseFileDialog() },
        "launch"     => lambda { LaunchDialog() }
      }

      # FIXME: adapt to your needs
      sequence = {
        "ws_start"   => "select",
        "select"     => { :abort => :abort, :next => "summary" },
        "summary"    => {
          :abort            => :abort,
          :next             => "savefile",
          :back             => "select",
          :srcAddr          => "srcaddr",
          :srceDir          => "srcedir",
          :destAddr         => "destaddr",
          :desteDir         => "destedir",
          :destLicense      => "license",
          :destProt         => "protocols",
          :destLang         => "language",
          :destSLP          => "slpconfig",
          :destDNS          => "dnsconfig",
          :timeZone         => "timezone",
          :timeSync         => "timesync",
          :installSet       => "install",
          :upgradeSet       => "upgrade",
          :storeConfig      => "storage",
          :selectPattern    => "patterns",
          :selectProducts   => "products",
          :nmasConfigure    => "nmas",
          :dnsdhcpConfigure => "dnsdhcp",
          :ifolderConfigure => "ifolder",
          :mysqlConfigure   => "mysql",
          :extendConfigure  => "extend",
        },
        "srcaddr"    => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "srcedir"    => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "destaddr"   => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "destedir"   => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "license"    => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "protocols"  => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "language"   => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "slpconfig"  => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "dnsconfig"  => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "timezone"   => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "timesync"   => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "install"    => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "upgrade"    => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "storage"    => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "patterns"   => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "products"   => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "nmas"       => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "dnsdhcp"    => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "ifolder"    => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "mysql"      => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "extend"     => {
          :abort => :abort,
          :next  => "summary",
          :back  => "summary"
        },
        "savefile"   => {
          :abort  => :abort,
          :next   => :next,
          :back   => "summary",
          :launch => "launch"
        },
        "launch"     => { :abort => :abort, :next => :next }
      }

      ret = Sequencer.Run(aliases, sequence)

      deep_copy(ret)
    end

    # Whole configuration of responsefile
    # @return sequence result
    def ResponsefileSequence
      Builtins.y2milestone("ResponsefileSequence() started")

      aliases = {
        "read"  => [lambda { ReadDialog() }, true],
        "main"  => lambda { MainSequence() },
        "write" => [lambda { WriteDialog() }, true]
      }

      sequence = {
        "ws_start" => "main",
        "read"     => { :abort => :abort, :next => "main" },
        "main"     => { :abort => :abort, :next => :next },
        "write"    => { :abort => :abort, :next => :next }
      }

      Wizard.CreateDialog

      ret = Sequencer.Run(aliases, sequence)

      UI.CloseDialog
      deep_copy(ret)
    end

    # Set to NetWare install Mode
    # @return sequence result
    def CommandHandler(options)
      options = deep_copy(options)
      Builtins.y2milestone("CommandHandler() started")
      Ops.set(Responsefile.map_branch_type, "netware", true)

      ResponsefileSequence()
      true
    end


    # Whole configuration of responsefile but without reading and writing.
    # For use with autoinstallation.
    # @return sequence result
    def ResponsefileAutoSequence
      Builtins.y2milestone("ResponsefileAutoSequence() started")
      # Initialization dialog caption
      caption = _("Responsefile Configuration")
      # Initialization dialog contents
      contents = Label(_("Initializing..."))

      Wizard.CreateDialog
      Wizard.SetContentsButtons(
        caption,
        contents,
        "",
        Label.BackButton,
        Label.NextButton
      )

      ret = MainSequence()

      UI.CloseDialog
      deep_copy(ret)
    end
  end
end
