# encoding: utf-8

# File:	modules/Responsefile.ycp
# Package:	Configuration of responsefile
# Summary:	Responsefile settings, input and output functions
# Authors:	Blair Merrell <bmerrell@novell.com>
#
# $Id: Responsefile.ycp 23442 2005-05-18 07:46:28Z visnov $
#
# Representation of the configuration of responsefile.
# Input and output routines.
require "yast"

module Yast
  class ResponsefileClass < Module
    def main
      textdomain "novell-responsefile"

      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Summary"
      Yast.import "Message"
      Yast.import "Popup"
      Yast.import "IP"

      # Prototypes

      #global string homesrcpath = "/home/bmerrell/source/responsefile/src/";
      #global string homebuildpath = "/home/bmerrell/source/responsefile/build/";
      @homesrcpath = "/opt/novell/responsefile/src/"
      @homebuildpath = "/opt/novell/responsefile/build/"

      #*********************************************************************************************************************************
      #************************** The empty map are initialized in InitializeVariables(); **********************************************

      @map_branch_type = {
        "type"       => 0,
        "netware"    => false,
        "launchvm"   => false,
        "launchcode" => "",
        "launchtype" => ""
      }

      @map_select_type = {
        "complete"     => false,
        "visible"      => false,
        "hardwaretype" => 0,
        "installtype"  => 2,
        "template"     => false,
        "path"         => ""
      }

      @map_src_addr = {}
      @map_src_edir = {}
      @map_dest_addr = {}
      @map_dest_edir = {}
      @map_license = {}
      @map_dest_lang = {}
      @map_dest_slp = {}
      @map_dest_dns = {}
      @map_timezones = {}
      @map_timesync = {}
      @map_install = {}
      @map_storage = {}
      @map_patterns = {}
      @map_products = {}
      @list_products_select = []
      @list_products_nc = []
      @map_nmas = {}
      @list_nmas_select = []
      @map_dns_dhcp = {}
      @map_ifolder = {}
      @map_mysql = {}
      @map_extend = {}
      @map_save_rf = {}

      #*********************************************************************************************************************************

      @list_frame_types = [
        "ETHERNET_802.2",
        "ETHERNET_802.3",
        "ETHERNET_SNAP",
        "ETHERNET_II"
      ]

      @list_timeserver_protocol = ["NTPV3", "TIMESYNC"]

      @list_timeserver_type = ["SINGLE", "REFERENCE", "PRIMARY", "SECONDARY"]

      @list_language = [
        _("English"),
        _("German"),
        _("Italian"),
        _("Spanish"),
        _("French"),
        _("Portuguese"),
        _("Russian")
      ]

      @list_language_en = [
        "English",
        "German",
        "Italian",
        "Spanish",
        "French",
        "Portuguese",
        "Russian"
      ]

      @list_language_number = ["4", "7", "8", "14", "6", "12", "13"]

      # language number, country code, codepage, keyboardcode
      @list_language_defaults = [
        [0, 0, 0, 18],
        [1, 23, 1, 5],
        [2, 13, 1, 6],
        [3, 2, 1, 8],
        [4, 9, 1, 4],
        [5, 26, 1, 1],
        [6, 4, 4, 12]
      ]

      @list_language_defaultx = [
        ["4", "001", "437", "United States"],
        ["7", "049", "850", "Germany"],
        ["8", "039", "850", "Italy"],
        ["14", "003", "850", "Latin America"],
        ["6", "033", "850", "France"],
        ["12", "055", "850", "Brazil"],
        ["13", "007", "866", "Russia"]
      ]

      @list_countrycode = [
        "001",
        "002",
        "003",
        "004",
        "007",
        "027",
        "030",
        "031",
        "032",
        "033",
        "034",
        "036",
        "038",
        "039",
        "040",
        "041",
        "042",
        "043",
        "044",
        "045",
        "046",
        "047",
        "048",
        "049",
        "052",
        "054",
        "055",
        "056",
        "057",
        "058",
        "060",
        "061",
        "064",
        "065",
        "066",
        "081",
        "082",
        "086",
        "088",
        "090",
        "091",
        "099",
        "351",
        "353",
        "354",
        "355",
        "358",
        "359",
        "381",
        "384",
        "386",
        "387",
        "389",
        "421",
        "593",
        "785",
        "852",
        "972"
      ]

      @list_countrycode_name = [
        _("001 (USA)"),
        _("002 (Canadian French-speaking)"),
        _("003 (Latin America)"),
        _("004 (Canada English)"),
        _("007 (Russia)"),
        _("027 (South Africa)"),
        _("030 (Greece)"),
        _("031 (Netherlands)"),
        _("032 (Belgium)"),
        _("033 (France)"),
        _("034 (Spain)"),
        _("036 (Hungary)"),
        _("038 (Yugoslavia)"),
        _("039 (Italy)"),
        _("040 (Romania)"),
        _("041 (Switzerland)"),
        _("042 (Czech Republic)"),
        _("043 (Austria)"),
        _("044 (United Kingdom)"),
        _("045 (Denmark)"),
        _("046 (Sweden)"),
        _("047 (Norway)"),
        _("048 (Poland)"),
        _("049 (Germany)"),
        _("052 (Mexico)"),
        _("054 (Argentina)"),
        _("055 (Brazil)"),
        _("056 (Chile)"),
        _("057 (Colombia)"),
        _("058 (Venezuela)"),
        _("060 (Malaysia)"),
        _("061 (Australia)"),
        _("064 (New Zealand)"),
        _("065 (Singapore)"),
        _("066 (Thailand)"),
        _("081 (Japan)"),
        _("082 (Korea)"),
        _("086 (China/Hong Kong)"),
        _("088 (Taiwan)"),
        _("090 (Turkey)"),
        _("091 (India)"),
        _("099 (Asian English)"),
        _("351 (Portugal)"),
        _("353 (Ireland)"),
        _("354 (Iceland)"),
        _("355 (Albania)"),
        _("358 (Finland)"),
        _("359 (Bulgaria)"),
        _("381 (Serbia/Montenegro)"),
        _("384 (Croatia)"),
        _("386 (Slovenia)"),
        _("387 (Bosnia/Herzegovina)"),
        _("389 (Macedonia)"),
        _("421 (Slovakia)"),
        _("593 (Ecuador)"),
        _("785 (Arabic-speaking)"),
        _("852 (Hong Kong)"),
        _("972 (Hebrew-speaking)")
      ]

      @map_codepage = {
        "437" => _("437 (United States English)"),
        "850" => _("850 (Multilingual)"),
        "852" => _("852 (Slavic)"),
        "855" => _("855 (Cyrillic I)"),
        "857" => _("857 (Turkish)"),
        "860" => _("860 (Portuguese)"),
        "861" => _("861 (Icelandic)"),
        "863" => _("863 (French Canadian)"),
        "865" => _("865 (Nordic)"),
        "866" => _("866 (Russian)"),
        "869" => _("869 (Greek)"),
        "897" => _("897 (Japanese)"),
        "932" => _("932 (Japanese)"),
        "934" => _("934 (Korean)"),
        "936" => _("936 (Chinese)"),
        "949" => _("949 (Korean)"),
        "950" => _("950 (Chinese)"),
        "874" => _("874 (Thailand)"),
        "862" => _("862 (Hebrew)")
      }

      # countrycode to codepage mapping
      @list_codepage = [
        ["437", "850"],
        ["850", "863"],
        ["437", "850"],
        ["850", "863"],
        ["437", "850", "852", "855", "866"],
        ["437", "850"],
        ["850"],
        ["437", "850"],
        ["437", "850"],
        ["437", "850"],
        ["437", "850"],
        ["850", "852"],
        ["850", "852", "855"],
        ["437", "850"],
        ["850", "852"],
        ["437", "850"],
        ["850", "852"],
        ["437", "850"],
        ["437", "850"],
        ["850", "852"],
        ["437", "850"],
        ["850", "865"],
        ["850", "852"],
        ["437", "850"],
        ["437", "850"],
        ["437", "850"],
        ["437", "850"],
        ["437", "850"],
        ["437", "850"],
        ["437", "850"],
        ["437"],
        ["437", "850"],
        ["437", "850"],
        ["437"],
        ["874"],
        ["932"],
        ["949"],
        ["936"],
        ["950"],
        ["850", "857"],
        ["437"],
        ["850", "860"],
        ["437", "850"],
        ["850", "861"],
        ["850", "852"],
        ["437", "850"],
        ["850", "855"],
        ["850", "855"],
        ["850", "852"],
        ["850", "852"],
        ["850", "852"],
        ["850", "855"],
        ["850", "852"],
        ["437", "850"],
        ["850"],
        ["437"],
        ["850", "862"]
      ]

      @list_keyboardcode = [
        _("Belgium"),
        _("Brazil"),
        _("Canadian French"),
        _("Denmark"),
        _("France"),
        _("Germany"),
        _("Italy"),
        _("Japan"),
        _("Latin America"),
        _("Netherlands"),
        _("Norway"),
        _("Portugal"),
        _("Russia"),
        _("Spain"),
        _("Sweden"),
        _("Swiss French"),
        _("Swiss German"),
        _("United Kingdom"),
        _("United States"),
        _("US International")
      ]

      @list_keyboardcode_en = [
        "Belgium",
        "Brazil",
        "Canadian French",
        "Denmark",
        "France",
        "Germany",
        "Italy",
        "Japan",
        "Latin America",
        "Netherlands",
        "Norway",
        "Portugal",
        "Russia",
        "Spain",
        "Sweden",
        "Swiss French",
        "Swiss German",
        "United Kingdom",
        "United States",
        "US International"
      ]

      @list_timezones = [
        "(GMT-12:00) Eniwetok, Kwajalein",
        "(GMT-11:00) Midway Island, Samoa",
        "(GMT-10:00) US Hawaiian-Aleutian Time",
        "(GMT-09:00) US Alaskan Time",
        "(GMT-08:00) US & Canada Pacific Time",
        "(GMT-08:00) Tijuana Pacific Time",
        "(GMT-07:00) US & Canada Mountain Time",
        "(GMT-07:00) Arizona Time",
        "(GMT-06:00) US & Canada Central Time",
        "(GMT-06:00) Saskatchewan",
        "(GMT-06:00) Mexico City",
        "(GMT-05:00) US & Canada Eastern Time",
        "(GMT-05:00) Bogota, Lima",
        "(GMT-04:00) Atlantic Time (Canada)",
        "(GMT-04:00) La Paz, Caracas",
        "(GMT-03:30) Newfoundland Time",
        "(GMT-03:00) Brasilia",
        "(GMT-03:00) Buenos Aires, Georgetown",
        "(GMT-02:00) Mid-Atlantic",
        "(GMT-01:00) West Africa, Azores, Canary Islands",
        "(GMT) Monrovia, Casablanca",
        "(GMT) Great Britain, Ireland, Greenwich Mean Time",
        "(GMT+01:00) Berlin, Rome, Bern, Brussels, Vienna",
        "(GMT+01:00) Sweden, Stockholm",
        "(GMT+01:00) Paris, Madrid",
        "(GMT+01:00) Amsterdam",
        "(GMT+01:00) Prague, Warsaw, Budapest",
        "(GMT+02:00) Israel",
        "(GMT+02:00) Athens, Helsinki, Istanbul",
        "(GMT+02:00) Cairo, Eastern Europe",
        "(GMT+02:00) Harare, Pretoria",
        "(GMT+03:00) Moscow, St. Petersburg, Kazan, Volgograd",
        "(GMT+03:00) Baghdad, Kuwait, Nairobi, Riyadh",
        "(GMT+03:30) Tehran",
        "(GMT+04:00) Abu Dhabi, Muscat, Tbilisi",
        "(GMT+04:30) Kabul",
        "(GMT+05:00) Islamabad, Karachi, Ekaterinburg, Tashkent",
        "(GMT+05:30) India",
        "(GMT+06:00) Almaty, Dhaka",
        "(GMT+07:00) Bangkok, Jakarta, Hanoi",
        "(GMT+08:00) Australia Western Time (WA) Perth",
        "(GMT+08:00) Beijing, Chongqing, Urumqi",
        "(GMT+08:00) Singapore, Taipei",
        "(GMT+09:00) Tokyo, Osaka, Sapporo, Seoul, Yakutsk",
        "(GMT+09:30) Australia Central Time (NT), Darwin",
        "(GMT+09:30) Australia Central Time (SA), Adelaide",
        "(GMT+10:00) Australia Eastern Time (ACT, NSW, Vic)",
        "(GMT+10:00) Australia, Eastern Time (Qld)",
        "(GMT+10:00) Australia Eastern Time (Tas), Hobart",
        "(GMT+10:00) Guam, Port Moresby, Vladivostok",
        "(GMT+11:00) Magadan, Solomon Islands, New Caledonia",
        "(GMT+12:00) New Zealand, Wellington",
        "(GMT+12:00) Fiji, Kamchatka, Marshall Islands"
      ]

      @list_timezones_name = [
        _("(GMT-12:00) Eniwetok, Kwajalein"),
        _("(GMT-11:00) Midway Island, Samoa"),
        _("(GMT-10:00) US Hawaiian-Aleutian Time"),
        _("(GMT-09:00) US Alaskan Time"),
        _("(GMT-08:00) US & Canada Pacific Time"),
        _("(GMT-08:00) Tijuana Pacific Time"),
        _("(GMT-07:00) US & Canada Mountain Time"),
        _("(GMT-07:00) Arizona Time"),
        _("(GMT-06:00) US & Canada Central Time"),
        _("(GMT-06:00) Saskatchewan"),
        _("(GMT-06:00) Mexico City"),
        _("(GMT-05:00) US & Canada Eastern Time"),
        _("(GMT-05:00) Bogota, Lima"),
        _("(GMT-04:00) Atlantic Time (Canada)"),
        _("(GMT-04:00) La Paz, Caracas"),
        _("(GMT-03:30) Newfoundland Time"),
        _("(GMT-03:00) Brasilia"),
        _("(GMT-03:00) Buenos Aires, Georgetown"),
        _("(GMT-02:00) Mid-Atlantic"),
        _("(GMT-01:00) West Africa, Azores, Canary Islands"),
        _("(GMT) Monrovia, Casablanca"),
        _("(GMT) Great Britain, Ireland, Greenwich Mean Time"),
        _("(GMT+01:00) Berlin, Rome, Bern, Brussels, Vienna"),
        _("(GMT+01:00) Sweden, Stockholm"),
        _("(GMT+01:00) Paris, Madrid"),
        _("(GMT+01:00) Amsterdam"),
        _("(GMT+01:00) Prague, Warsaw, Budapest"),
        _("(GMT+02:00) Israel"),
        _("(GMT+02:00) Athens, Helsinki, Istanbul"),
        _("(GMT+02:00) Cairo, Eastern Europe"),
        _("(GMT+02:00) Harare, Pretoria"),
        _("(GMT+03:00) Moscow, St. Petersburg, Kazan, Volgograd"),
        _("(GMT+03:00) Baghdad, Kuwait, Nairobi, Riyadh"),
        _("(GMT+03:30) Tehran"),
        _("(GMT+04:00) Abu Dhabi, Muscat, Tbilisi"),
        _("(GMT+04:30) Kabul"),
        _("(GMT+05:00) Islamabad, Karachi, Ekaterinburg, Tashkent"),
        _("(GMT+05:30) India"),
        _("(GMT+06:00) Almaty, Dhaka"),
        _("(GMT+07:00) Bangkok, Jakarta, Hanoi"),
        _("(GMT+08:00) Australia Western Time (WA) Perth"),
        _("(GMT+08:00) Beijing, Chongqing, Urumqi"),
        _("(GMT+08:00) Singapore, Taipei"),
        _("(GMT+09:00) Tokyo, Osaka, Sapporo, Seoul, Yakutsk"),
        _("(GMT+09:30) Australia Central Time (NT), Darwin"),
        _("(GMT+09:30) Australia Central Time (SA), Adelaide"),
        _("(GMT+10:00) Australia Eastern Time (ACT, NSW, Vic)"),
        _("(GMT+10:00) Australia, Eastern Time (Qld)"),
        _("(GMT+10:00) Australia Eastern Time (Tas), Hobart"),
        _("(GMT+10:00) Guam, Port Moresby, Vladivostok"),
        _("(GMT+11:00) Magadan, Solomon Islands, New Caledonia"),
        _("(GMT+12:00) New Zealand, Wellington"),
        _("(GMT+12:00) Fiji, Kamchatka, Marshall Islands")
      ]

      @list_timezones_abr = [
        "KJT",
        "MIT",
        "HST",
        "AKST",
        "PST",
        "TPST",
        "MST",
        "PNT",
        "CST",
        "CST",
        "CST",
        "EST",
        "IET",
        "AST",
        "PRT",
        "CNT",
        "BET",
        "AGT",
        "MAT",
        "WAT",
        "CGMT",
        "GMT",
        "CET",
        "CET",
        "CET",
        "CET",
        "CET",
        "IST",
        "EET",
        "EET",
        "ART",
        "EAT",
        "EAT",
        "MET",
        "NET",
        "KST",
        "PLT",
        "IST",
        "BST",
        "VST",
        "AWST",
        "CTT",
        "CTT",
        "JST",
        "ACST",
        "ACST",
        "EAST",
        "AEST",
        "AET",
        "AET",
        "SST",
        "NZST",
        "KST"
      ]

      @list_timezones_dst = [
        false,
        false,
        false,
        true,
        true,
        true,
        true,
        false,
        true,
        false,
        true,
        true,
        false,
        true,
        false,
        true,
        true,
        false,
        false,
        false,
        true,
        true,
        true,
        true,
        true,
        true,
        true,
        false,
        true,
        true,
        false,
        true,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        true,
        true,
        false,
        true,
        false,
        false,
        true,
        false
      ]

      @list_patterns = [
        "CustomNetWare",
        "BasicNetWare",
        "PreMigration",
        "DNS_DHCPPattern",
        "exteNd Application Server",
        "LDAPPattern",
        "NetWareAMP",
        "SMSPattern",
        "QuickFinderPattern",
        "NASPattern",
        "PrintPattern",
        "Tomcat Servlet Container",
        "iSCSI Pattern",
        "iFolderPattern",
        "iManagerPattern",
        "Virtual Office Bundle"
      ]

      @list_patterns_name = [
        _("Customized NetWare Server"),
        _("Basic NetWare File Server"),
        _("Pre-Migration Server"),
        _("DNS/DHCP Server"),
        _("exteNd J2EE Web Application Server"),
        _("LDAP Server"),
        _("NetWare AMP (Apache, MySQL, PHP, Perl) Server"),
        _("NetWare Backup Server"),
        _("QuickFinder Server"),
        _("Network Attached Storage (NAS) Server"),
        _("Novell iPrint Server"),
        _("Apache / Tomcat Server"),
        _("iSCSI SAN Storage Server"),
        _("Novell iFolder Storage Services"),
        _("Management Server"),
        _("Virtual Office Server")
      ]

      @list_pattern_products = [
        [
          "Script",
          "Imonitor",
          "Portal",
          "ConsoleOneProducts",
          "6pkLdap",
          "SMS",
          "Novell Certificate Server",
          "NWNMAS",
          "Apache2 Admin Server",
          "Native File Services",
          "Tomcat4",
          "NWEMBOX",
          "PHP",
          "Perl",
          "NSN",
          "Beans",
          "NSBS2Copy",
          "NSBS2Extra",
          "OpenWbem"
        ],
        [
          "NFA Module",
          "6pkLdap",
          "NWNMAS",
          "Novell Certificate Server",
          "Portal",
          "ConsoleOneProducts",
          "Tomcat4",
          "Apache2 Admin Server",
          "Script"
        ],
        [
          "SMS",
          "NWNMAS",
          "Novell Certificate Server",
          "Perl",
          "NSN",
          "PHP",
          "Portal",
          "Native File Services",
          "iMonitor"
        ],
        [
          "Apache2 Webserver",
          "Tomcat4",
          "iManager2.0 Product",
          "DNSDHCP",
          "Tomcat5"
        ],
        [
          "Portal",
          "Apache2 Webserver",
          "Apache2 Admin Server",
          "NFA Module",
          "ConsoleOneProducts",
          "DNSDHCP",
          "eGuide",
          "NWEMBOX",
          "eMFrame",
          "Imonitor",
          "Tomcat5",
          "j2ee",
          "6pkLdap",
          "MySQL",
          "NAAS",
          "Native File Service",
          "NWNMAS",
          "Novell Certificate Server",
          "SMS",
          "Secure Shell",
          "Tomcat4",
          "Wan Connectivity",
          "ipWanMan",
          "Script",
          "iManager2.0 Product"
        ],
        [
          "NFA Module",
          "6pkLdap",
          "NWNMAS",
          "Novell Certificate Server",
          "Portal",
          "ConsoleOneProducts",
          "Tomcat4",
          "Apache2 Admin Server",
          "Script",
          "AP2WEB"
        ],
        [
          "Imonitor",
          "Portal",
          "ConsoleOneProducts",
          "6pkLdap",
          "SMS",
          "Novell Certificate Server",
          "W0 install",
          "NWNMAS",
          "Apache2 Admin Server",
          "Native File Services",
          "Tomcat4",
          "Apache2 Webserver",
          "Secure Shell",
          "MySQL",
          "FTP Server",
          "Script"
        ],
        ["NWNMAS", "iManager2.0 Product", "Apache2 Webserver", "Tomcat5"],
        [
          "Apache2 Webserver",
          "Apache2 Admin Server",
          "6pkLdap",
          "SMS",
          "Novell Certificate Server",
          "QuickFinder Server",
          "Tomcat4",
          "NWNMAS",
          "Portal"
        ],
        [
          "Apache2 Webserver",
          "Apache2 Admin Server",
          "NFA Module",
          "ConsoleOneProducts",
          "iStorage",
          "6pkLdap",
          "NWNMAS",
          "Native File Services",
          "NICIUO",
          "FTP Server",
          "Tomcat4"
        ],
        [
          "Imonitor",
          "Portal",
          "SMS",
          "Novell Certificate Server",
          "NWNMAS",
          "NDPS",
          "Tomcat4",
          "NPS",
          "6pkLdap",
          "Apache2 Admin Server",
          "Apache2 Webserver",
          "iManager2.0 Product",
          "Tomcat5"
        ],
        [
          "Portal",
          "Apache2 Webserver",
          "Apache2 Admin Server",
          "ConsoleOneProducts",
          "eGuide",
          "NWEMBOX",
          "6pkLdap",
          "NWNMAS",
          "Novell Certificate Server",
          "SMS",
          "Secure Shell",
          "Tomcat4",
          "Script",
          "iManager2.0 Product",
          "MySQL",
          "Tomcat5"
        ],
        ["iscsi Module"],
        [
          "Apache2 Admin Server",
          "Apache2 Webserver",
          "NFA Module",
          "ConsoleOneProducts",
          "eMFrame",
          "Imonitor",
          "6pkLdap",
          "NAAS",
          "Native File Services",
          "NWNMAS",
          "Novell Certificate Server",
          "Portal",
          "iManager2.0 Product",
          "SMS",
          "Secure Shell",
          "Tomcat4",
          "Tomcat5",
          "Wan Connectivity",
          "W0 Install",
          "iFolder"
        ],
        [
          "Apache2 Webserver",
          "Apache2 Admin Server",
          "NFA Module",
          "ConsoleOneProducts",
          "eGuide",
          "iManager2.0 Product",
          "Imonitor",
          "6pkLdap",
          "Tomcat4",
          "Tomcat5"
        ],
        [
          "Virtual Office",
          "Apache2 Webserver",
          "Apache2 Admin Server",
          "NFA Module",
          "ConsoleOneProducts",
          "eGuide",
          "NWEMBOX",
          "iStorage",
          "iFolder",
          "Imonitor",
          "6pkLdap",
          "NAAS",
          "NDPS",
          "Native File Services",
          "NWNMAS",
          "NPS",
          "QuickFinder Server",
          "Secure Shell",
          "Tomcat4",
          "Wan Connectivity",
          "eGuideCF"
        ]
      ]

      # Products in the patterns that need configuration (from list_product_params)
      @list_pattern_prod_params = [
        [],
        [13],
        [13],
        [15],
        [15, 21, 19, 13],
        [13],
        [13, 19],
        [13],
        [13],
        [27, 13],
        [13],
        [13, 19],
        [],
        [13, 16],
        [],
        [27, 16, 13]
      ]

      @list_patterns_desc = [
        _(
          "This option allows you to select and install any combination of optional products and components from the component selection screen. Some products are installed\n" +
            "by default and therefore are not displayed on the product selection screen.  They include: eDirectory 8.7, Java Virtual Machine (JVM), Novell International Cryptographic \n" +
            "Infrastructure (NICI), Storage Management Services, NetWare Remote Manager, and Novell Certificate Server."
        ),
        _(
          "This option installs a basic NetWare operating system, which includes an Apache 2 Admin Server and Tomcat 4 Servlet Container, in addition to eDirectory 8.7, Java \n" +
            "Virtual Machine (JVM), Novell International Cryptographic Infrastructure (NICI), Storage Management Services, NetWare Remote Manager, and Novell Certificate Server, \n" +
            "which are installed by default on every NetWare 6.5 server. For a complete list of all products installed with this pattern, see, \"Installing a Basic NetWare File Server\" \n" +
            "in the NetWare 6.5 Installation and Overview Guide. If you decide you want to include additional products on this server after the basic installation, you can perform \n" +
            "a post-installation product install.  For instructions on this process, see, \"Installing Additional Products\" in the NetWare 6.5 Installation and Overview Guide."
        ),
        _(
          "This option installs a NetWare server that an existing server will be migrated to at a later time using the NetWare Migration Wizard. The NetWare Migration Wizard \n" +
            "is part of the Novell Server Consolidation and Migration Toolkit. The Migration Wizard connects to servers involved in a migration using one of the following two \n" +
            "methods: SLP, or the sys:etchosts file. In order to successfully complete a migration, either the sys:etchosts file on the Pre-Migration Server must contain \n" +
            "the IP address and server name of the source server involved in the migration, or SLP must be configured on the Pre-Migration Server. For information on configuring \n" +
            "SLP, see \"Installing a Pre-Migration Server\" in the NetWare 6.5 Installation and Overview Guide."
        ),
        _(
          "The Novell DNS/DHCP Server integrates the Domain Name System (DNS) and Dynamic Host Configuration Protocol (DHCP) with the NetIQ eDirectory. This provides additional \n" +
            "value for enterprise deployment, such as centralized administration and management of DNS/DHCP servers across the enterprise and DNS Zone/DHCP subnet replication via \n" +
            "NetIQ eDirectory. The Novell DNS/DHCP Server installation extends the schema for DNS/DHCP and creates a dedicated DNS/DHCP server by copying only the files needed \n" +
            "for a DNS/DHCP server."
        ),
        _(
          "This selection installs a highly-optimized configuration of Novell exteNd Application Server 5.0 (Enterprise Edition). With this comprehensive J2EE server, you can \n" +
            "deploy enterprise-class Web applications that take advantage of the full Java 2 Enterprise Edition standard: JavaServer Pages (JSP pages), Enterprise JavaBeans (EJBs), \n" +
            "and all other J2EE 1.3 components and technologies. The Novell exteNd Application Server provides high performance, scalability, and reliability; support for rapid \n" +
            "application development; application deployment facilities; and server management facilities. This install also includes MySQL, Apache, and other supporting software.\n" +
            "RELATED CLIENT SOFTWARE  To help you work with the application server, the Novell Clients Software CD includes a couple of exteNd products you can install and run on \n" +
            "your Windows machines:\n" +
            " * Novell exteNd Workbench -- the integrated development environment (IDE) designed specifically to help you build Web Services and J2EE applications, then deploy them to \n" +
            "   your J2EE server\n" +
            " * Novell exteNd Application Server Clients -- tools for deployment, application server administration, and J2EE application client runtime"
        ),
        _(
          "Lightweight Directory Access Protocol (LDAP) Services for NetIQ eDirectory is a server application that lets LDAP clients (for example, Netscape Communicator, Internet \n" +
            "Explorer, or the Novell Import Conversion Export utility) access information stored in eDirectory. LDAP Services includes access \n" +
            "to the following eDirectory features through LDAP: provisioning, account management, authentication, authorization, identity management, notification, reporting, \n" +
            "qualification, and segmentation. The LDAP Server pattern includes the following components:\n" +
            " * iMonitor\n" +
            " * NetWare Remote Manager\n" +
            " * ConsoleOne\n" +
            " * LDAP\n" +
            " * Storage Management Services\n" +
            " * Novell Certificate Server\n" +
            " * Novell Modular Authentication Service (NMAS)\n" +
            " * Apache administration server\n" +
            " * Native File Services\n" +
            " * Tomcat\n" +
            " * Scripting\n" +
            " * eDirectory Management Toolbox (eMBox)\n" +
            " * Native File Access"
        ),
        _(
          "NetWare AMP (Apache, MySQL, PHP, and Perl) lets you host Web-based database applications on your NetWare 6.5 server.  Web database applications are \n" +
            "available from the open-source community. They can be downloaded from the World Wide Web and easily deployed to your NetWare AMP server. Or, if you are familiar \n" +
            "with the structured query language (SQL) and PHP or Perl scripting languages, you can develop and host your own Web-based database applications. NetWare AMP includes \n" +
            "the following components:\n" +
            " * Apache Web server: The leading Web server on the Internet today.\n" +
            " * MySQL: A fast, easy-to-use, relational database management system.\n" +
            " * phpMyAdmin: A utility for managing MySQL from a Web browser.\n" +
            " * PHP/Perl scripting engines: Engines for processing PHP or Perl scripts.\n" +
            " * Apache Manager: A browser-based utility that lets you manage multiple Apache Web servers as an eDirectory object, regardless of the platform they are running on."
        ),
        _(
          "The NetWare(r) Backup Server provides the infrastructure for backup and restore services on the NetWare platform. It allows you to back up targets such as the file \n" +
            "system, Novell(r) eDirectory (tm), and GroupWise(r) to removable tape media for off-site storage. The NetWare Backup Server has been designed to scale with optimized hardware \n" +
            "configurations."
        ),
        _(
          "QuickFinder Server lets you add high-end search capabilities to any Web site, anywhere on the World Wide Web or on a company intranet.Search millions of documents \n" +
            "on public and private Web sites and remote File Systems while preserving individual users' file access rights.  Now available for both NetWare and Linux, version 4.0 \n" +
            "is better than ever at delivering the most relevant information to your desktop. Using QuickFinder Manager, a browser-based administration utility, you can control \n" +
            "almost any aspect of your search experience including the Web sites and File Servers to index, the document types to include, artificial relevance adjustments, \n" +
            "rights-based search results, speller suggestions, search term highlighting, limited inclusion of \"off site\" links, and many other powerful search features. \n" +
            "You can even host search services for hundreds of business partners and clients all from a single machine."
        ),
        _(
          "Network Attached Storage Server solution gives you multiple file protocol storage for your network.  The protocols included support Windows Networking (CIFS), \n" +
            "Novell Networking (NCP), Unix/Linux Networking (NFS), Web Storage (HTTP, HTTPs, FTP, and  WebDAV). NetWare 6.5 is able to drop in and \n" +
            "integrate into any of these environments seemlessly, as well as bridge these environments together."
        ),
        _(
          "This selection installs and creates an iPrint server, Novell's recommended print solution. iPrint provides you with global access to printers using Internet \n" +
            "connections, flexible print deployment configurations, and secure printing. Using a Web browser, users can install printers through the iPrint Web page or \n" +
            "custom maps that you create using iPrint Map Designer. Files required to run iPrint, including the Apache Web Server, are copied to the server."
        ),
        _(
          "This pattern installs the Jakarta-Tomcat Servlet Container 4.1.18 and Apache 2.0.45 http server for use in hosting dynamic, application-driven Web sites. Once \n" +
            "installed, you can deploy and host servlets and Java Server Pages (JSPs). Novell LDAP Command Beans are also included to provide a broad range of eDirectory \n" +
            "authentication and identity mechanisms that can be customized for use in setting up secure browser-based access to directory information. If you require more \n" +
            "functionality than you can get from servlets and JSPs, select the pre-configured exteNd J2EE Web Application Server deployment. For additional development and \n" +
            "deployment power, install the Novell exteNd Workbench IDE from the NetWare 6.5 Client CD that can be used with Tomcat."
        ),
        _(
          "iSCSI is an industry standard that lets you build low-cost, fully functional Storage Area Networks (SANs) using commodity LAN (Ethernet) hardware. (Improved storage \n" +
            "system performance is possible with Gigabit Ethernet hardware.) This patterned deployment creates an iSCSI Storage Server (also known as an iSCSI Target) out of a \n" +
            "NetWare server. The NetWare iSCSI Storage Server includes LDAP access control so you can limit the servers on the iSCSI SAN that can access the storage on the iSCSI \n" +
            "Storage Server. With iSCSI, existing direct-attached or SAN-attached servers can be redeployed as iSCSI Storage Servers. This lets you improve the manageability and \n" +
            "deployment of your storage while also adding high availability with Novell Cluster Services, which Novell's iSCSI Storage Server fully supports."
        ),
        _(
          "Novell iFolder is a Net services software solution that allows your users' local data files to follow them everywhere--online, offline, all the time--across \n" +
            "multiple workstations, your corporate network and the Internet. It provides a simple, convenient and secure way to access, back up, and protect the local data \n" +
            "files of your mobile professionals. With iFolder, each user has a single virtual work folder that provides convenient and secure access to the most recent version \n" +
            "of local documents. All the user needs is an active network or Internet connection and the iFolder client, a Web browser, or Novell NetDrive. The iFolder server helps \n" +
            "guard against local data loss by automatically backing up the user's local files to an iFolder server, making them available for regularly scheduled data backup. When \n" +
            "the user logs in to an iFolder server from another workstation, the iFolder client synchronizes the files by downloading the new and changed files."
        ),
        _(
          "Installs Novell iManager 2.5 and Novell ConsoleOne 1.3.6 network administration software.  Both products together provide a complete management solution for your \n" +
            "NetWare 6.5 server environment. Novell iManager 2.5 is a web-based network administration console used to manage Novell products and services on NetWare 6.5. Novell \n" +
            "ConsoleOne 1.3.6 is the latest version of Novell's Java-based, GUI network management application."
        ),
        _(
          "The NetWare 6.5 Virtual Office greatly improves end-user productivity by introducing self-help measures and enhanced collaboration that create a work experience that \n" +
            "follows the user.  End users create the Virtual Teams that maximize group productivity through: sharing, organizing, and finding information and ideas within their \n" +
            "virtual communities.\n" +
            "Productivity components of Virtual Office:\n" +
            "* Novell iFolder\n" +
            "* iPrint\n" +
            "* Novell eGuide\n" +
            "* Team Folders\n" +
            "* Web Mail"
        )
      ]

      #                                    0                           1         2          3                      4         5                   6        7
      #           8                    9      10           11                   12         13       14                     15        16        17        18       19
      #           20        21     22             23         24           25      26         27    28     29    30        31                 32
      #           33                    34
      @list_products = [
        "Novell Certificate Server",
        "6pkLdap",
        "Imonitor",
        "Apache2 Admin Server",
        "Tomcat4",
        "Apache2 Webserver",
        "Portal",
        "SMS",
        "ConsoleOneProducts",
        "NDPS",
        "FTP Server",
        "QuickFinder Server",
        "ipWanMan",
        "NWNMAS",
        "Native File Services",
        "DNSDHCP",
        "iFolder",
        "NWEMBOX",
        "NWSNMP",
        "MySQL",
        "Tomcat5",
        "j2ee",
        "iSCSI Module",
        "OpenWbem",
        "NFA Module",
        "Beans",
        "iStorage",
        "NSN",
        "Perl",
        "PHP",
        "OpenSSH",
        "Wan Connectivity",
        "eGuide",
        "iManager2.5 Product",
        "Virtual Office"
      ]

      @list_products_depend = [
        [],
        [],
        [],
        [],
        [],
        [],
        [],
        [],
        [],
        [5],
        [],
        [5],
        [],
        [],
        [],
        [],
        [5],
        [],
        [],
        [],
        [],
        [5, 19],
        [],
        [],
        [],
        [],
        [5],
        [],
        [],
        [],
        [],
        [],
        [5],
        [5, 20],
        [5]
      ]

      @list_products_dep_name = [
        [],
        [],
        [],
        [],
        [],
        [],
        [],
        [],
        [],
        ["Apache2 Webserver"],
        [],
        ["Apache2 Webserver"],
        [],
        [],
        [],
        [],
        ["Apache2 Webserver"],
        [],
        [],
        [],
        [],
        ["Apache2 Webserver", "MySQL"],
        [],
        [],
        [],
        [],
        ["Apache2 Webserver"],
        [],
        [],
        [],
        [],
        [],
        ["Apache2 Webserver"],
        ["Apache2 Webserver"],
        ["Apache2 Webserver"]
      ]

      @list_products_params = [
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        true,
        false,
        true,
        true,
        false,
        false,
        true,
        false,
        true,
        false,
        false,
        false,
        false,
        true,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false
      ]

      @list_products_name = [
        _("Novell Certificate Server v2.76"),
        _("LDAP Services"),
        _("NDS iMonitor Services"),
        _("Apache2 Admin Server"),
        _("Tomcat 4 Admin Instance"),
        _("Apache2 Web Server and Tomcat 4 Servlet Container"),
        _("NetWare Remote Manager (NRM)"),
        _("Storage Management Services (SMS)"),
        _("ConsoleOne 1.3.6e "),
        _("iPrint"),
        _("NetWare FTP Server"),
        _("QuickFinder Server"),
        _("WAN Traffic Manager Services"),
        _("Novell Modular Authentication Service"),
        _("Novell Native File Access"),
        _("Novell DNS/DHCP Services"),
        _("Novell iFolder Storage Services"),
        _("eDirectory Management Utilities Toolbox"),
        _("eDirectory SNMP Subagent"),
        _("MySQL "),
        _("Tomcat 5 Servlet Container"),
        _("exteNd Application Server"),
        _("iSCSI Target"),
        _("OpenWBEM"),
        _("Refresh Native File Access Login Methods"),
        _("Beans for Novell Services"),
        _("NSN"),
        _("Perl"),
        _("PHP"),
        _("OpenSSH"),
        _("WAN Connectivity"),
        _("eGuide"),
        _("Novell iManager 2.7"),
        _("Novell Virtual Office")
      ]

      @list_products_desc = [
        _("Novell Certificate Server v2.76"),
        _(
          "Lightweight Directory Access Protocol (LDAP) Services for NetIQ eDirectory is a server application that lets LDAP clients such as Netscape Communicator, \n" +
            "Internet Explorer, or the Novell Import Conversion Export utility access information stored in eDirectory. LDAP Services provides access to the following \n" +
            "eDirectory features through LDAP: provisioning, account management, authentication, authorization, identity management, notification, reporting, \n" +
            "qualification, and segmentation."
        ),
        _(
          "iMonitor provides cross-platform monitoring and diagnostic capability to all servers in your NetIQ eDirectory tree. This utility lets you monitor your \n" +
            "servers from any location on your network where a Web browser is available. You can look at the eDirectory environment in depth on a partition, replica, or \n" +
            "server basis. You can also examine what tasks are taking place, when they are happening, what their results are, and how long they are taking."
        ),
        _(
          "This selection installs an administration instance of the Apache Web server. Apache is used on NetWare in two ways: as an administration server (Apache2 \n" +
            "Admin Server) and as a dedicated Web server (Apache2 Web Server). The Apache2 Admin Server is installed by default and runs in protected address space on your \n" +
            "NetWare 6.5 server."
        ),
        _(
          "Installs Tomcat 4 which support both servlets and Java Server Pages, used primarily for web-based administration of NetWare."
        ),
        _(
          "This selection installs \n" +
            "the Apache Web server and the Jakarta-Tomcat Servlet Container 4.1 for your own use as a dedicated Web hosting solution. Tomcat is used for running Web servlets \n" +
            "and Java Server Pages (JSPs)."
        ),
        _(
          "NRM is a Web-based utility for accessing and managing one or more NetWare servers from any location through a standard Web browser. NRM includes secure access \nto the server console screens and file system, server health monitoring, diagnostic tools, and powerful inventory reporting features."
        ),
        _(
          "The Storage Management Services (SMS) provides the infrastructure for  backup and restore services on the NetWare platform. It allows you to  back up targets \nsuch as the file system, eDirectory, and GroupWise  to removable tape media for off-site storage."
        ),
        _("ConsoleOne application and snapin installation"),
        _(
          "iPrint provides you with global access to printers using Internet connections, flexible print deployment configurations, and secure printing. Using a Web \nbrowser, users can install printers through the iPrint Web page or custom maps that you create using iPrint Map Designer."
        ),
        _(
          "NetWare(r) FTP Server software provides file transfer protocol service for transferring files to and from NetWare volumes. It also supports secure login. \nYou can perform file transfers from and to any NetWare server in the same Novell(r) eDirectory? tree."
        ),
        _(
          "QuickFinder Server adds sophisticated search functionality to any Web site, anywhere on the World Wide Web or on a company intranet.  Search millions of \n" +
            "documents in milliseconds (obeying user file-access rights).  Spotlight the most relevant results.  Include content from partners' and clients' web sites.  \n" +
            "Note: If upgrading from Web Search, all indexes will need to be regenerated."
        ),
        _(
          "WAN Traffic Manager lets you control WAN traffic by applying policies to the use of WAN links by network services, including NDS."
        ),
        _(
          "Novell Modular Authentication Service (NMAS) helps you protect the information on your network by providing additional ways of authenticating to eDirectory, \nthereby ensuring that the people accessing your network resources are who they say they are."
        ),
        _("Novell Native File Access."),
        _(
          "The Novell DNS/DHCP Server integrates the Domain Name System (DNS) and Dynamic Host Configuration Protocol (DHCP) with the NetIQ eDirectory. This provides \n" +
            "additional value for enterprise deployment, such as centralized administration and management of DNS/DHCP servers across the enterprise and DNS Zone/DHCP subnet \n" +
            "replication via NetIQ eDirectory."
        ),
        _(
          "Novell iFolder is a Net services software solution that allows your users' local data files to follow them everywhere--online, offline, all the time--across \n" +
            "multiple workstations, your corporate network and the Internet. It provides a simple, convenient and secure way to access, back up, and protect the local data \n" +
            "files of your mobile professionals. With iFolder, each user has a single virtual work folder that provides convenient and secure access to the most recent \n" +
            "version of local documents. All the user needs is an active network or Internet connection and the iFolder client, a Web browser, or Novell NetDrive. The \n" +
            "iFolder server helps guard against local data loss by automatically backing up the user's local files to an iFolder server, making them available for regularly \n" +
            "scheduled data backup. When the user logs in to an iFolder server from another workstation, the iFolder client synchronizes the files by downloading the new and \n" +
            "changed files."
        ),
        _(
          "The eDirectory Management Toolbox (eMBox) includes repair, merge, and backup/restore tools that allow you to manage and maintain NetIQ eDirectory. eMBox \n" +
            "lets you access all of the eDirectory backend utilities remotely as well as on the server, and it works with Novell iManager to provide Web-based access to \n" +
            "eDirectory utilities such as DSRepair, DSMerge, Backup and Restore, and Service Manager."
        ),
        _(
          "The eDirectory Simple Network Management Protocol (SNMP) Service delivers a directory server with real-time monitoring services, which minimizes the number \n" +
            "and complexity of network management functions. The eDirectory SNMP Service provides real-time monitoring for eDirectory servers, the ability to monitor \n" +
            "eDirectory from any third-party SNMP MIB browser, the ability to track the status of eDirectory to verify normal operations, and the ability to configuring \n" +
            "traps and statistics for selective monitoring."
        ),
        _(
          "MySQL is an open-source, structured query language (SQL) database. When combined with a Web application, MySQL serves as a very reliable and scalable \ndatabase for use with business solutions such as eCommerce and business-to-business."
        ),
        _("This selection installs Tomcat 5."),
        _(
          "Novell exteNd Application Server 5.0 (Enterprise Edition) is a comprehensive J2EE 1.3 server for deploying enterprise-class Web applications."
        ),
        _(
          "iSCSI is an industry standard that lets you build low-cost, fully functional Storage Area Networks (SANs) using commodity LAN (Ethernet) hardware. \n" +
            "(Improved storage system performance is possible with Gigabit Ethernet hardware.) This creates an iSCSI Storage Server (also known as an iSCSI Target) out \n" +
            "of a NetWare server. The NetWare iSCSI Storage Server includes LDAP access control so you can limit the servers on the iSCSI SAN that can access the storage \n" +
            "on the iSCSI Storage Server. With iSCSI, existing direct-attached or SAN-attached servers can be redeployed as iSCSI Storage Servers. This lets you improve \n" +
            "the manageability and deployment of your storage while also adding high availability with Novell Cluster Services, which Novell's iSCSI Storage Server fully \n" +
            "supports."
        ),
        _("OpenWBEM"),
        _(
          "Use this option to restore the native file access login methods to eDirectory after using the migration tool which has restored the tree from a backup."
        ),
        _(
          "Beans for Novell Services provides an easy-to-use suite of Java components for rapid and effective development of network applications. Beans are provided \nfor LDAP, UDDI, WebDAV and Novell SecretStore."
        ),
        _("Novell Script for NetWare"),
        _("Perl for NetWare"),
        _("PHP for NetWare"),
        _(
          "OpenSSH provides a secure shell that encrypts all traffic (including passwords) to effectively eliminate eavesdropping, connection hijacking, and other \n" +
            "network-level attacks. It provides secure connections, secure forwarding of arbitrary TCP connections, and a myriad of secure tunneling capabilities. Through \n" +
            "the secure shell, users with security equal to Admin can remotely access any server in your network and copy files and directories to and from other servers \n" +
            "using SSH utilities without the use of a browser or proprietary client. Other users can access the files and directories that they have rights to. OpenSSH \n" +
            "integrated with NetWare uses Novell(r) International Cryptographic Infrastructure (NICI) rather than SSL to provide stronger encryption and fewer software import \n" +
            "liabilities."
        ),
        _(
          "The WAN Connectivity component allows you to set up a WAN connection between two NetWare Servers using the PPP protocol."
        ),
        _(
          "eGuide is a Web-based application that provides a GUI for searching, viewing and editing information stored in NetIQ eDirectory and other directory data \nsources.  eGuide is highly customizable for any type of environment."
        ),
        _("This will install iManager 2.5."),
        _(
          "This option installs the components that offer users the Virtual Office Home page and Virtual Teams.  For access to iPrint, eGuide, and Web Mail, \nThese individual services must be either selected as part of the custom installation or configured through iManager."
        )
      ]

      #  Remove some of the nmas logins
      # global list<string> list_nmas = ["AdvX509", "CertMutual", "ChallengeResponse", "DigestMD5", "EnhancedPwd", "EntrustAdv", "NDS", "NDSChangePwd", "SimplePassword",
      # 			"USmartCard", "X509Cert"];
      #
      # global list<string> list_nmas_name = [_("X509 Advanced Certificate"), _("CertMutual"), _("Challenge Response"), _("DIGEST-MD5"), _("Enhanced Password"),
      # 			_("Entrust"), _("NDS"), _("NDS Change Password"), _("Simple Password"), _("Universal Smart Card"), _("X509 Certificate")];
      #
      # global list<string> list_nmas_desc = [
      # _("NMAS Advanced X.509 Authentication Method: This method requires the user to have a PKCS#12 password encrypted file, also known as a PFX file. The private key in this
      # file is used to authenticate the user to the NetIQ eDirectory by verifying the user's certificate chain to a trusted root installed on the server."),
      # _("The Certificate Mutual login method implements the Simple Authentication and Security Layer (SASL) EXTERNAL mechanism to provide SSL certificate based client
      # authentication to eDirectory through LDAP."),
      # _("Novell's Challenge Response NMAS Method"),
      # _("The Digest MD5 login method implements the Simple Authentication and Security Layer (SASL) DIGEST-MD5 mechanism as a means of authenticating the user to eDirectory
      # through LDAP."),
      # _("Enhanced Password Login NMAS Method"),
      # _("Entrust Certificate Login Method"),
      # _("The NDS login method provides secure password challenge-response user authentication to eDirectory. This method supports the traditional NDS password when the NMAS
      # client is in use and is installed by default. Reinstallation is necessary only if the NDS login method object has been removed from the directory."),
      # _("NDS Change Password Post Login Method"),
      # _("The Simple Password NMAS login method provides password authentication to eDirectory.  The Simple Password is a more flexible but less secure alternative to the NDS
      # password.  Simple Passwords are stored in a secret store on the user object."),
      # _("NMAS Universal SmartCard X.509 Authentication Method One of the private keys in the smartcard is used to authenticate the user to the NetIQ eDirectory by verifying the
      # user's certificate chain to a trusted root installed on the server. This version of NMAS auto-detects the following smartcard providers and their PKCS#11 provider library
      # file names. You can enter a custom PKCS#11 provider name if you wish to. ActivCard Gold (acpkcs.dll), GemPlus (pk2priv.dll), SmartTrust (id2cbox.dll)"),
      # _("Certificate based authentication. This method employs a Novell Certificate Server X509 certificate to authenticate the user.")
      # ];

      @list_nmas = [
        "CertMutual",
        "ChallengeResponse",
        "DigestMD5",
        "NDS",
        "SimplePassword"
      ]

      @list_nmas_name = [
        _("CertMutual"),
        _("Challenge Response"),
        _("DIGEST-MD5"),
        _("NDS"),
        _("Simple Password")
      ]

      @list_nmas_desc = [
        _(
          "The Certificate Mutual login method implements the Simple Authentication and Security Layer (SASL) EXTERNAL mechanism to provide SSL certificate based client \nauthentication to eDirectory through LDAP."
        ),
        _("Novell's Challenge Response NMAS Method"),
        _(
          "The Digest MD5 login method implements the Simple Authentication and Security Layer (SASL) DIGEST-MD5 mechanism as a means of authenticating the user to eDirectory \nthrough LDAP."
        ),
        _(
          "The NDS login method provides secure password challenge-response user authentication to eDirectory. This method supports the traditional NDS password when the NMAS \nclient is in use and is installed by default. Reinstallation is necessary only if the NDS login method object has been removed from the directory."
        ),
        _(
          "The Simple Password NMAS login method provides password authentication to eDirectory.  The Simple Password is a more flexible but less secure alternative to the NDS \npassword.  Simple Passwords are stored in a secret store on the user object."
        )
      ]

      # Data was modified?
      @modified = false


      @proposal_valid = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # return next virtual disk name
    #
    # Function return the next virtual disk name
    #
    # @param [String] initial		initial virtual disk name
    # @return [String]		return next vdisk name
    def getVirtualDiskName(initial)
      if initial == ""
        return "xvda"
      elsif initial == "xvda"
        return "xvdb"
      elsif initial == "xvdb"
        return "xvdc"
      elsif initial == "xvdc"
        return "xvdd"
      elsif initial == "xvdd"
        return "xvde"
      elsif initial == "xvde"
        return "xvdf"
      elsif initial == "xvdf"
        return "xvdg"
      elsif initial == "xvdg"
        return "xvdh"
      elsif initial == "xvdh"
        return "xvdi"
      elsif initial == "xvdi"
        return "xvdj"
      elsif initial == "xvdk"
        return "xvdl"
      elsif initial == "xvdl"
        return "xvdm"
      elsif initial == "xvdm"
        return "xvdn"
      elsif initial == "xvdn"
        return "xvdo"
      elsif initial == "xvdo"
        return "xvdp"
      elsif initial == "xvdp"
        return "xvdq"
      elsif initial == "xvdq"
        return "xvdr"
      elsif initial == "xvdr"
        return "xvds"
      elsif initial == "xvds"
        return "xvdt"
      elsif initial == "xvdt"
        return "xvdu"
      elsif initial == "xvdu"
        return "xvdv"
      elsif initial == "xvdv"
        return "xvdw"
      elsif initial == "xvdw"
        return "xvdx"
      elsif initial == "xvdx"
        return "xvdy"
      elsif initial == "xvdy"
        return "xvdz"
      end
      ""
    end

    # Search in List
    #
    # Function to search for an string in a list
    #
    # @param [Array<String>] myList		list to be searched
    # @param [String] myString		string to search for
    # @return [Fixnum]		-1 if string not found, otherwise the entry number in the list
    def FindInList(myList, myString)
      myList = deep_copy(myList)
      i = 0
      k = Builtins.size(myList)
      Builtins.y2milestone(
        "Responsefile::FindInList() String %1  Size %2  List %3",
        myString,
        k,
        myList
      )
      while Ops.less_than(i, k)
        if Ops.get(myList, i, "") == myString
          Builtins.y2milestone("Responsefile::FindInList() found %1", i)
          return i
        end
        i = Ops.add(i, 1)
      end
      Builtins.y2milestone("Responsefile::FindInList() not found")

      nil
    end

    def ListToString(mylist)
      mylist = deep_copy(mylist)
      return "" if Builtins.size(mylist) == 0
      pos = 0
      ret = ""
      Builtins.foreach(mylist) do |val|
        ret = Ops.add(Ops.add(ret, val), ",")
        pos = Ops.add(pos, 1)
      end
      ret = Builtins.substring(ret, 0, Ops.subtract(Builtins.size(ret), 1))
      #   y2milestone ("Responsefile::ListToString() %1", ret);
      ret
    end

    def MergeLists(mylist1, mylist2)
      mylist1 = deep_copy(mylist1)
      mylist2 = deep_copy(mylist2)
      templist = deep_copy(mylist2)
      pos = Ops.subtract(Builtins.size(mylist2), 1)
      while Ops.greater_or_equal(pos, 0)
        if Builtins.find(mylist1) { |var| var == Ops.get(mylist2, pos, "") } != nil
          templist = Builtins.remove(templist, pos)
        end
        pos = Ops.subtract(pos, 1)
      end
      templist = Convert.convert(
        Builtins.merge(mylist1, templist),
        :from => "list",
        :to   => "list <string>"
      )
      #   y2milestone ("Responsefile::MergeLists() %1", templist);
      deep_copy(templist)
    end

    # Validate IP address
    # @param IP address to check
    # @param string message to dispay if IP address is not valid
    # @return [Boolean] true if valid IP address
    def ValidateIPMsg(ip, msg)
      if !IP.Check4(ip)
        if ip != ""
          Popup.Error(IP.Valid4)
        else
          Popup.Error(msg)
        end
        return false
      end
      true
    end

    # Validate an IP address
    # @param string IP address
    # @return [Boolean] true if valid IP address
    def ValidateIP(ip)
      ValidateIPMsg(ip, _("Enter the existing server IP address."))
    end

    # Validates two passwords to ensure they are the same and do not contain any invalid chars
    # @param string first password
    # @param string second password
    # @return [Boolean] true if the password is valid
    def ValidatePasswords(pw1, pw2)
      valid_password_chars = "-0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!@\#$%^&*(),;:._+/|?{}=[]"
      if pw1 != pw2
        Report.Error(
          _(
            "The first and the second version of the password don't match! Please try again."
          )
        )
        return false
      elsif Builtins.findfirstnotof(pw1, valid_password_chars) != nil
        Report.Error(
          _(
            "The password may only contain the following characters:\n" +
              " 0..9, a..z, A..Z, and any of \"#*,.;:._-+!$%&/|?{[()]}\".\n" +
              "Please try again."
          )
        )
        return false
      end
      true
    end

    # Validates name to ensure it does not contain any invalid chars
    # @param [String] name
    # @return [Boolean] true if the name is valid
    def ValidateName(name)
      valid_password_chars = "-0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_"

      if Builtins.findfirstnotof(name, valid_password_chars) != nil
        Report.Error(
          _(
            "The name may only contain the following characters:\n" +
              " 0..9, a..z, A..Z, and  _-\n" +
              "Please try again."
          )
        )
        return false
      end
      true
    end

    # ValidatePathName - Validate the characters in a proposed pathname
    # NOTE: May not work for double-byte languages???
    #
    # @param [String] pathname to validate
    # @param [Boolean] absolute - if true validate that pathname is absolute pathname
    # @return [Boolean] true on success
    def ValidatePathName(pathname, absolute)
      if Builtins.issubstring(pathname, "//") == true
        Report.Error(
          Builtins.sformat(_("%1 is not a valid pathname"), pathname)
        )
        return false
      end
      if !absolute
        return true
      else
        slashloc = Builtins.findfirstof(pathname, "/")
        if slashloc != 0
          Report.Error(
            Builtins.sformat(_("%1 is not a valid absolute pathname"), pathname)
          )
          return false
        end
      end
      true
    end

    # EndWithSlash - add slash to end of pathname
    # NOTE: May not work for double-byte languages???
    #
    # @param [String] pathname
    # @return [String]
    def EndWithSlash(pathname)
      slashloc = Builtins.findlastof(pathname, "/")
      if slashloc != Ops.subtract(Builtins.size(pathname), 1)
        pathname = Ops.add(pathname, "/")
      end
      pathname
    end


    # ValidateNWPathname - Validate the characters in a proposed pathname
    # NOTE: May not work for double-byte languages???
    #
    # @param [String] pathname to validate
    # @param boolean absolute - if true validate that pathname is absolute pathname
    # @return [Boolean] true on success
    def ValidateNWPathname(pathname)
      ccode = true
      # disallow //
      if Builtins.issubstring(pathname, "//") == true ||
          Builtins.issubstring(pathname, "\\\\") == true
        ccode = false
      else
        colonloc = Builtins.findfirstof(pathname, ":")
        slashloc = Builtins.findfirstof(pathname, "/")
        backslashloc = Builtins.findfirstof(pathname, "\\")
        Builtins.y2milestone(
          "Responsefile::ValidateNWPathname() colonloc: %1  slashloc: %2  backslashloc: %3",
          colonloc,
          slashloc,
          backslashloc
        )
        colonloc = -1 if colonloc == nil
        slashloc = Ops.add(colonloc, 1) if slashloc == nil
        backslashloc = Ops.add(colonloc, 1) if backslashloc == nil
        slashloc = backslashloc if Ops.less_than(backslashloc, slashloc)
        if Ops.less_than(colonloc, 2) || Ops.greater_than(colonloc, slashloc)
          ccode = false
        end
      end
      if !ccode
        Report.Error(
          Builtins.sformat(_("%1 is not a valid pathname"), pathname)
        )
      end
      ccode
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # Settings: Define all variables needed for configuration of responsefile
    # TODO FIXME: Define all the variables necessary to hold
    # TODO FIXME: the configuration here (with the appropriate
    # TODO FIXME: description)
    # TODO FIXME: For example:
    #   /**
    #    * List of the configured cards.
    #    */
    #   list cards = [];
    #
    #   /**
    #    * Some additional parameter needed for the configuration.
    #    */
    #   boolean additional_parameter = true;

    # Read all responsefile settings
    # @return true on success
    def Read
      Builtins.y2milestone("Read() started")
      # Responsefile read dialog caption
      caption = _("Initializing responsefile Configuration")

      # TODO FIXME Set the right number of stages
      steps = 1

      sl = 500
      #    sleep(sl);

      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Read the Responsefile settings")
        ],
        [
          # Progress step 1/3
          _("Reading the settings..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read database
      return false if Abort()
      Progress.NextStage
      # Error message
      return false if Abort()
      @modified = false
      true
    end

    # Write all responsefile settings
    # @return true on success
    def Write
      Builtins.y2milestone("Write() started")
      # Responsefile read dialog caption
      caption = _("Saving responsefile Configuration")

      # TODO FIXME And set the right number of stages
      steps = 1

      sl = 500
      #    sleep(sl);

      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Write the Responsefile settings")
        ],
        [
          # Progress step 1/1
          _("Writing the settings..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # write settings
      return false if Abort()
      Progress.NextStage

      return false if Abort()
      # Progress finished
      Progress.Finish
      #    sleep(sl);

      return false if Abort()
      true
    end

    # Get all responsefile settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      # TODO FIXME: your code here (fill the above mentioned variables)...
      true
    end

    # Dump the responsefile settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      # TODO FIXME: your code here (return the above mentioned variables)...
      {}
    end

    # ReadRF - Read the Key from the response file
    #
    # @param path pathname to read
    # @return [String] read key or "" if none
    def ReadRF(mypath)
      ret = Convert.to_string(SCR.Read(mypath))
      ret = "" if ret == nil
      ret
    end

    # WriteRF - Read the Key from the response file
    #
    # @param path pathname to write
    # @return [Boolean] true if successful false if not
    def WriteRF(mypath, mystring)
      SCR.Write(mypath, mystring)
    end

    # CheckProductPrameter - check if there are product parameters to configure
    #
    # @param [Fixnum] index of product
    # @return [Boolean] true or false
    def CheckProductPrameter(index)
      Builtins.y2milestone("Responsefile::CheckProductPrameter() started")
      ret = false
      if Ops.get_boolean(@map_patterns, "visible", false) &&
          Ops.get_integer(@map_patterns, "pattern", 0) != 0 # A pattern has been selected
        patternlist = Ops.get(
          @list_pattern_prod_params,
          Ops.get_integer(@map_patterns, "pattern", 0),
          []
        )

        Builtins.y2milestone(
          "Responsefile::CheckProductPrameters() index: %1  sizepattern: %2  list: %3",
          index,
          Builtins.size(patternlist),
          patternlist
        )
        patternsize = Builtins.size(patternlist)
        pos = 0
        while Ops.less_than(pos, patternsize)
          if index == Ops.get(patternlist, pos, 0)
            ret = true
            break
          end
          pos = Ops.add(pos, 1)
        end
      elsif Ops.get_boolean(@map_products, "visible", false) # Custom product has been selected
        Builtins.y2milestone(
          "Responsefile::CheckProductPrameter() products index: %1  %2",
          index,
          Ops.get(@list_products_select, index, false)
        )
        if Ops.get(@list_products_select, index, false) ||
            Ops.get(@list_products_nc, index, false)
          ret = true
        end
      end
      #	y2milestone ("Responsefile::CheckProductPrameter() ended");
      ret
    end

    # Substitute - replace pattern in string with replacement string
    #
    # @param [String] mystring string to fixup
    # @param [String] pattern string to search for
    # @param [String] replace replacement string
    # @return [String] fixed up string
    def Substitute(mystring, pattern, replace)
      newstring = ""
      searchstring = mystring
      found = Builtins.regexppos(mystring, pattern)
      #	y2milestone ("Responsefile::Substitute() started size of found %1", size(found));

      while Builtins.size(found) == 2
        newstring = Ops.add(
          Ops.add(
            newstring,
            Builtins.substring(searchstring, 0, Ops.get(found, 0, 0))
          ),
          replace
        )
        searchstring = Builtins.substring(
          searchstring,
          Ops.add(Ops.get(found, 0, 0), Ops.get(found, 1, 0)),
          Ops.subtract(
            Ops.subtract(Builtins.size(searchstring), Ops.get(found, 0, 0)),
            Ops.get(found, 1, 0)
          )
        )
        found = Builtins.regexppos(searchstring, pattern)
      end
      if Ops.greater_than(Builtins.size(searchstring), 0)
        newstring = Ops.add(newstring, searchstring)
      end
      #	y2milestone ("Responsefile::Substitute() ended");
      newstring
    end

    # FixupString - double (escape) the '\' characters in the string
    #
    # @param [String] str string to fixup
    # @return [String] fixed up string
    def FixupString(str)
      tmp = ""
      pos = 0

      while Builtins.findfirstof(str, "\\") != nil
        pos = Builtins.findfirstof(str, "\\")
        tmp = Ops.add(
          Ops.add(tmp, Builtins.substring(str, 0, Ops.add(pos, 1))),
          "\\"
        )
        Builtins.y2milestone(
          "Responsefile::FixupString() pos: %1  tmp: %2",
          pos,
          tmp
        )
        str = Builtins.substring(
          str,
          Ops.add(pos, 1),
          Ops.subtract(Builtins.size(str), Ops.add(pos, 1))
        )
        Builtins.y2milestone("Responsefile::FixupString() str: %1", str)
      end
      tmp = Ops.add(tmp, str)
      tmp
    end

    # CreateFileName - remove the extension from name and add a new one
    #
    # @param [String] name string name of original file
    # @param [String] extension string new extension
    # @return [String] new file name
    def CreateFileName(name, extension)
      return "" if name == nil || extension == nil
      #	y2milestone ("Responsefile::CreateFileName() name: %1  ext: %2", name, extension);
      pos = Builtins.findlastof(name, ".")
      name = Builtins.substring(name, 0, pos) if pos != nil
      name = Ops.add(Ops.add(name, "."), extension)
      name
    end

    # GetSeparator -
    #
    # @param [String] mypath string
    # @return [String] separator string
    def GetSeparator(mypath)
      separator = ""
      if Builtins.findfirstof(mypath, "/") != nil
        separator = "/"
      elsif Builtins.findfirstof(mypath, "\\") != nil
        separator = "\\"
      else
        separator = "/"
      end
      separator
    end

    # CreateICSString -
    #
    # @param name string name of original file
    # @param path string
    # @return [String] ICS string
    def CreateIPSString(rfname, rfpath)
      separator = GetSeparator(rfpath)
      #	y2milestone ("Responsefile::CreateICSString() rfname: %1  rfpath: %2", rfname, rfpath);
      tempstring = Ops.add(Ops.add(rfpath, separator), rfname)
      #	y2milestone ("Responsefile::CreateICSString() string1: %1", tempstring);
      tempstring = FixupString(Ops.add(Ops.add(rfpath, separator), rfname))
      #	y2milestone ("Responsefile::CreateICSString() string2: %1", tempstring);
      tempstring = Ops.add(
        Ops.add("Console 'sys:/ni/bin/runsetup /RF=", tempstring),
        "', 1"
      )
      #	y2milestone ("Responsefile::CreateICSString() string3: %1", tempstring);
      tempstring
    end

    # CreateNCFString -
    #
    # @param name string name of original file
    # @param path string
    # @return [String] NCF string
    def CreateNCFString(rfname, rfpath)
      separator = GetSeparator(rfpath)
      ncfname = CreateFileName(rfname, "ncf")
      logname = CreateFileName(rfname, "log")
      ipsname = CreateFileName(rfname, "ips")
      username = Ops.get_string(@map_src_edir, "srcusername", "")
      context = Ops.get_string(@map_src_edir, "srcuserctx", "")
      password = Ops.get_string(@map_src_edir, "srcpassword", "")
      treename = Ops.get_string(@map_src_edir, "srctreename", "")
      domain = Ops.get_string(@map_src_addr, "srcipaddr", "")
      servername = Ops.get_string(@map_src_addr, "srcname", "")
      location = Ops.get_string(@map_src_addr, "srclocpath", "")
      ret = Ops.add(
        Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              Ops.add(
                                Ops.add(
                                  Ops.add(
                                    Ops.add(
                                      Ops.add(
                                        Ops.add(
                                          Ops.add(
                                            Ops.add(
                                              Ops.add(
                                                Ops.add(
                                                  Ops.add(
                                                    Ops.add(
                                                      Ops.add(
                                                        Ops.add(
                                                          Ops.add(
                                                            "load NWCONFIG.NLM TRANSPORT=UDP u=\"",
                                                            username
                                                          ),
                                                          "\" DS c=\""
                                                        ),
                                                        username
                                                      ),
                                                      "."
                                                    ),
                                                    context
                                                  ),
                                                  "\" p="
                                                ),
                                                password
                                              ),
                                              " z="
                                            ),
                                            treename
                                          ),
                                          " a="
                                        ),
                                        domain
                                      ),
                                      " e="
                                    ),
                                    rfpath
                                  ),
                                  separator
                                ),
                                logname
                              ),
                              " b="
                            ),
                            rfpath
                          ),
                          separator
                        ),
                        ipsname
                      ),
                      " d=C:\\NWSERVER s="
                    ),
                    servername
                  ),
                  "/"
                ),
                location
              ),
              ",3 n="
            ),
            servername
          ),
          "/"
        ),
        location
      )
      #	y2milestone ("Responsefile::CreateNCFString() rfname: %1  rfpath: %2", rfname, rfpath);
      #	y2milestone ("Responsefile::CreateICSString() string1: %1", ret);
      ret
    end

    def InitializeVariables
      installtype = Ops.get_integer(@map_select_type, "installtype", 0)
      hardwaretype = Ops.get_integer(@map_select_type, "hardwaretype", 0)

      @map_src_addr = {
        "complete"   => false,
        "visible"    => false,
        "type"       => 0,
        "srcname"    => "",
        "srcipaddr"  => "",
        "srclocpath" => ""
      }
      if hardwaretype == 0 # Local upgrade (physical) always 0, Virtual install always 1
        Ops.set(@map_src_addr, "type", 1)
      end

      @map_src_edir = {
        "complete"    => false,
        "visible"     => false,
        "srctreename" => "",
        "srcsrvctx"   => "",
        "srcusername" => "",
        "srcuserctx"  => "",
        "srcpassword" => ""
      }
      @map_dest_addr = {
        "complete"   => false,
        "complete1"  => false,
        "visible"    => false,
        "visible1"   => false,
        "destname"   => "",
        "selectip"   => true,
        "destipaddr" => "",
        "destsubnet" => "",
        "destrouter" => "",
        "selectipx"  => false,
        "ipxframe"   => 0,
        "destipxid"  => ""
      }
      @map_dest_edir = {
        "complete"      => false,
        "visible"       => false,
        "destnewtree"   => false,
        "destreename"   => "",
        "destsrvctx"    => "",
        "replicaipaddr" => "",
        "destadminname" => "",
        "destadminctx"  => "",
        "destpassword"  => ""
      }
      @map_license = {
        "complete"  => false,
        "visible"   => false,
        "context"   => "",
        "nolicense" => false
      }

      @map_dest_lang = {
        "complete"     => false,
        "visible"      => false,
        "language"     => 0,
        "countrycode"  => 0,
        "codepage"     => 0,
        "keyboardcode" => 0
      }
      templist = Convert.convert(
        Ops.get(@list_language_defaults, 0) { [0, 0, 0, 18] },
        :from => "any",
        :to   => "list <const integer>"
      )
      Ops.set(@map_dest_lang, "language", Ops.get(templist, 0, 0))
      Ops.set(@map_dest_lang, "countrycode", Ops.get(templist, 1, 0))
      Ops.set(@map_dest_lang, "codepage", Ops.get(templist, 2, 0))
      Ops.set(@map_dest_lang, "keyboardcode", Ops.get(templist, 3, 18))

      @map_dest_slp = {
        "complete"     => false,
        "visible"      => false,
        "skipslp"      => false,
        "daserver1"    => "",
        "daserver2"    => "",
        "daserver3"    => "",
        "slpscopelist" => "",
        "diragent"     => false,
        "multicast"    => false
      }
      @map_dest_dns = {
        "complete"  => false,
        "visible"   => false,
        "hostname"  => "",
        "domain"    => "",
        "dnserver1" => "",
        "dnserver2" => "",
        "dnserver3" => "",
        "verifydns" => false
      }
      @map_timezones = {
        "complete"       => false,
        "visible"        => false,
        "timezone"       => 6,
        "daylightsaving" => false
      }
      @map_timesync = {
        "complete"    => false,
        "visible"     => false,
        "protocol"    => 1,
        "servertype"  => 2,
        "timesync"    => false,
        "timesource1" => "",
        "timesource2" => "",
        "timesource3" => "",
        "timesrcntp1" => false,
        "timesrcntp1" => false,
        "timesrcntp3" => false
      }
      @map_install = {
        "complete"  => false,
        "complete1" => false,
        "visible"   => false,
        "visible1"  => false,
        "backup"    => 0,
        "backuploc" => "",
        "reboot"    => 2,
        "drivers"   => 5,
        "express"   => 6,
        "autoexec"  => 8
      }
      @map_storage = {
        "complete"  => false,
        "complete1" => false,
        "visible"   => false,
        "fatpart"   => 500,
        "fatmin"    => 85,
        "fatmax"    => 8000,
        "fatdef"    => 8000,
        "sysvol"    => 9500,
        "sysmin"    => 1900,
        "sysmax"    => 2000000,
        "sysdef"    => 4000
      }

      @map_patterns = {
        "complete" => false,
        "visible"  => false,
        "pattern"  => 0
      }
      @map_products = { "complete" => false, "visible" => false }
      @list_products_select = [
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false,
        false
      ]
      @list_products_nc = [
        true,
        true,
        true,
        true,
        true,
        false,
        true,
        true,
        true,
        false,
        false,
        false,
        false,
        true,
        true,
        false,
        false,
        true,
        false,
        false,
        false,
        false,
        false,
        false,
        true,
        false,
        true,
        false,
        true,
        true,
        true,
        false,
        false,
        false,
        false,
        false
      ]

      @map_nmas = { "complete" => false, "visible" => false }
      @list_nmas_select = [false, true, false, true, false]
      @map_dns_dhcp = {
        "complete" => false,
        "visible"  => false,
        "locator"  => "",
        "group"    => "",
        "zone"     => ""
      }
      @map_ifolder = {
        "complete"     => false,
        "visible"      => false,
        "ldaphost"     => "",
        "ldapport"     => "636",
        "ldapcontext"  => "",
        "ifolderhost"  => "",
        "ifolderadmin" => "",
        "dbpath"       => "sys:/ifolder"
      }
      @map_mysql = {
        "complete" => false,
        "visible"  => false,
        "datadir"  => "sys:/mysql/data",
        "password" => "root",
        "secure"   => true
      }
      @map_extend = {
        "complete"   => false,
        "visible"    => false,
        "adminname"  => "admin",
        "password"   => "admin",
        "httpport"   => "83",
        "restrict"   => true,
        "dbhost"     => "localhost",
        "mysqlport"  => "3306",
        "dbusername" => "appserver",
        "dbpassword" => "appserver",
        "smname"     => "SilverMaster50",
        "executesmi" => true
      }
      @map_save_rf = {
        "complete"    => false,
        "visible"     => false,
        "rfname"      => "netware.rsp",
        "tmprfpath"   => "",
        "rfpath"      => "A:\\",
        "srclocpath"  => "",
        "destlocpath" => ""
      }
      if installtype == 4 && hardwaretype == 1
        Ops.set(@map_save_rf, "rfname", "RESPONSE.TMP")
        Ops.set(@map_save_rf, "rfpath", "C:\\NWINST.TMP")
      end

      true
    end

    def ImportRFTemplate
      Builtins.y2milestone("Responsefile::ImportRFTemplate() started")
      installtype = Ops.get_integer(@map_select_type, "installtype", 0)
      hardwaretype = Ops.get_integer(@map_select_type, "hardwaretype", 0)
      tempstring = ""
      tempstring1 = ""
      tempint = 0
      tempint1 = 0
      tempint2 = 0
      tempint3 = 0
      templist = []
      templist1 = []
      templist2 = []
      tempbool = false
      tempbool1 = false
      finished = true
      temppath = nil

      #  ******************* Source Address ********************************************************************************
      finished = true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:MISC\".\"Source Server Name\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_src_addr, "srcname", tempstring)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:MISC\".\"Source Domain Address\""
        )
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_src_addr, "srcipaddr", tempstring)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:MISC\".\"Source Location Path\""
        )
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_src_addr, "srclocpath", tempstring)
      end
      if installtype != 1 && hardwaretype == 2 # Local Physical Upgrade always 0, VM install always 1
        tempstring = ReadRF(
          path(
            ".responsefile.template.value.\"RF Generator\".\"NetWare Source Location\""
          )
        )
        if tempstring == ""
          finished = false
        elsif Builtins.toupper(tempstring) == "FALSE"
          Ops.set(@map_src_addr, "type", 1)
          finished = true
        end
      end
      Ops.set(@map_src_addr, "complete", finished)

      #  ******************* Source eDirectory *****************************************************************************
      finished = true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:MISC\".\"Source Tree Name\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_src_edir, "srctreename", tempstring)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:MISC\".\"Source Server Context\""
        )
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_src_edir, "srcsrvctx", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:MISC\".\"Source User Name\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_src_edir, "srcusername", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:MISC\".\"Source Context\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_src_edir, "srcuserctx", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:MISC\".\"Relogin Password\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_src_edir, "srcpassword", tempstring)
      end
      Ops.set(@map_src_edir, "complete", finished)

      #  ******************* Destination Address ***************************************************************************
      finished = true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:File Server\".\"Servername\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_dest_addr, "destname", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:NDS\".\"Domain Address\"")
      )
      if tempstring == ""
        tempstring = ReadRF(
          path(".responsefile.template.value.\"NWI:TCPIP\".\"IP Address 1\"")
        )
      end
      if tempstring == ""
        finished = false
      elsif !ValidateIP(tempstring)
        finished = false
      end
      finished = true if Ops.get_integer(@map_src_addr, "type", 0) == 1

      Ops.set(@map_dest_addr, "destipaddr", tempstring) if finished == true
      Ops.set(@map_dest_addr, "complete", finished)

      #  ******************* Destination eDirectory ************************************************************************
      finished = true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:NDS\".\"Tree Name\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_dest_edir, "destreename", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:NDS\".\"New Tree\"")
      )
      if tempstring == ""
        finished = false
      else
        if Builtins.toupper(tempstring) == "TRUE"
          tempbool = true
        else
          tempbool = false
        end
        Ops.set(@map_dest_edir, "destnewtree", tempbool)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:NDS\".\"Server Context\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_dest_edir, "destsrvctx", tempstring)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:NDS\".\"Replica Server IP Address\""
        )
      )
      Ops.set(@map_dest_edir, "replicaipaddr", tempstring) if tempstring != ""
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:NDS\".\"Admin Login Name\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_dest_edir, "destadminname", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:NDS\".\"Admin Context\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_dest_edir, "destadminctx", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:NDS\".\"Admin Password\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_dest_edir, "destpassword", tempstring)
      end
      Ops.set(@map_dest_edir, "complete", finished)

      #  ******************* License Configuration *************************************************************************
      finished = true
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:License\".\"MLA License Context\""
        )
      )
      if tempstring != ""
        Ops.set(@map_license, "context", tempstring)
      else
        finished = false
        if Ops.get_string(@map_dest_edir, "destsrvctx", "") != ""
          Ops.set(
            @map_license,
            "context",
            Ops.get_string(@map_dest_edir, "destsrvctx", "")
          )
        end
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:License\".\"Install Licenses Later\""
        )
      )
      if tempstring == ""
        finished = false
      elsif Builtins.toupper(tempstring) == "TRUE"
        Ops.set(@map_license, "nolicense", true)
      else
        Ops.set(@map_license, "nolicense", false)
      end
      Ops.set(@map_license, "complete", finished)

      #  ******************* Protocols *************************************************************************************
      finished = true
      tempint = 4
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Protocols\".\"Default IPX Frame Type\""
        )
      )
      if tempstring != ""
        if Builtins.toupper(tempstring) == Ops.get(@list_frame_types, 0, "")
          tempint = 0
        elsif Builtins.toupper(tempstring) == Ops.get(@list_frame_types, 1, "")
          tempint = 1
        elsif Builtins.toupper(tempstring) == Ops.get(@list_frame_types, 2, "")
          tempint = 2
        elsif Builtins.toupper(tempstring) == Ops.get(@list_frame_types, 3, "")
          tempint = 3
        end
        if Ops.less_than(tempint, 4)
          Ops.set(@map_dest_addr, "ipxframe", tempint)
        else
          finished = false
        end
        tempstring = ReadRF(
          path(".responsefile.template.value.\"NWI:IPX\".\"Bind LAN Card 1\"")
        )
        if Builtins.toupper(tempstring) == "TRUE"
          Ops.set(@map_dest_addr, "selectipx", true)
        end
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:File Server\".\"Server ID Number\""
        )
      )
      Ops.set(@map_dest_addr, "destipxid", tempstring) if tempstring != ""
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:TCPIP\".\"IP Address 1\"")
      )
      if tempstring == ""
        tempstring = Ops.get_string(@map_dest_addr, "destipaddr", "")
      end
      if tempstring == "" ||
          !ValidateIPMsg(tempstring, _("Invalid IP address."))
        finished = false
      else
        Ops.set(@map_dest_addr, "destipaddr", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:TCPIP\".\"Subnet Mask 1\"")
      )
      if tempstring == "" ||
          !ValidateIPMsg(tempstring, _("Invalid IP address."))
        finished = false
      else
        Ops.set(@map_dest_addr, "destsubnet", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:TCPIP\".\"Gateway 1\"")
      )
      if tempstring == "" ||
          !ValidateIPMsg(tempstring, _("Invalid IP address."))
        finished = false
      else
        Ops.set(@map_dest_addr, "destrouter", tempstring)
      end
      Ops.set(@map_dest_addr, "complete1", finished)

      #  ******************* Language **************************************************************************************
      finished = true
      tempint = 0
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Locale\".\"Country Code\"")
      )
      if tempstring == ""
        finished = false
        Builtins.y2milestone(
          "Responsefile::ImportRFTemplate() can't find NWI:Locale - Country Code"
        )
      else
        tempint = 0
        tempint1 = Builtins.size(@list_countrycode)
        while Ops.less_than(tempint, tempint1)
          if Builtins.toupper(Ops.get(@list_countrycode, tempint, "")) ==
              Builtins.toupper(tempstring)
            break
          else
            tempint = Ops.add(tempint, 1)
          end
        end
        if Ops.greater_or_equal(tempint, tempint1)
          tempint = 0
          finished = false
          Builtins.y2milestone(
            "Responsefile::ImportRFTemplate() can't translate NWI:Locale - Country Code %1",
            tempstring
          )
        end
      end
      Ops.set(@map_dest_lang, "countrycode", tempint) if finished
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Locale\".\"Code Page\"")
      )
      if tempstring == ""
        finished = false
        Builtins.y2milestone(
          "Responsefile::ImportRFTemplate() can't find NWI:Locale - Code Page"
        )
      else
        tempint = 0
        templist = Ops.get(
          @list_codepage,
          Ops.get_integer(@map_dest_lang, "language", 0),
          []
        )
        tempint1 = Builtins.size(templist)
        while Ops.less_than(tempint, tempint1)
          if Builtins.toupper(Ops.get(templist, tempint, "")) ==
              Builtins.toupper(tempstring)
            break
          else
            tempint = Ops.add(tempint, 1)
          end
        end
        if Ops.greater_or_equal(tempint, tempint1)
          tempint = 0
          finished = false
          Builtins.y2milestone(
            "Responsefile::ImportRFTemplate() can't translate NWI:Locale - Code Page %1",
            tempstring
          )
        end
      end
      Ops.set(@map_dest_lang, "codepage", tempint) if finished
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Locale\".\"Keyboard\"")
      )
      if tempstring == ""
        finished = false
        Builtins.y2milestone(
          "Responsefile::ImportRFTemplate() can't find NWI:Locale - Keyboard"
        )
      else
        tempint = 0
        tempint1 = Builtins.size(@list_keyboardcode_en)
        while Ops.less_than(tempint, tempint1)
          if Builtins.toupper(Ops.get(@list_keyboardcode_en, tempint, "")) ==
              Builtins.toupper(tempstring)
            break
          else
            tempint = Ops.add(tempint, 1)
          end
        end
        if Ops.greater_or_equal(tempint, tempint1)
          tempint = 0
          finished = false
          Builtins.y2milestone(
            "Responsefile::ImportRFTemplate() can't translate NWI:Locale - Keyboard %1",
            tempstring
          )
        end
      end
      Ops.set(@map_dest_lang, "keyboardcode", tempint) if finished
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Language\".\"Server Language\""
        )
      )
      if tempstring == ""
        finished = false
        Builtins.y2milestone(
          "Responsefile::ImportRFTemplate() can't find NWI:Language - Server Language"
        )
      else
        tempint = 0
        tempint1 = Builtins.size(@list_language_number)
        while Ops.less_than(tempint, tempint1)
          if Builtins.toupper(
              Ops.get_string(@list_language_number, tempint, "")
            ) ==
              Builtins.toupper(tempstring)
            break
          else
            tempint = Ops.add(tempint, 1)
          end
        end
        if Ops.greater_or_equal(tempint, tempint1)
          tempint = 0
          finished = false
          Builtins.y2milestone(
            "Responsefile::ImportRFTemplate() can't translate NWI:Language - Server Language %1",
            tempstring
          )
        end
      end
      Ops.set(@map_dest_lang, "language", tempint) if finished
      if finished == false
        templist1 = Convert.convert(
          Ops.get(@list_language_defaults, 0) { [0, 0, 0, 18] },
          :from => "any",
          :to   => "list <const integer>"
        )
        Ops.set(@map_dest_lang, "language", Ops.get(templist1, 0, 0))
        Ops.set(@map_dest_lang, "countrycode", Ops.get(templist1, 1, 0))
        Ops.set(@map_dest_lang, "codepage", Ops.get(templist1, 2, 0))
        Ops.set(@map_dest_lang, "keyboardcode", Ops.get(templist1, 3, 18))
      end
      Ops.set(@map_dest_lang, "complete", finished)

      #  ******************* SLP Configuration *****************************************************************************
      finished = true
      tempint = 0
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:SLP Configuration\".\"DA Server 1\""
        )
      )
      if tempstring != "" && ValidateIPMsg(tempstring, _("Invalid IP address."))
        tempint = Ops.add(tempint, 1)
        Ops.set(
          @map_dest_slp,
          Builtins.sformat("daserver%1", tempint),
          tempstring
        )
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:SLP Configuration\".\"DA Server 2\""
        )
      )
      if tempstring != "" && ValidateIPMsg(tempstring, _("Invalid IP address."))
        tempint = Ops.add(tempint, 1)
        Ops.set(
          @map_dest_slp,
          Builtins.sformat("daserver%1", tempint),
          tempstring
        )
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:SLP Configuration\".\"DA Server 3\""
        )
      )
      if tempstring != "" && ValidateIPMsg(tempstring, _("Invalid IP address."))
        tempint = Ops.add(tempint, 1)
        Ops.set(
          @map_dest_slp,
          Builtins.sformat("daserver%1", tempint),
          tempstring
        )
      end
      tempbool = false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:SLP Configuration\".\"Directory Agent\""
        )
      )
      if tempstring != ""
        tempbool = true if Builtins.toupper(tempstring) == "TRUE"
        Ops.set(@map_dest_slp, "diragent", tempbool)
        tempint = Ops.add(tempint, 1)
      end
      tempbool = false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:SLP Configuration\".\"Multicast Routing\""
        )
      )
      if tempstring != ""
        tempbool = true if Builtins.toupper(tempstring) == "TRUE"
        Ops.set(@map_dest_slp, "multicast", tempbool)
        tempint = Ops.add(tempint, 1)
      end
      if tempint == 0
        tempbool = false
        tempstring = ReadRF(
          path(
            ".responsefile.template.value.\"NWI:SLP Configuration\".\"Skip SLP\""
          )
        )
        if tempstring != ""
          if Builtins.toupper(tempstring) == "TRUE"
            tempbool = true
            tempint = Ops.add(tempint, 1)
          end
          Ops.set(@map_dest_slp, "skipslp", tempbool)
        end
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:SLP Configuration\".\"SLP Scope List\""
        )
      )
      Ops.set(@map_dest_slp, "slpscopelist", tempstring) if tempstring != ""
      finished = false if tempint == 0
      Ops.set(@map_dest_slp, "complete", finished)

      #  ******************* DNS Configuration *****************************************************************************
      finished = true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:DNS\".\"Domain\"")
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_dest_dns, "domain", tempstring)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:DNS\".\"Nameservers\"")
      )
      if tempstring != ""
        templist = Builtins.regexptokenize(
          tempstring,
          "[ ,]*([0-9.]+)[ ,]*([0-9.]*)[ ,]*([0-9.]*)"
        )
        tempint1 = Builtins.size(templist)
        if Ops.greater_than(tempint1, 0) && Ops.get(templist, 0, "") != ""
          Ops.set(@map_dest_dns, "dnserver1", Ops.get(templist, 0, ""))
          if Ops.greater_than(tempint1, 1) && Ops.get(templist, 1, "") != ""
            Ops.set(@map_dest_dns, "dnserver2", Ops.get(templist, 1, ""))
            if Ops.greater_than(tempint1, 0) && Ops.get(templist, 2, "") != ""
              Ops.set(@map_dest_dns, "dnserver3", Ops.get(templist, 2, ""))
            end
          end
        end
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Host Name\".\"Verify\"")
      )
      tempbool = false
      if tempstring == ""
        finished = false
      else
        tempbool = true if Builtins.toupper(tempstring) == "TRUE"
        Ops.set(@map_dest_dns, "verifydns", tempbool)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Host Name\".\"Host Name 1\"")
      )
      if tempstring == ""
        finished = false
      else
        tempint = Builtins.findfirstof(tempstring, ".")
        if tempint != nil
          tempstring = Builtins.substring(tempstring, 0, tempint)
        end
        Ops.set(@map_dest_dns, "hostname", tempstring)
      end
      Ops.set(@map_dest_dns, "complete", finished)

      #  ******************* Time Zone Configuration ***********************************************************************
      finished = true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Time Zone\".\"Time Zone\"")
      )
      if tempstring == ""
        finished = false
      else
        tempint = 0
        tempint1 = Builtins.size(@list_timezones)
        while Ops.less_than(tempint, tempint1)
          if Builtins.toupper(Ops.get(@list_timezones, tempint, "")) ==
              Builtins.toupper(tempstring)
            break
          else
            tempint = Ops.add(tempint, 1)
          end
        end
        if Ops.greater_or_equal(tempint, tempint1)
          finished = false
        else
          Ops.set(@map_timezones, "timezone", tempint)
        end
      end
      tempbool = false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Time Zone\".\"Use Daylight Saving Time\""
        )
      )
      if tempstring == ""
        finished = false
      else
        if Builtins.toupper(tempstring) == "TRUE" &&
            Ops.get(@list_timezones_dst, tempint, false)
          tempbool = true
        end
        Ops.set(@map_timezones, "daylightsaving", tempbool)
      end
      Ops.set(@map_timezones, "complete", finished)

      #  ******************* Time Sync Configuration ***********************************************************************
      finished = true
      tempint = 1
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Time Synchronization\".\"Time Server Protocol\""
        )
      )
      if tempstring == ""
        finished = false
      else
        if Builtins.toupper(tempstring) ==
            Builtins.toupper(Ops.get(@list_timeserver_protocol, 0, ""))
          tempint = 0
        end
        Ops.set(@map_timesync, "protocol", tempint)
      end
      if tempint == 1
        tempstring = ReadRF(
          path(
            ".responsefile.template.value.\"NWI:Time Synchronization\".\"Time Server Type\""
          )
        )
        if tempstring == ""
          finished = false
        elsif Builtins.toupper(tempstring) ==
            Builtins.toupper(Ops.get(@list_timeserver_type, 0, ""))
          tempint = 0
        elsif Builtins.toupper(tempstring) ==
            Builtins.toupper(Ops.get(@list_timeserver_type, 1, ""))
          tempint = 1
        elsif Builtins.toupper(tempstring) ==
            Builtins.toupper(Ops.get(@list_timeserver_type, 2, ""))
          tempint = 2
        elsif Builtins.toupper(tempstring) ==
            Builtins.toupper(Ops.get(@list_timeserver_type, 3, ""))
          tempint = 3
        else
          finished = false
        end
        Ops.set(@map_timesync, "servertype", tempint) if finished
      end
      tempint = 0
      tempbool = false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Time Synchronization\".\"Use Time Source\""
        )
      )
      if tempstring == ""
        finished = false
      else
        tempbool = true if Builtins.toupper(tempstring) == "TRUE"
        Ops.set(@map_timesync, "timesync", tempbool)
      end
      tempbool1 = false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Time Synchronization\".\"Time Source 1\""
        )
      )
      if tempstring != ""
        if Builtins.issubstring(tempstring, ":123")
          tempbool1 = true
          tempstring = Builtins.substring(
            tempstring,
            0,
            Ops.subtract(Builtins.size(tempstring), 4)
          )
        end
        tempint = Ops.add(tempint, 1)
        Ops.set(
          @map_timesync,
          Builtins.sformat("timesource%1", tempint),
          tempstring
        )
        Ops.set(
          @map_timesync,
          Builtins.sformat("timesrcntp%1", tempint),
          tempbool1
        )
      end
      tempbool1 = false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Time Synchronization\".\"Time Source 2\""
        )
      )
      if tempstring != ""
        if Builtins.issubstring(tempstring, ":123")
          tempbool1 = true
          tempstring = Builtins.substring(
            tempstring,
            0,
            Ops.subtract(Builtins.size(tempstring), 4)
          )
        end
        tempint = Ops.add(tempint, 1)
        Ops.set(
          @map_timesync,
          Builtins.sformat("timesource%1", tempint),
          tempstring
        )
        Ops.set(
          @map_timesync,
          Builtins.sformat("timesrcntp%1", tempint),
          tempbool1
        )
      end
      tempbool1 = false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Time Synchronization\".\"Time Source 3\""
        )
      )
      if tempstring != ""
        if Builtins.issubstring(tempstring, ":123")
          tempbool1 = true
          tempstring = Builtins.substring(
            tempstring,
            0,
            Ops.subtract(Builtins.size(tempstring), 4)
          )
        end
        tempint = Ops.add(tempint, 1)
        Ops.set(
          @map_timesync,
          Builtins.sformat("timesource%1", tempint),
          tempstring
        )
        Ops.set(
          @map_timesync,
          Builtins.sformat("timesrcntp%1", tempint),
          tempbool1
        )
      end
      if Ops.greater_than(tempint, 0) && !tempbool
        tempbool = true
        Ops.set(@map_timesync, "timesync", true)
      end
      finished = false if tempbool && tempint == 0
      Ops.set(@map_timesync, "complete", finished)

      #  ******************* Install Settings ******************************************************************************
      finished = true
      tempint = 0 # true
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Reboot\""
        )
      )
      if tempstring == ""
        finished = false
      else
        tempint = 1 if Builtins.toupper(tempstring) == "FALSE"
        Ops.set(@map_install, "reboot", tempint)
      end
      tempint = 1 # false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Install Options\".\"Use Unsupported Drivers\""
        )
      )
      if tempstring == ""
        finished = false
      else
        tempint = 0 if Builtins.toupper(tempstring) == "TRUE"
        Ops.set(@map_install, "drivers", tempint)
      end
      tempint = 0 # true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Install Options\".\"Express\"")
      )
      if tempstring == ""
        finished = false
      else
        tempint = 1 if Builtins.toupper(tempstring) == "FALSE"
        Ops.set(@map_install, "express", tempint)
      end
      tempint = 0 # true
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Server Settings\".\"Load server at reboot\""
        )
      )
      if tempstring == ""
        finished = false
      else
        tempint = 1 if Builtins.toupper(tempstring) == "FALSE"
        Ops.set(@map_install, "autoexec", tempint)
      end

      tempint = 1
      tempstring1 = Ops.add("PrependLine ", Builtins.tostring(tempint))
      temppath = Builtins.add(
        path(".responsefile.template.value.\"NWI:Add To Startup\""),
        tempstring1
      )

      tempstring = ReadRF(temppath)
      while tempstring != ""
        Ops.set(
          @map_install,
          "setparam",
          Builtins.add(Ops.get_list(@map_install, "setparam", []), tempstring)
        )
        tempint = Ops.add(tempint, 1)
        tempstring1 = Ops.add("PrependLine ", Builtins.tostring(tempint))
        temppath = Builtins.add(
          path(".responsefile.template.value.\"NWI:Add To Startup\""),
          tempstring1
        )
        tempstring = ReadRF(temppath)
      end

      Ops.set(@map_install, "complete", finished)

      #  ******************* Upgrade Settings ******************************************************************************
      finished = true
      tempint = 0 # true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Install Options\".\"Backup\"")
      )
      if tempstring == ""
        finished = false
      else
        tempint = 1 if Builtins.toupper(tempstring) == "FALSE"
        Ops.set(@map_install, "backup", tempint)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Install Options\".\"Backup Dir\""
        )
      )
      if tempstring == "" && tempint == 0
        finished = false
      else
        Ops.set(@map_install, "backuploc", tempstring)
      end
      Ops.set(@map_install, "complete1", finished)

      #  ******************* Storage Settings ******************************************************************************
      finished = false
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"NWI:Boot Partition\".\"Dos Partition Size\""
        )
      )
      if tempstring != ""
        tempint = Builtins.tointeger(tempstring)
        if tempint != nil
          Ops.set(@map_storage, "fatpart", tempint)
          finished = true
        end
      end
      Ops.set(@map_storage, "complete", finished)
      finished = false
      tempstring = ReadRF(
        path(".responsefile.template.value.\"NWI:Pool Sys\".\"Sys Pool Size\"")
      )
      if tempstring != ""
        tempint = Builtins.tointeger(tempstring)
        if tempint != nil
          Ops.set(@map_storage, "sysvol", tempint)
          finished = true
        end
      end
      Ops.set(@map_storage, "complete1", finished)

      #  ******************* Pattern Selection *****************************************************************************
      finished = true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"Selected Nodes\".\"Patterns\"")
      )
      if tempstring == ""
        finished = false
      else
        tempint = 0
        tempint1 = Builtins.size(@list_patterns)
        while Ops.less_than(tempint, tempint1)
          if Builtins.toupper(Ops.get(@list_patterns, tempint, "")) ==
              Builtins.toupper(tempstring)
            break
          else
            tempint = Ops.add(tempint, 1)
          end
        end
        if Ops.greater_or_equal(tempint, tempint1)
          tempint = 0
          finished = false
        else
          Ops.set(@map_patterns, "pattern", tempint)
        end
      end
      Ops.set(@map_patterns, "complete", finished)

      #  ******************* Product Selection *****************************************************************************
      finished = true
      tempstring = ReadRF(
        path(".responsefile.template.value.\"Selected Nodes\".\"Products\"")
      )
      if tempstring == ""
        finished = false
      else
        templist = Builtins.splitstring(tempstring, ",")
        tempint = 0
        tempint1 = Builtins.size(templist)
        tempint3 = Builtins.size(@list_products)
        while Ops.less_than(tempint, tempint1)
          tempint2 = 0
          while Ops.less_than(tempint2, tempint3)
            if Builtins.toupper(Ops.get(templist, tempint, "0")) ==
                Builtins.toupper(Ops.get(@list_products, tempint2, "1"))
              Ops.set(@list_products_select, tempint2, true)
              break
            else
              tempint2 = Ops.add(tempint2, 1)
            end
          end
          tempint = Ops.add(tempint, 1)
        end
      end
      Ops.set(@map_products, "complete", finished)

      #  ******************* NMAS Configuration *****************************************************************************
      finished = true
      tempint = 0
      tempint3 = 0
      tempstring1 = Ops.add("Method ", Builtins.tostring(tempint))
      temppath = Builtins.add(
        path(".responsefile.template.value.\"NWNMAS\""),
        tempstring1
      )
      #	y2milestone ("Responsefile::ImportRFTemplate() Path: %1", temppath);
      #	y2milestone ("Responsefile::ImportRFTemplate() String: %1  Integer: %2", tempstring1, tempint);

      tempstring = ReadRF(temppath)
      while tempstring != ""
        tempint1 = 0
        tempint2 = 0
        tempint1 = Builtins.size(templist)
        while Ops.less_than(tempint2, tempint1)
          if Builtins.toupper(Ops.get(@list_nmas, tempint2, "")) ==
              Builtins.toupper(tempstring)
            break
          else
            tempint2 = Ops.add(tempint2, 1)
          end
        end
        if Ops.greater_or_equal(tempint, tempint1)
          Builtins.y2milestone(
            "Responsefile::ImportRFTemplate() can't translate %1  value: %2",
            temppath,
            tempstring
          )
        else
          tempint3 = Ops.add(tempint3, 1)
          Ops.set(@list_nmas_select, tempint2, true) 
          #			y2milestone ("Responsefile::ImportRFTemplate() translated %1  value: %2", temppath, tempstring);
        end
        tempint = Ops.add(tempint, 1)
        tempstring1 = Ops.add("Method ", Builtins.tostring(tempint))
        temppath = Builtins.add(
          path(".responsefile.template.value.\"NWNMAS\""),
          tempstring1
        )
        tempstring = ReadRF(temppath)
      end
      finished = false if tempint3 == 0
      Ops.set(@map_nmas, "complete", finished)

      #  ******************* DNS/DHCP Configuration ************************************************************************
      finished = true
      tempint = 0
      tempstring = ReadRF(
        path(".responsefile.template.value.\"DNSDHCP\".\"GroupNDSContext\"")
      )
      if tempstring != ""
        Ops.set(@map_dns_dhcp, "locator", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"DNSDHCP\".\"LocatorNDSContext\"")
      )
      if tempstring != ""
        Ops.set(@map_dns_dhcp, "group", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"DNSDHCP\".\"RootSrvrNDSNDSContext\""
        )
      )
      if tempstring != ""
        Ops.set(@map_dns_dhcp, "zone", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      finished = false if tempint != 3
      Ops.set(@map_dns_dhcp, "complete", finished)

      # ******************** iFolder Configuration *************************************************************************
      finished = true
      tempint = 0
      tempstring = ReadRF(
        path(".responsefile.template.value.\"iFolder\".\"ifPrimaryLdapHost\"")
      )
      if tempstring != ""
        Ops.set(@map_ifolder, "ldaphost", tempstring)
        tempint = Ops.add(tempint, 1)
      elsif Ops.get_string(@map_dest_dns, "hostname", "") != "" &&
          Ops.get_string(@map_dest_dns, "domain", "") != ""
        Ops.set(
          @map_ifolder,
          "ldaphost",
          Ops.add(
            Ops.add(Ops.get_string(@map_dest_dns, "hostname", ""), "."),
            Ops.get_string(@map_dest_dns, "domain", "")
          )
        )
      end

      tempstring = ReadRF(
        path(".responsefile.template.value.\"iFolder\".\"ifPrimaryLdapPort\"")
      )
      if tempstring != ""
        Ops.set(@map_ifolder, "ldapport", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"iFolder\".\"ifPrimaryLdapLoginDNContext\""
        )
      )
      if tempstring != ""
        Ops.set(@map_ifolder, "ldapcontext", tempstring)
        tempint = Ops.add(tempint, 1)
      else
        tempstring = Ops.get_string(@map_dest_edir, "destadminctx", "")
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"iFolder\".\"ifHostName\"")
      )
      if tempstring != ""
        Ops.set(@map_ifolder, "ifolderhost", tempstring)
        tempint = Ops.add(tempint, 1)
      elsif Ops.get_string(@map_dest_dns, "hostname", "") != "" &&
          Ops.get_string(@map_dest_dns, "domain", "") != ""
        Ops.set(
          @map_ifolder,
          "ldaphost",
          Ops.add(
            Ops.add(Ops.get_string(@map_dest_dns, "hostname", ""), "."),
            Ops.get_string(@map_dest_dns, "domain", "")
          )
        )
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"iFolder\".\"ifFolderAdminName\"")
      )
      if tempstring != ""
        Ops.set(@map_ifolder, "ifolderadmin", tempstring)
        tempint = Ops.add(tempint, 1)
      else
        Ops.set(
          @map_ifolder,
          "ifolderadmin",
          Ops.get_string(@map_dest_edir, "destadminname", "")
        )
      end

      tempstring = ReadRF(
        path(".responsefile.template.value.\"iFolder\".\"ifDataDirectory\"")
      )
      if tempstring != ""
        Ops.set(@map_ifolder, "dbpath", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      finished = false if tempint != 6
      Ops.set(@map_ifolder, "complete", finished)

      #  ******************* MySQL Configuration****************************************************************************
      tempint = 0
      tempstring = ReadRF(
        path(".responsefile.template.value.\"MySQL\".\"dataDirectory\"")
      )
      if tempstring != ""
        Ops.set(@map_mysql, "datadir", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"MySQL\".\"rootPassword\"")
      )
      if tempstring != ""
        Ops.set(@map_mysql, "password", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"MySQL\".\"secureInstall\"")
      )
      if tempstring != ""
        if Builtins.toupper(tempstring) == "FALSE"
          Ops.set(@map_mysql, "secure", false)
        end
        tempint = Ops.add(tempint, 1)
      end
      finished = false if tempint != 3
      Ops.set(@map_mysql, "complete", finished)

      #  ******************* exteNd Application Server Configuration *******************************************************
      finished = true
      tempint = 0
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"Admin User\"")
      )
      if tempstring != ""
        Ops.set(@map_extend, "adminname", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"Admin Password\"")
      )
      if tempstring != ""
        Ops.set(@map_extend, "password", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"HTTP Port\"")
      )
      if tempstring != ""
        Ops.set(@map_extend, "httpport", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"Restrict Access\"")
      )
      if tempstring != ""
        if Builtins.toupper(tempstring) == "FALSE"
          Ops.set(@map_extend, "restrict", false)
        end
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"Hostname\"")
      )
      if tempstring != ""
        Ops.set(@map_extend, "dbhost", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"Port\"")
      )
      if tempstring != ""
        Ops.set(@map_extend, "mysqlport", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"Database User\"")
      )
      if tempstring != ""
        Ops.set(@map_extend, "dbusername", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"Database Password\"")
      )
      if tempstring != ""
        Ops.set(@map_extend, "dbpassword", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"j2ee\".\"SilverMaster Name\"")
      )
      if tempstring != ""
        Ops.set(@map_extend, "smname", tempstring)
        tempint = Ops.add(tempint, 1)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"j2ee\".\"Execute SilverMasterInit\""
        )
      )
      if tempstring != ""
        if Builtins.toupper(tempstring) == "FALSE"
          Ops.set(@map_extend, "executesmi", false)
        end
        tempint = Ops.add(tempint, 1)
      end
      finished = false if tempint != 10
      Ops.set(@map_extend, "complete", finished)

      #  ******************* Save Response File ****************************************************************************
      finished = true
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"RF Generator\".\"Response File Local Path\""
        )
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_save_rf, "tmprfpath", tempstring)
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"RF Generator\".\"Response File Name\""
        )
      )
      if tempstring == ""
        finished = false
      else
        Ops.set(@map_save_rf, "rfname", tempstring)
      end
      if hardwaretype == 2 # in VM mode the path is always A:/
        tempstring = ReadRF(
          path(
            ".responsefile.template.value.\"RF Generator\".\"Response File Path\""
          )
        )
        if tempstring == ""
          finished = false
        else
          Ops.set(@map_save_rf, "rfpath", tempstring)
        end
      end
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"RF Generator\".\"Source Location Path\""
        )
      )
      Ops.set(@map_save_rf, "srclocpath", tempstring) if tempstring != ""
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"RF Generator\".\"Destination Location Path\""
        )
      )
      Ops.set(@map_save_rf, "destlocpath", tempstring) if tempstring != ""
      tempstring = ReadRF(
        path(
          ".responsefile.template.value.\"RF Generator\".\"Launch VM Creator\""
        )
      )
      if tempstring != ""
        if Builtins.toupper(tempstring) == "TRUE"
          Ops.set(@map_branch_type, "launchvm", true)
        end
      end
      tempstring = ReadRF(
        path(".responsefile.template.value.\"RF Generator\".\"Custom VM Name\"")
      )
      Ops.set(@map_save_rf, "vmname", tempstring) if tempstring != ""
      Ops.set(@map_save_rf, "complete", finished)

      true
    end

    def ExportRFile
      Builtins.y2milestone("Responsefile::ExportRFile() started")
      ret = false
      tempstring = ""
      templist = []
      installtype = Ops.get_integer(@map_select_type, "installtype", 0)
      hardwaretype = Ops.get_integer(@map_select_type, "hardwaretype", 0)

      #  Select *****************************************************************************
      if Ops.get_boolean(@map_select_type, "visible", false) == true
        if installtype == 4 # Factory Install
          WriteRF(
            path(".responsefile.rfile.value.\"RF Generator\".\"Install Type\""),
            "Factory"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Initialization\".\"Install State\""
            ),
            "Second"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Upgrade\""
            ),
            "FALSE"
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Factory\".\"Image Scope\""),
            "3"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Factory\".\"Copy Image Files\""
            ),
            "FALSE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Factory\".\"Run Install From Image\""
            ),
            "TRUE"
          )
        elsif installtype == 5 # Image Install
          WriteRF(
            path(".responsefile.rfile.value.\"RF Generator\".\"Install Type\""),
            "Image"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Initialization\".\"Install State\""
            ),
            "First"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Upgrade\""
            ),
            "FALSE"
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Factory\".\"Image Scope\""),
            "3"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Factory\".\"Copy Image Files\""
            ),
            "TRUE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Factory\".\"Run Install From Image\""
            ),
            "FALSE"
          )
        elsif installtype == 2 # New Install
          WriteRF(
            path(".responsefile.rfile.value.\"RF Generator\".\"Install Type\""),
            "New"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Initialization\".\"Install State\""
            ),
            "First"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Upgrade\""
            ),
            "FALSE"
          )
        else
          if installtype == 3 # Local Upgrade
            WriteRF(
              path(
                ".responsefile.rfile.value.\"RF Generator\".\"Install Type\""
              ),
              "Upgrade Local"
            )
          else
            WriteRF(
              path(
                ".responsefile.rfile.value.\"RF Generator\".\"Install Type\""
              ),
              "Upgrade Remote"
            )
          end
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Upgrade\""
            ),
            "TRUE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Health Check\".\"Display Selection Screen\""
            ),
            "FALSE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Health Check\".\"Display Summary Screen\""
            ),
            "FALSE"
          )
          WriteRF(
            path(".responsefile.rfile.value.\"Health Check\".\"Prompt\""),
            "FALSE"
          )
        end
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"silent\""
          ),
          "TRUE"
        )
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Prompt\""
          ),
          "FALSE"
        )
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"closeScreen\""
          ),
          "SilentCloseScreen"
        )
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"allowCloseScreen\""
          ),
          "TRUE"
        )
        if installtype != 3 # Not Upgrade
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Reboot\""
            ),
            "TRUE"
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Install Options\".\"Prompt\""),
          "FALSE"
        )
      end

      #  Source Address *****************************************************************************
      if Ops.get_boolean(@map_src_addr, "visible", false) == true
        if Ops.get_integer(@map_src_addr, "type", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:MISC\".\"Source Server Name\""
            ),
            Ops.get_string(@map_src_addr, "srcname", "")
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:MISC\".\"Source Location Path\""
            ),
            Ops.get_string(@map_src_addr, "srclocpath", "")
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:MISC\".\"Prompt\""),
            "FALSE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:MISC\".\"Source Domain Address\""
            ),
            Ops.get_string(@map_src_addr, "srcipaddr", "")
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:MISC\".\"Source Domain Address Type\""
            ),
            "UDP"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"RF Generator\".\"NetWare Source Location\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"RF Generator\".\"NetWare Source Location\""
            ),
            "FALSE"
          )
        end
      end

      #  Source eDirectory *****************************************************************************
      if Ops.get_boolean(@map_src_edir, "visible", false) == true
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:MISC\".\"Source Tree Name\""),
          Ops.get_string(@map_src_edir, "srctreename", "")
        )
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NWI:MISC\".\"Source Server Context\""
          ),
          Ops.get_string(@map_src_edir, "srcsrvctx", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:MISC\".\"Source User Name\""),
          Ops.get_string(@map_src_edir, "srcusername", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:MISC\".\"Source Context\""),
          Ops.get_string(@map_src_edir, "srcuserctx", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:MISC\".\"Relogin Password\""),
          Ops.get_string(@map_src_edir, "srcpassword", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:MISC\".\"Prompt\""),
          "FALSE"
        )
      end

      #  Destination Address *****************************************************************************
      if Ops.get_boolean(@map_dest_addr, "visible", false) == true
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:File Server\".\"Servername\""),
          Ops.get_string(@map_dest_addr, "destname", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:File Server\".\"Prompt\""),
          "FALSE"
        )
        if installtype == 3 # Upgrade
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:NDS\".\"Domain Address\""),
            Ops.get_string(@map_dest_addr, "destipaddr", "")
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:NDS\".\"Domain Address Type\""
            ),
            "UDP"
          )
        end
      end

      #  Destination eDirectory *****************************************************************************
      if Ops.get_boolean(@map_dest_edir, "visible", false) == true
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:NDS\".\"Tree Name\""),
          Ops.get_string(@map_dest_edir, "destreename", "")
        )
        if Ops.get_boolean(@map_dest_edir, "destnewtree", false)
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:NDS\".\"New Tree\""),
            "TRUE"
          )
        else
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:NDS\".\"New Tree\""),
            "FALSE"
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:NDS\".\"Server Context\""),
          Ops.get_string(@map_dest_edir, "destsrvctx", "")
        )
        if Ops.get_string(@map_dest_edir, "replicaipaddr", "") != ""
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:NDS\".\"Replica Server IP Address\""
            ),
            Ops.get_string(@map_dest_edir, "replicaipaddr", "")
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:NDS\".\"Admin Context\""),
          Ops.get_string(@map_dest_edir, "destadminctx", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:NDS\".\"Admin Login Name\""),
          Ops.get_string(@map_dest_edir, "destadminname", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:NDS\".\"Admin Password\""),
          Ops.get_string(@map_dest_edir, "destpassword", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:NDS\".\"Prompt\""),
          "FALSE"
        )
      end

      # License Configuration *****************************************************************************
      if Ops.get_boolean(@map_dest_edir, "visible", false) == true
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NWI:License\".\"MLA License Context\""
          ),
          Ops.get_string(@map_license, "context", "")
        )
        if Ops.get_boolean(@map_license, "nolicense", false)
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:License\".\"Install Licenses Later\""
            ),
            "TRUE"
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:License\".\"Licenses File\""),
            ""
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:License\".\"Install Licenses Later\""
            ),
            "FALSE"
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:License\".\"Licenses File\""),
            Ops.get_string(@map_save_rf, "rfpath", "")
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:License\".\"Prompt\""),
          "FALSE"
        )
      end

      #  Protocols *****************************************************************************
      if Ops.get_boolean(@map_dest_addr, "visible1", false) == true
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NWI:Protocols\".\"Single Network Adapter\""
          ),
          "TRUE"
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Protocols\".\"Prompt\""),
          "FALSE"
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:TCPIP\".\"Prompt\""),
          "FALSE"
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:IPX\".\"Prompt\""),
          "FALSE"
        )
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NWI:File Server\".\"Server ID Number\""
          ),
          Ops.get_string(@map_dest_addr, "destipxid", "")
        )
        if Ops.get_boolean(@map_dest_addr, "selectip", false)
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Protocols\".\"Default IP Frame Type\""
            ),
            "Ethernet_II"
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:TCPIP\".\"IP Address 1\""),
            Ops.get_string(@map_dest_addr, "destipaddr", "")
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Host Name\".\"IP Address 1\""),
            Ops.get_string(@map_dest_addr, "destipaddr", "")
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:TCPIP\".\"Subnet Mask 1\""),
            Ops.get_string(@map_dest_addr, "destsubnet", "")
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:TCPIP\".\"Gateway 1\""),
            Ops.get_string(@map_dest_addr, "destrouter", "")
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Protocols\".\"Default IP Frame Type\""
            ),
            ""
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:TCPIP\".\"IP Address 1\""),
            ""
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Host Name\".\"IP Address 1\""),
            ""
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:TCPIP\".\"Subnet Mask 1\""),
            ""
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:TCPIP\".\"Gateway 1\""),
            ""
          )
        end
        if Ops.get_boolean(@map_dest_addr, "selectipx", false)
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Protocols\".\"Default IPX Frame Type\""
            ),
            Ops.get(
              @list_frame_types,
              Ops.get_integer(@map_dest_addr, "ipxframe", 0),
              ""
            )
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:IPX\".\"Bind LAN Card 1\""),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Protocols\".\"Default IPX Frame Type\""
            ),
            ""
          )
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:IPX\".\"Bind LAN Card 1\""),
            "FALSE"
          )
        end
      end

      #  Language *****************************************************************************
      if Ops.get_boolean(@map_dest_lang, "visible", false) == true
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Locale\".\"Country Code\""),
          Ops.get(
            @list_countrycode,
            Ops.get_integer(@map_dest_lang, "countrycode", 0),
            ""
          )
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Locale\".\"Code Page\""),
          Ops.get(
            @list_codepage,
            [
              Ops.get_integer(@map_dest_lang, "countrycode", 0),
              Ops.get_integer(@map_dest_lang, "codepage", 0)
            ],
            ""
          )
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Locale\".\"Keyboard\""),
          Ops.get(
            @list_keyboardcode_en,
            Ops.get_integer(@map_dest_lang, "keyboardcode", 0),
            ""
          )
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Language\".\"Server Language\""),
          Ops.get_string(
            @list_language_number,
            Ops.get_integer(@map_dest_lang, "language", 0),
            ""
          )
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:NDS\".\"Admin Language\""),
          Ops.get_string(
            @list_language_number,
            Ops.get_integer(@map_dest_lang, "language", 0),
            ""
          )
        )
      end

      #  SLP Configuration *****************************************************************************
      if Ops.get_boolean(@map_dest_slp, "visible", false) == true
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:SLP Configuration\".\"Prompt\""),
          "FALSE"
        )
        if Ops.get_boolean(@map_dest_slp, "skipslp", false)
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:SLP Configuration\".\"Skip SLP\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:SLP Configuration\".\"Skip SLP\""
            ),
            "FALSE"
          )
          if Ops.get_boolean(@map_dest_slp, "diragent", false)
            WriteRF(
              path(
                ".responsefile.rfile.value.\"NWI:SLP Configuration\".\"Directory Agent\""
              ),
              "TRUE"
            )
          else
            WriteRF(
              path(
                ".responsefile.rfile.value.\"NWI:SLP Configuration\".\"Directory Agent\""
              ),
              "FALSE"
            )
          end
          if Ops.get_boolean(@map_dest_slp, "multicast", false)
            WriteRF(
              path(
                ".responsefile.rfile.value.\"NWI:SLP Configuration\".\"Multicast Routing\""
              ),
              "TRUE"
            )
          else
            WriteRF(
              path(
                ".responsefile.rfile.value.\"NWI:SLP Configuration\".\"Multicast Routing\""
              ),
              "FALSE"
            )
          end
          defkey = path(".responsefile.rfile.value.\"NWI:SLP Configuration\"")
          num = 0
          if Ops.get_string(@map_dest_slp, "daserver1", "") != ""
            num = Ops.add(num, 1)
            WriteRF(
              Builtins.add(defkey, Builtins.sformat("DA Server %1", num)),
              Ops.get_string(@map_dest_slp, "daserver1", "")
            )
          end
          if Ops.get_string(@map_dest_slp, "daserver2", "") != ""
            num = Ops.add(num, 1)
            WriteRF(
              Builtins.add(defkey, Builtins.sformat("DA Server %1", num)),
              Ops.get_string(@map_dest_slp, "daserver2", "")
            )
          end
          if Ops.get_string(@map_dest_slp, "daserver3", "") != ""
            num = Ops.add(num, 1)
            WriteRF(
              Builtins.add(defkey, Builtins.sformat("DA Server %1", num)),
              Ops.get_string(@map_dest_slp, "daserver3", "")
            )
          end
          if Ops.get_string(@map_dest_slp, "slpscopelist", "") != ""
            WriteRF(
              path(
                ".responsefile.rfile.value.\"NWI:SLP Configuration\".\"SLP Scope List\""
              ),
              Ops.get_string(@map_dest_slp, "slpscopelist", "")
            )
          end
        end
      end

      #  DNS Configuration *****************************************************************************
      if Ops.get_boolean(@map_dest_dns, "visible", false) == true
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:DNS\".\"Domain\""),
          Ops.get_string(@map_dest_dns, "domain", "")
        )
        tempstring = ""
        if Ops.get_string(@map_dest_dns, "dnserver1", "") != ""
          tempstring = Ops.add(
            tempstring,
            Ops.get_string(@map_dest_dns, "dnserver1", "")
          )
        end
        if Ops.get_string(@map_dest_dns, "dnserver2", "") != ""
          tempstring = Ops.add(tempstring, ",") if tempstring != ""
          tempstring = Ops.add(
            tempstring,
            Ops.get_string(@map_dest_dns, "dnserver2", "")
          )
        end
        if Ops.get_string(@map_dest_dns, "dnserver3", "") != ""
          tempstring = Ops.add(tempstring, ",") if tempstring != ""
          tempstring = Ops.add(
            tempstring,
            Ops.get_string(@map_dest_dns, "dnserver3", "")
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:DNS\".\"Nameservers\""),
          tempstring
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:DNS\".\"Prompt\""),
          "FALSE"
        )
        if Ops.get_boolean(@map_dest_dns, "verifydns", false)
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Host Name\".\"Verify\""),
            "TRUE"
          )
        else
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Host Name\".\"Verify\""),
            "FALSE"
          )
        end
        tempstring = Ops.add(
          Ops.add(Ops.get_string(@map_dest_dns, "hostname", ""), "."),
          Ops.get_string(@map_dest_dns, "domain", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Host Name\".\"Host Name 1\""),
          tempstring
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Host Name\".\"Prompt\""),
          "FALSE"
        )
      end

      # Time Zone Configuration *****************************************************************************
      if Ops.get_boolean(@map_timezones, "visible", false) == true
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Time Zone\".\"Time Zone\""),
          Ops.get(
            @list_timezones,
            Ops.get_integer(@map_timezones, "timezone", 0),
            ""
          )
        )
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Time Zone\".\"Prompt\""),
          "FALSE"
        )
        if Ops.get_boolean(@map_timezones, "daylightsaving", false)
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Time Zone\".\"Use Daylight Saving Time\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Time Zone\".\"Use Daylight Saving Time\""
            ),
            "FALSE"
          )
        end
      end

      # Time Sync Configuration *****************************************************************************
      if Ops.get_boolean(@map_dest_dns, "visible", false) == true
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NWI:Time Synchronization\".\"Prompt\""
          ),
          "FALSE"
        )
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NWI:Time Synchronization\".\"Time Server Protocol\""
          ),
          Ops.get(
            @list_timeserver_protocol,
            Ops.get_integer(@map_timesync, "protocol", 0),
            ""
          )
        )
        if Ops.get_integer(@map_timesync, "protocol", 0) == 1
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Time Synchronization\".\"Time Server Type\""
            ),
            Ops.get(
              @list_timeserver_type,
              Ops.get_integer(@map_timesync, "servertype", 0),
              ""
            )
          )
        end
        num = 0
        if Ops.get_boolean(@map_timesync, "timesync", false)
          tempstring2 = ""
          defkey = path(
            ".responsefile.rfile.value.\"NWI:Time Synchronization\""
          )
          if Ops.get_string(@map_timesync, "timesource1", "") != ""
            tempstring2 = Ops.get_string(@map_timesync, "timesource1", "")
            num = Ops.add(num, 1)
            if Ops.get_boolean(@map_timesync, "timesrcntp1", false) &&
                !Builtins.issubstring(tempstring2, ":123")
              tempstring2 = Ops.add(
                Ops.get_string(@map_timesync, "timesource1", ""),
                ":123"
              )
            end
            WriteRF(
              Builtins.add(defkey, Builtins.sformat("Time Source %1", num)),
              tempstring2
            )
          end
          if Ops.get_string(@map_timesync, "timesource2", "") != ""
            tempstring2 = Ops.get_string(@map_timesync, "timesource2", "")
            num = Ops.add(num, 1)
            if Ops.get_boolean(@map_timesync, "timesrcntp2", false) &&
                !Builtins.issubstring(tempstring2, ":123")
              tempstring2 = Ops.add(
                Ops.get_string(@map_timesync, "timesource2", ""),
                ":123"
              )
            else
              WriteRF(
                Builtins.add(defkey, Builtins.sformat("Time Source %1", num)),
                tempstring2
              )
            end
          end
          if Ops.get_string(@map_timesync, "timesource3", "") != ""
            tempstring2 = Ops.get_string(@map_timesync, "timesource3", "")
            num = Ops.add(num, 1)
            if Ops.get_boolean(@map_timesync, "timesrcntp3", false) &&
                !Builtins.issubstring(tempstring2, ":123")
              tempstring2 = Ops.add(
                Ops.get_string(@map_timesync, "timesource3", ""),
                ":123"
              )
            end
            WriteRF(
              Builtins.add(defkey, Builtins.sformat("Time Source %1", num)),
              tempstring2
            )
          end
        end
        if Ops.greater_than(num, 0)
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Time Synchronization\".\"Use Time Source\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Time Synchronization\".\"Use Time Source\""
            ),
            "FALSE"
          )
        end
      end

      # Install Settings *****************************************************************************
      if Ops.get_boolean(@map_install, "visible", false) == true
        if Ops.get_integer(@map_install, "reboot", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Reboot\""
            ),
            "TRUE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"closeScreen\""
            ),
            "SilentCloseScreen"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Reboot\""
            ),
            "FALSE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"closeScreen\""
            ),
            "NWICloseScreen"
          )
        end
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Prompt\""
          ),
          "FALSE"
        )
        if Ops.get_integer(@map_install, "drivers", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Use Unsupported Drivers\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Use Unsupported Drivers\""
            ),
            "FALSE"
          )
        end
        if Ops.get_integer(@map_install, "express", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Express\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Express\""
            ),
            "FALSE"
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Install Options\".\"Prompt\""),
          "FALSE"
        )
        if Ops.get_integer(@map_install, "autoexec", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Server Settings\".\"Load server at reboot\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Server Settings\".\"Load server at reboot\""
            ),
            "FALSE"
          )
        end
      end

      # Upgrade Settings *****************************************************************************
      if Ops.get_boolean(@map_install, "visible1", false) == true
        if Ops.get_integer(@map_install, "reboot", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Reboot\""
            ),
            "TRUE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"closeScreen\""
            ),
            "SilentCloseScreen"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Reboot\""
            ),
            "FALSE"
          )
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"closeScreen\""
            ),
            "NWICloseScreen"
          )
        end
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NOVELL:NOVELL_ROOT:1.0.0\".\"Prompt\""
          ),
          "FALSE"
        )
        if Ops.get_integer(@map_install, "backup", 0) == 0
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Install Options\".\"Backup\""),
            "TRUE"
          )
        else
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Install Options\".\"Backup\""),
            "FALSE"
          )
        end
        WriteRF(
          path(
            ".responsefile.rfile.value.\"NWI:Install Options\".\"Backup Dir\""
          ),
          Ops.get_string(@map_install, "backuploc", "")
        )
        if Ops.get_integer(@map_install, "drivers", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Use Unsupported Drivers\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Use Unsupported Drivers\""
            ),
            "FALSE"
          )
        end
        if Ops.get_integer(@map_install, "express", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Express\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Install Options\".\"Express\""
            ),
            "FALSE"
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Install Options\".\"Prompt\""),
          "FALSE"
        )
        if Ops.get_integer(@map_install, "autoexec", 0) == 0
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Server Settings\".\"Load server at reboot\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Server Settings\".\"Load server at reboot\""
            ),
            "FALSE"
          )
        end
      end

      if Ops.get_boolean(@map_install, "visible", false) == true ||
          Ops.get_boolean(@map_install, "visible1", false) == true
        tempint = 0
        templist2 = Ops.get_list(@map_install, "setparam", [])
        tempint1 = Builtins.size(templist2)
        tempint2 = 1
        tempstring2 = ""
        while Ops.less_than(tempint, tempint1)
          tempstring2 = Ops.add("PrependLine ", Builtins.tostring(tempint2))
          temppath = Builtins.add(
            path(".responsefile.rfile.value.\"NWI:Add To Startup\""),
            tempstring2
          )
          WriteRF(temppath, Ops.get(templist2, tempint, ""))
          tempint = Ops.add(tempint, 1)
          tempint2 = Ops.add(tempint2, 1)
        end
      end

      # Storage Settings *****************************************************************************
      if Ops.get_boolean(@map_storage, "visible", false) == true
        if Ops.get_boolean(@map_storage, "complete", false)
          WriteRF(
            path(
              ".responsefile.rfile.value.\"NWI:Boot Partition\".\"Dos Partition Size\""
            ),
            Builtins.tostring(Ops.get_integer(@map_storage, "fatpart", 0))
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Boot Partition\".\"Prompt\""),
          "FALSE"
        )

        if Ops.get_boolean(@map_storage, "complete1", false)
          WriteRF(
            path(".responsefile.rfile.value.\"NWI:Pool Sys\".\"Sys Pool Size\""),
            Builtins.tostring(Ops.get_integer(@map_storage, "sysvol", 0))
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWI:Pool Sys\".\"Prompt\""),
          "FALSE"
        )
      end

      # Pattern Selection *****************************************************************************
      if Ops.get_boolean(@map_patterns, "visible", false) == true
        WriteRF(
          path(".responsefile.rfile.value.\"Selected Nodes\".\"Patterns\""),
          Ops.get(
            @list_patterns,
            Ops.get_integer(@map_patterns, "pattern", 0),
            ""
          )
        )
        if Ops.get_integer(@map_patterns, "pattern", 0) != 0
          tempstring = ListToString(
            MergeLists(
              Ops.get(@list_pattern_products, 0, []),
              Ops.get(
                @list_pattern_products,
                Ops.get_integer(@map_patterns, "pattern", 0),
                []
              )
            )
          )
          WriteRF(
            path(".responsefile.rfile.value.\"Selected Nodes\".\"Products\""),
            tempstring
          )
        end
      end

      # Product Selection *****************************************************************************
      if Ops.get_boolean(@map_products, "visible", false) == true
        mylist = []
        pos = 0
        Builtins.foreach(@list_products) do |val|
          if Ops.get(@list_products_select, pos, false) ||
              Ops.get(@list_products_nc, pos, false)
            mylist = Builtins.add(mylist, Ops.get(@list_products, pos, ""))
          end
          pos = Ops.add(pos, 1)
        end
        WriteRF(
          path(".responsefile.rfile.value.\"Selected Nodes\".\"Products\""),
          ListToString(mylist)
        )
      end

      # if vm upgrade then remove the "DOS_INST" and "SYS_INST" from [Selected Nodes] "NetWare OS" key list.
      if installtype == 1 && hardwaretype != 2
        mylist = []
        tempstring = ReadRF(
          path(".responsefile.rfile.value.\"Selected Nodes\".\"NetWare OS\"")
        )
        if tempstring != ""
          mylist = Builtins.splitstring(tempstring, ",")
          mylist = Builtins.filter(mylist) do |str|
            str != "DOS_INST" && str != "SYS_INST"
          end
        end
        tempstring = ListToString(mylist)
        WriteRF(
          path(".responsefile.rfile.value.\"Selected Nodes\".\"NetWare OS\""),
          ListToString(mylist)
        )
      end

      # NMAS Configuration *****************************************************************************
      if Ops.get_boolean(@map_nmas, "visible", false)
        tempint = 0
        tempint1 = Builtins.size(@list_nmas_select)
        tempint2 = 0
        tempstring2 = ""
        while Ops.less_than(tempint, tempint1)
          if Ops.get(@list_nmas_select, tempint, false)
            tempstring2 = Ops.add("Method ", Builtins.tostring(tempint2))
            temppath = Builtins.add(
              path(".responsefile.rfile.value.\"NWNMAS\""),
              tempstring2
            )
            WriteRF(temppath, Ops.get(@list_nmas, tempint, ""))
            tempint2 = Ops.add(tempint2, 1)
          end
          tempint = Ops.add(tempint, 1)
        end
        WriteRF(
          path(".responsefile.rfile.value.\"NWNMAS\".\"Prompt\""),
          "FALSE"
        )
      end

      # DNS/DHCP Configuration *****************************************************************************
      if Ops.get_boolean(@map_dns_dhcp, "visible", false)
        WriteRF(
          path(".responsefile.rfile.value.\"DNSDHCP\".\"LocatorNDSContext\""),
          Ops.get_string(@map_dns_dhcp, "locator", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"DNSDHCP\".\"GroupNDSContext\""),
          Ops.get_string(@map_dns_dhcp, "group", "")
        )
        WriteRF(
          path(
            ".responsefile.rfile.value.\"DNSDHCP\".\"RootSrvrNDSNDSContext\""
          ),
          Ops.get_string(@map_dns_dhcp, "zone", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"DNSDHCP\".\"Prompt\""),
          "FALSE"
        ) 
        #   y2milestone ("Responsefile::SelectTypeDialog() [DNSDHCP]LocatorNDSContext = %1", map_dns_dhcp["locator"]:"");
      end

      # iFolder Configuration *****************************************************************************
      if Ops.get_boolean(@map_ifolder, "visible", false)
        WriteRF(
          path(".responsefile.rfile.value.\"iFolder\".\"ifPrimaryLdapHost\""),
          Ops.get_string(@map_ifolder, "ldaphost", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"iFolder\".\"ifPrimaryLdapPort\""),
          Ops.get_string(@map_ifolder, "ldapport", "")
        )
        WriteRF(
          path(
            ".responsefile.rfile.value.\"iFolder\".\"ifPrimaryLdapLoginDNContext\""
          ),
          Ops.get_string(@map_ifolder, "ldapcontext", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"iFolder\".\"ifHostName\""),
          Ops.get_string(@map_ifolder, "ifolderhost", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"iFolder\".\"ifFolderAdminName\""),
          Ops.get_string(@map_ifolder, "ifolderadmin", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"iFolder\".\"ifDataDirectory\""),
          Ops.get_string(@map_ifolder, "dbpath", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"iFolder\".\"Prompt\""),
          "FALSE"
        )
      end

      # MySQL Configuration*****************************************************************************
      if Ops.get_boolean(@map_mysql, "visible", false)
        WriteRF(
          path(".responsefile.rfile.value.\"MySQL\".\"dataDirectory\""),
          Ops.get_string(@map_mysql, "datadir", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"MySQL\".\"rootPassword\""),
          Ops.get_string(@map_mysql, "password", "")
        )
        if Ops.get_boolean(@map_mysql, "secure", true)
          WriteRF(
            path(".responsefile.rfile.value.\"MySQL\".\"secureInstall\""),
            "TRUE"
          )
        else
          WriteRF(
            path(".responsefile.rfile.value.\"MySQL\".\"secureInstall\""),
            "FALSE"
          )
        end
        WriteRF(path(".responsefile.rfile.value.\"MySQL\".\"Prompt\""), "FALSE")
      end

      # exteNd Application Server Configuration ********************************************************
      if Ops.get_boolean(@map_extend, "visible", false)
        WriteRF(
          path(".responsefile.rfile.value.\"j2ee\".\"Admin User\""),
          Ops.get_string(@map_extend, "adminname", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"j2ee\".\"Admin Password\""),
          Ops.get_string(@map_extend, "password", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"j2ee\".\"HTTP Port\""),
          Ops.get_string(@map_extend, "httpport", "")
        )
        if Ops.get_boolean(@map_extend, "restrict", true)
          WriteRF(
            path(".responsefile.rfile.value.\"j2ee\".\"Restrict Access\""),
            "TRUE"
          )
        else
          WriteRF(
            path(".responsefile.rfile.value.\"j2ee\".\"Restrict Access\""),
            "FALSE"
          )
        end
        WriteRF(
          path(".responsefile.rfile.value.\"j2ee\".\"Hostname\""),
          Ops.get_string(@map_extend, "dbhost", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"j2ee\".\"Port\""),
          Ops.get_string(@map_extend, "mysqlport", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"j2ee\".\"Database User\""),
          Ops.get_string(@map_extend, "dbusername", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"j2ee\".\"Database Password\""),
          Ops.get_string(@map_extend, "dbpassword", "")
        )
        WriteRF(
          path(".responsefile.rfile.value.\"j2ee\".\"SilverMaster Name\""),
          Ops.get_string(@map_extend, "smname", "")
        )
        if Ops.get_boolean(@map_extend, "executesmi", true)
          WriteRF(
            path(
              ".responsefile.rfile.value.\"j2ee\".\"Execute SilverMasterInit\""
            ),
            "TRUE"
          )
        else
          WriteRF(
            path(
              ".responsefile.rfile.value.\"j2ee\".\"Execute SilverMasterInit\""
            ),
            "FALSE"
          )
        end
        WriteRF(path(".responsefile.rfile.value.\"j2ee\".\"Prompt\""), "FALSE")
      end

      # Save Response File *****************************************************************************
      WriteRF(
        path(".responsefile.rfile.value.\"Initialization\".\"Description\""),
        ""
      )
      WriteRF(
        path(".responsefile.rfile.value.\"Initialization\".\"DescriptionID\""),
        ""
      )
      WriteRF(
        path(
          ".responsefile.rfile.value.\"RF Generator\".\"Response File Name\""
        ),
        Ops.get_string(@map_save_rf, "rfname", "")
      )
      WriteRF(
        path(
          ".responsefile.rfile.value.\"RF Generator\".\"Response File Local Path\""
        ),
        Ops.get_string(@map_save_rf, "tmprfpath", "")
      )
      WriteRF(
        path(
          ".responsefile.rfile.value.\"RF Generator\".\"Response File Path\""
        ),
        Ops.get_string(@map_save_rf, "rfpath", "")
      )
      WriteRF(
        path(
          ".responsefile.rfile.value.\"RF Generator\".\"Source Location Path\""
        ),
        Ops.get_string(@map_save_rf, "srclocpath", "")
      )
      WriteRF(
        path(
          ".responsefile.rfile.value.\"RF Generator\".\"Destination Location Path\""
        ),
        Ops.get_string(@map_save_rf, "destlocpath", "")
      )
      if Ops.get_integer(@map_src_addr, "type", 0) != 1 &&
          Ops.get_integer(@map_src_addr, "type", 0) != 2
        tempstring = ReadRF(
          path(
            ".responsefile.rfile.value.\"Selected Nodes\".\"DestinationNode\""
          )
        )
        if tempstring == ""
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Selected Nodes\".\"DestinationNode\""
            ),
            "HealthChecks"
          )
        end
        tempstring = ReadRF(
          path(".responsefile.rfile.value.\"Selected Nodes\".\"HealthChecks\"")
        )
        if tempstring == ""
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Selected Nodes\".\"HealthChecks\""
            ),
            "NetWare Services"
          )
        end
        tempstring = ReadRF(
          path(
            ".responsefile.rfile.value.\"Selected Nodes\".\"NetWare Services\""
          )
        )
        if tempstring == ""
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Selected Nodes\".\"NetWare Services\""
            ),
            "NetWare OS,Patterns,Products,NWUpdateGroup"
          )
        end
        tempstring = ReadRF(
          path(".responsefile.rfile.value.\"Selected Nodes\".\"NetWare OS\"")
        )
        if tempstring == ""
          WriteRF(
            path(".responsefile.rfile.value.\"Selected Nodes\".\"NetWare OS\""),
            "NetWare65OS,NICI,Protocols,Time Zone,DS_Install,LicensePrompt,WO install,DOS_INST,SYS_INST"
          )
        end
        tempstring = ReadRF(
          path(".responsefile.rfile.value.\"Selected Nodes\".\"NWUpdateGroup\"")
        )
        if tempstring == ""
          WriteRF(
            path(
              ".responsefile.rfile.value.\"Selected Nodes\".\"NWUpdateGroup\""
            ),
            "NWUpdate"
          )
        end
      end
      if hardwaretype != 1
        WriteRF(
          path(
            ".responsefile.rfile.value.\"RF Generator\".\"NetWare Source Location\""
          ),
          "FALSE"
        )
        if Ops.get_string(@map_save_rf, "srclocpath", "") != ""
          WriteRF(
            path(
              ".responsefile.rfile.value.\"RF Generator\".\"Source Location Path\""
            ),
            Ops.get_string(@map_save_rf, "srclocpath", "")
          )
        end
        if Ops.get_string(@map_save_rf, "destlocpath", "") != ""
          WriteRF(
            path(
              ".responsefile.rfile.value.\"RF Generator\".\"Destination Location Path\""
            ),
            Ops.get_string(@map_save_rf, "destlocpath", "")
          )
        end
        if Ops.get_boolean(@map_branch_type, "launchvm", false)
          tempstring = "TRUE"
          WriteRF(
            path(
              ".responsefile.rfile.value.\"RF Generator\".\"VM Creator Launch String\""
            ),
            Ops.get_string(@map_branch_type, "launchcode", "")
          )
        else
          tempstring = "FALSE"
        end
        WriteRF(
          path(
            ".responsefile.rfile.value.\"RF Generator\".\"Launch VM Creator\""
          ),
          tempstring
        )
        if Ops.get_string(@map_save_rf, "vmname", "") != ""
          WriteRF(
            path(
              ".responsefile.rfile.value.\"RF Generator\".\"Custom VM Name\""
            ),
            Ops.get_string(@map_save_rf, "vmname", "")
          )
        end
      end
      WriteRF(path(".responsefile.rfile"), nil)
      true
    end

    def Summary
      summary = ""
      tempstr = ""
      installtype = Ops.get_integer(@map_select_type, "installtype", 0)

      if Ops.get_boolean(@map_src_addr, "visible", false)
        if Ops.get_boolean(@map_src_addr, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"srcAddr\">" + _("Source Address") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"srcAddr\">" + "*" +
              _("Source Address") + "</a></b></font>"
          )
        end
      end
      if Ops.get_boolean(@map_src_edir, "visible", false)
        if Ops.get_boolean(@map_src_edir, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"srceDir\">" + _("Source eDirectory") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"srceDir\">" + "*" +
              _("Source eDirectory") + "</a></b></font>"
          )
        end
      end
      if Ops.get_boolean(@map_dest_addr, "visible", false)
        if Ops.get_boolean(@map_dest_addr, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"destAddr\">" + _("Destination Address") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"destAddr\">" + "*" +
              _("Destination Address") + "</a></b></font>"
          )
        end
      end
      if Ops.get_boolean(@map_dest_edir, "visible", false)
        if Ops.get_boolean(@map_dest_edir, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"desteDir\">" + _("Destination eDirectory") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"desteDir\">" + "*" +
              _("Destination eDirectory") + "</a></b></font>"
          )
        end
      end
      if installtype == 5
        if Ops.get_boolean(@map_dest_lang, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"destLang\">" + _("Language") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"destLang\">" + "*" + _("Language") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_storage, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"storeConfig\">" + _("Storage Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"storeConfig\">" + "*" +
              _("Storage Configuration") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_patterns, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"selectPattern\">" + _("Pattern Selection") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"selectPattern\">" + "*" +
              _("Pattern Selection") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_products, "visible", false)
          if Ops.get_boolean(@map_products, "complete", false)
            summary = Summary.AddHeader(
              summary,
              "<b><a href=\"selectProducts\">" + _("Product Selection") + "</a></b>"
            )
          else
            summary = Summary.AddHeader(
              summary,
              "<font color=\"red\"><b><a href=\"selectProducts\">" + "*" +
                _("Product Selection") + "</a></b></font>"
            )
          end
        end
      elsif installtype != 4
        if Ops.get_boolean(@map_license, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"destLicense\">" + _("License") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"destLicense\">" + "*" +
              _("License") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_dest_addr, "complete1", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"destProt\">" + _("Protocols") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"destProt\">" + "*" +
              _("Protocols") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_dest_lang, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"destLang\">" + _("Language") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"destLang\">" + "*" + _("Language") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_dest_slp, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"destSLP\">" + _("SLP Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"destSLP\">" + "*" +
              _("SLP Configuration") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_dest_dns, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"destDNS\">" + _("DNS Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"destDNS\">" + "*" +
              _("DNS Configuration") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_timezones, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"timeZone\">" + _("Time Zone Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"timeZone\">" + "*" +
              _("Time Zone Configuration") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_timesync, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"timeSync\">" + _("Time Sync Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"timeSync\">" + "*" +
              _("Time Sync Configuration") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_install, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"installSet\">" + _("Install Settings") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"installSet\">" + "*" +
              _("Install Settings") + "</a></b></font>"
          )
        end
        if installtype != 5
          if Ops.get_boolean(@map_storage, "complete", false)
            summary = Summary.AddHeader(
              summary,
              "<b><a href=\"storeConfig\">" + _("Storage Configuration") + "</a></b>"
            )
          else
            summary = Summary.AddHeader(
              summary,
              "<font color=\"red\"><b><a href=\"storeConfig\">" + "*" +
                _("Storage Configuration") + "</a></b></font>"
            )
          end
        end
        if Ops.get_boolean(@map_patterns, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"selectPattern\">" + _("Pattern Selection") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"selectPattern\">" + "*" +
              _("Pattern Selection") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_products, "visible", false)
          if Ops.get_boolean(@map_products, "complete", false)
            summary = Summary.AddHeader(
              summary,
              "<b><a href=\"selectProducts\">" + _("Product Selection") + "</a></b>"
            )
          else
            summary = Summary.AddHeader(
              summary,
              "<font color=\"red\"><b><a href=\"selectProducts\">" + "*" +
                _("Product Selection") + "</a></b></font>"
            )
          end
        end
      else
        if Ops.get_boolean(@map_install, "complete1", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"upgradeSet\">" + _("Upgrade Settings") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"upgradeSet\">" + "*" +
              _("Upgrade Settings") + "</a></b></font>"
          )
        end
        if Ops.get_boolean(@map_products, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"selectProducts\">" + _("Product Selection") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"selectProducts\">" + "*" +
              _("Product Selection") + "</a></b></font>"
          )
        end
      end
      if Ops.get_boolean(@map_nmas, "visible", false)
        if Ops.get_boolean(@map_nmas, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"nmasConfigure\">" + _("NMAS Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"nmasConfigure\">" + "*" +
              _("NMAS Configuration") + "</a></b></font>"
          )
        end
      end
      if Ops.get_boolean(@map_dns_dhcp, "visible", false)
        if Ops.get_boolean(@map_dns_dhcp, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"dnsdhcpConfigure\">" + _("DNS/DHCP Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"dnsdhcpConfigure\">" + "*" +
              _("DNS/DHCP Configuration") + "</a></b></font>"
          )
        end
      end
      if Ops.get_boolean(@map_ifolder, "visible", false)
        if Ops.get_boolean(@map_ifolder, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"ifolderConfigure\">" +
              _("iFolder Server Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"ifolderConfigure\">" + "*" +
              _("iFolder Server Configuration") + "</a></b></font>"
          )
        end
      end
      if Ops.get_boolean(@map_mysql, "visible", false)
        if Ops.get_boolean(@map_mysql, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"mysqlConfigure\">" + _("MySQL Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"mysqlConfigure\">" + "*" +
              _("MySQL Configuration") + "</a></b></font>"
          )
        end
      end
      if Ops.get_boolean(@map_extend, "visible", false)
        if Ops.get_boolean(@map_extend, "complete", false)
          summary = Summary.AddHeader(
            summary,
            "<b><a href=\"extendConfigure\">" +
              _("exteNd Application Server Configuration") + "</a></b>"
          )
        else
          summary = Summary.AddHeader(
            summary,
            "<font color=\"red\"><b><a href=\"extendConfigure\">" + "*" +
              _("exteNd Application Server Configuration") + "</a></b></font>"
          )
        end
      end

      # Configuration summary text for autoyast
      [summary, []]
    end

    def CheckIfComplete
      complete = true

      if Ops.get_boolean(@map_src_addr, "visible", false) &&
          !Ops.get_boolean(@map_src_addr, "complete", false)
        Report.Error(_("Source Address entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_src_edir, "visible", false) &&
          !Ops.get_boolean(@map_src_edir, "complete", false)
        Report.Error(_("Source eDirectory entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_dest_addr, "visible", false) &&
          !Ops.get_boolean(@map_dest_addr, "complete", false)
        Report.Error(_("Destination Address entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_dest_addr, "visible1", false) &&
          !Ops.get_boolean(@map_dest_addr, "complete1", false)
        Report.Error(_("Protocol entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_dest_edir, "visible", false) &&
          !Ops.get_boolean(@map_dest_edir, "complete", false)
        Report.Error(_("Destination eDirectory entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_license, "visible", false) &&
          !Ops.get_boolean(@map_license, "complete", false)
        Report.Error(_("License entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_dest_lang, "visible", false) &&
          !Ops.get_boolean(@map_dest_lang, "complete", false)
        Report.Error(_("Language entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_dest_slp, "visible", false) &&
          !Ops.get_boolean(@map_dest_slp, "complete", false)
        Report.Error(_("SLP Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_dest_dns, "visible", false) &&
          !Ops.get_boolean(@map_dest_dns, "complete", false)
        Report.Error(_("DNS Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_timezones, "visible", false) &&
          !Ops.get_boolean(@map_timezones, "complete", false)
        Report.Error(_("Time Zone Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_timesync, "visible", false) &&
          !Ops.get_boolean(@map_timesync, "complete", false)
        Report.Error(_("Time Sync Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_install, "visible", false) &&
          !Ops.get_boolean(@map_install, "complete", false)
        Report.Error(_("Install Settings entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_install, "visible1", false) &&
          !Ops.get_boolean(@map_install, "complete1", false)
        Report.Error(_("Upgrade Settings entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_storage, "visible", false) &&
          !Ops.get_boolean(@map_storage, "complete", false)
        Report.Error(_("Storage Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_patterns, "visible", false) &&
          !Ops.get_boolean(@map_patterns, "complete", false)
        Report.Error(_("Pattern Selection entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_products, "visible", false) &&
          !Ops.get_boolean(@map_products, "complete", false)
        Report.Error(_("Product Selection entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_nmas, "visible", false) &&
          !Ops.get_boolean(@map_nmas, "complete", false)
        Report.Error(_("NMAS Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_dns_dhcp, "visible", false) &&
          !Ops.get_boolean(@map_dns_dhcp, "complete", false)
        Report.Error(_("DNS/DHCP Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_ifolder, "visible", false) &&
          !Ops.get_boolean(@map_ifolder, "complete", false)
        Report.Error(_("iFolder Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_mysql, "visible", false) &&
          !Ops.get_boolean(@map_mysql, "complete", false)
        Report.Error(_("MySQL Configuration entries are not complete."))
        complete = false
      elsif Ops.get_boolean(@map_extend, "visible", false) &&
          !Ops.get_boolean(@map_extend, "complete", false)
        Report.Error(_("exteNd Configuration entries are not complete."))
        complete = false
      end

      complete
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      Builtins.y2milestone("AutoPackages() started")
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :homesrcpath, :type => "string"
    publish :variable => :homebuildpath, :type => "string"
    publish :variable => :map_branch_type, :type => "map"
    publish :variable => :map_select_type, :type => "map"
    publish :variable => :map_src_addr, :type => "map"
    publish :variable => :map_src_edir, :type => "map"
    publish :variable => :map_dest_addr, :type => "map"
    publish :variable => :map_dest_edir, :type => "map"
    publish :variable => :map_license, :type => "map"
    publish :variable => :map_dest_lang, :type => "map"
    publish :variable => :map_dest_slp, :type => "map"
    publish :variable => :map_dest_dns, :type => "map"
    publish :variable => :map_timezones, :type => "map"
    publish :variable => :map_timesync, :type => "map"
    publish :variable => :map_install, :type => "map"
    publish :variable => :map_storage, :type => "map"
    publish :variable => :map_patterns, :type => "map"
    publish :variable => :map_products, :type => "map"
    publish :variable => :list_products_select, :type => "list <boolean>"
    publish :variable => :list_products_nc, :type => "list <boolean>"
    publish :variable => :map_nmas, :type => "map"
    publish :variable => :list_nmas_select, :type => "list <boolean>"
    publish :variable => :map_dns_dhcp, :type => "map"
    publish :variable => :map_ifolder, :type => "map"
    publish :variable => :map_mysql, :type => "map"
    publish :variable => :map_extend, :type => "map"
    publish :variable => :map_save_rf, :type => "map"
    publish :variable => :list_frame_types, :type => "list <string>"
    publish :variable => :list_timeserver_protocol, :type => "list <string>"
    publish :variable => :list_timeserver_type, :type => "list <string>"
    publish :variable => :list_language, :type => "list <string>"
    publish :variable => :list_language_en, :type => "list <string>"
    publish :variable => :list_language_number, :type => "list"
    publish :variable => :list_language_defaults, :type => "list"
    publish :variable => :list_language_defaultx, :type => "list"
    publish :variable => :list_countrycode, :type => "list <string>"
    publish :variable => :list_countrycode_name, :type => "list <string>"
    publish :variable => :map_codepage, :type => "map"
    publish :variable => :list_codepage, :type => "list <list <string>>"
    publish :variable => :list_keyboardcode, :type => "list <string>"
    publish :variable => :list_keyboardcode_en, :type => "list <string>"
    publish :variable => :list_timezones, :type => "list <string>"
    publish :variable => :list_timezones_name, :type => "list <string>"
    publish :variable => :list_timezones_abr, :type => "list <string>"
    publish :variable => :list_timezones_dst, :type => "list <boolean>"
    publish :variable => :list_patterns, :type => "list <string>"
    publish :variable => :list_patterns_name, :type => "list <string>"
    publish :variable => :list_pattern_products, :type => "list <list <string>>"
    publish :variable => :list_pattern_prod_params, :type => "list <list <integer>>"
    publish :variable => :list_patterns_desc, :type => "list <string>"
    publish :variable => :list_products, :type => "list <string>"
    publish :variable => :list_products_depend, :type => "list <list <integer>>"
    publish :variable => :list_products_dep_name, :type => "list <list <string>>"
    publish :variable => :list_products_params, :type => "list <boolean>"
    publish :variable => :list_products_name, :type => "list <string>"
    publish :variable => :list_products_desc, :type => "list <string>"
    publish :variable => :list_nmas, :type => "list <string>"
    publish :variable => :list_nmas_name, :type => "list <string>"
    publish :variable => :list_nmas_desc, :type => "list <string>"
    publish :function => :getVirtualDiskName, :type => "string (string)"
    publish :function => :FindInList, :type => "integer (list <string>, string)"
    publish :function => :ListToString, :type => "string (list <string>)"
    publish :function => :MergeLists, :type => "list <string> (list <string>, list <string>)"
    publish :function => :ValidateIPMsg, :type => "boolean (string, string)"
    publish :function => :ValidateIP, :type => "boolean (string)"
    publish :function => :ValidatePasswords, :type => "boolean (string, string)"
    publish :function => :ValidateName, :type => "boolean (string)"
    publish :function => :ValidatePathName, :type => "boolean (string, boolean)"
    publish :function => :EndWithSlash, :type => "string (string)"
    publish :function => :ValidateNWPathname, :type => "boolean (string)"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :ReadRF, :type => "string (path)"
    publish :function => :WriteRF, :type => "boolean (path, string)"
    publish :function => :CheckProductPrameter, :type => "boolean (integer)"
    publish :function => :Substitute, :type => "string (string, string, string)"
    publish :function => :FixupString, :type => "string (string)"
    publish :function => :CreateFileName, :type => "string (string, string)"
    publish :function => :GetSeparator, :type => "string (string)"
    publish :function => :CreateIPSString, :type => "string (string, string)"
    publish :function => :CreateNCFString, :type => "string (string, string)"
    publish :function => :InitializeVariables, :type => "boolean ()"
    publish :function => :ImportRFTemplate, :type => "boolean ()"
    publish :function => :ExportRFile, :type => "boolean ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :CheckIfComplete, :type => "boolean ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  Responsefile = ResponsefileClass.new
  Responsefile.main
end
